#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "cg_user.h"


#define NPOLY 2
double *px;
double *py;
double *ux;
double *uy;
double *r;
int ndata;

typedef struct {
	int ndata;
	double *hc;
	double *Ac;
	double *uhc;
	double *uAc;
	double *rho;
} Data;	

void print_help()
{
	printf(" fit the area calibration file with an Oliver Polynomial\n");
	printf(" -h    show this help\n");
	printf(" -f    input file \n");
}

void	least_squares_fit(double *x, double *y, int n, double *a, double *b)
{
	double xavg, yavg, sxx, sxy;
        int i;

	xavg = 0;
	yavg = 0;

        for (i = 0; i < n; i++){
		xavg += x[i];
		yavg += y[i];
	}
	xavg/= n;
	yavg/= n;

        sxx = 0;
        sxy = 0;
        for (i = 0; i < n; i++){
                sxy += (x[i] - xavg) * (y[i] - yavg);
                sxx += (x[i] - xavg) * (x[i] - xavg);
        }

        sxy /= n;
        sxx /= n;

        *a = sxy/sxx;
        *b = yavg - (*a)*xavg;

}

int read_area_data( char *fnm, Data *data)
{
	FILE *file;
	int ndata = 0;
	int i;
	size_t size; 
	char *line = NULL;
	double x1, x2, x3, x4, x5, x6;

	file = fopen(fnm, "r");

	if (!file)
		return -1;
	
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL)
				ndata ++;
	}
	printf("ndata %d \n", ndata);

	data->ndata = ndata;
	data->hc = (double *)malloc(ndata *sizeof(double));
	data->Ac = (double *)malloc(ndata *sizeof(double));
	data->uhc = (double *)malloc(ndata *sizeof(double));
	data->uAc = (double *)malloc(ndata *sizeof(double));
	data->rho = (double *)malloc(ndata *sizeof(double));

	rewind(file);

	ndata = 0; 
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL){
			i = sscanf(line, "%lf %lf %lf %lf %lf %lf", &x1, &x2, &x3, &x4, &x5, &x6);
			if (i == 6 ){
				data->hc[ndata] = x1;
				data->Ac[ndata] = x2;//1e6;
				data->uhc[ndata] = x3;
				data->uAc[ndata] = x4;//1e6;
				data->rho[ndata] = x6;
				ndata++;
			}
		}
	}
	printf("ndata %d data->ndata %d \n", ndata, data->ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return ndata;
	}

	return ndata;
}

double chi2val(double *p, INT n)
{
	double sum = 0;
	double *pc, *ystar;
	int i;

	pc = p;
	ystar = (double *) malloc(ndata*sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] =  pc[0]*px[i] + pc[1] ;
	}
	/*
	printf("\n star data \n");
	for (i = 0; i< ndata; i ++){
		printf(" %g %g \n", px[i], ystar[i]);
	}
	printf("\n star data \n");
	*/
	

	for (i=0; i < ndata ; i++)
	{
		/* (x - X)^2/ux^2 + 2 rho (x-X)(y-Y)/ux/uy + (y-Y)^2/uy^2 */
		sum += (py[i] - ystar[i])*(py[i] - ystar[i]);
	}
	printf("sum %g \n", sum);

	free(ystar);
	return sum;
}

void chi2grad(double *g, double *p, INT n)
{
	double *ystar, **dystar;
	double *pc;
	int i, j;
	double w;

	pc = p;

	for (i = 0 ;i < n; i++){
		g[i] = 0;
	}

	ystar = (double *) malloc(ndata*sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = pc[0]*px[i] + pc[1] ;
	}

	/* dystar/dp */
	for (i = 0; i< ndata; i ++){
		dystar[i][0] += px[i];
		dystar[i][1] += 1;
	}

	for (j=0; j< NPOLY; j++)
	{
		for (i=0; i < ndata; i++){
			w =  -2*(py[i]-ystar[i]); /* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	free(ystar);
	for (i=0; i < ndata; i++)
		free(dystar[i]);
	free(dystar);
	return ;
}

double chi2valgrad(double *g, double *p, INT n)
{
	double sum = 0;
	double *ystar, **dystar;
	double *pc;
	int i,j;
	double w;

	pc = p;
	for (i = 0 ;i < n; i++){
		g[i] = 0;
	}


	ystar = (double *) malloc(ndata*sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = pc[0]*px[i] + pc[1]; 
	}
	for (i = 0; i< ndata; i ++){
		//		printf("x[%d] %g xstar[%d] %g y[%d] %g ystar[%d] %g r[%d] %g \n", i,px[i], i, xstar[i] , i,py[i], i, ystar[i], i, r[i]);
	}


	/* dystar /dp */
	for (i = 0; i< ndata; i ++){
		dystar[i][0] += px[i];
		dystar[i][1] += 1;
	}

	for (j=0; j< NPOLY; j++)
	{
		for (i=0; i < ndata; i++){
			w =  -2*(py[i]-ystar[i]) ; /* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	sum = 0;
	for (i=0; i < ndata; i++)
	{
		sum += (py[i] - ystar[i])*(py[i] - ystar[i]);
	}
	free(ystar);
	for (i=0; i < ndata; i++){
		free(dystar[i]);
	}
	free(dystar);
	return sum;
}


int main (int argc, char *argv[])
{

	int c, i;
	char *fnm = NULL;
	Data data;
	int n, npar;
	double *par;
	double *dpar;
	cg_stats Stats ;
	cg_parameter Parm ;
	double ain, bin;
	double a, b;

	if (argc == 1 ){
		print_help();
		exit(0);
	}

	while (( c = getopt (argc , argv, "f:ha:b:")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'a':
				ain=strtod(optarg, NULL);
				break;
			case 'b':
				bin =strtod(optarg, NULL);
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}

	n = read_area_data( fnm, &data);
	if (n == -1){
		printf(" Failed to load data from file %s\n", fnm);
		exit(1);
	}
	px = data.hc;
	py = data.Ac;
	ux = data.uhc;
	uy = data.uAc;
	r = data.rho;
	ndata = data.ndata;

	printf("read data \n");
	for (i = 0; i< ndata; i++)
		printf("%g %g \n", px[i], py[i]);
	printf("\n\n");
	/*	for (i=0; i< data.ndata; i++)
		r[i] = 0;
		*/

	npar =  NPOLY;
	par = (double *)malloc(npar*sizeof(double));
	dpar = (double *)malloc(npar*sizeof(double));
	/*initial estimate */
	//	for (i = 0; i< NPOLY; i++)
	//		par[i] = 0;
	par[0] = ain;
	par[1] = bin;

	printf("initial conditions %g %g \n", par[0], par[1]);

	cg_default(&Parm);
	Parm.PrintLevel = 1;
	Parm.PrintFinal = TRUE;

	printf(" chi2val %g \n", chi2val(par, npar));
	printf(" chi2valgrad %g \n", chi2valgrad(dpar, par, npar));
	/*	chi2grad(dpar, par,npar); */

	for (i = 0; i< npar; i++)
		printf("dpar[%d] %g \n", i,dpar[i]);
	/*		*/

	cg_descent (par, npar, &Stats, &Parm, 1.e-8, chi2val, chi2grad, chi2valgrad, NULL) ;
	//cg_descent (par, npar, NULL, NULL, 1.e-8, chi2val, chi2grad, NULL, NULL) ;
	//	cg_descent (par, npar, NULL, NULL, 1.e-8, chi2val, chi2grad, chi2valgrad, NULL) ;
	printf(" Results \n");
	for (i = 0; i< NPOLY; i++)
		printf(" %g ", par[i]);
	printf("\n");
	printf("in %ld steps\n", Stats.iter);

	least_squares_fit(data.hc, data.Ac, data.ndata, &a, &b);

	printf("analytical solution: a = %g, b = %g \n", a, b);
	return 0;
}
