#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "cg_user.h"


double *px;
double *py;
double *ux;
double *uy;
double *r;
int npoly = 5;

typedef struct {
	int ndata;
	double *hc;
	double *Ac;
	double *uhc;
	double *uAc;
	double *rho;
} Data;	

void print_help()
{
	printf(" fit the area calibration file with an Oliver Pharr Polynomial\n");
	printf(" -h    show this help\n");
	printf(" -f    input file \n");
	printf(" -p    degree of Oliver Pharr polynomial \n");
}

int read_area_data( char *fnm, Data *data)
{
	FILE *file;
	int ndata = 0;
	int i;
	size_t size; 
	char *line = NULL;
	double x1, x2, x3, x4, x5, x6;

	file = fopen(fnm, "r");

	if (!file)
		return -1;
	
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL)
				ndata ++;
	}
	printf("ndata %d \n", ndata);

	data->ndata = ndata;
	data->hc = (double *)malloc(ndata *sizeof(double));
	data->Ac = (double *)malloc(ndata *sizeof(double));
	data->uhc = (double *)malloc(ndata *sizeof(double));
	data->uAc = (double *)malloc(ndata *sizeof(double));
	data->rho = (double *)malloc(ndata *sizeof(double));

	rewind(file);

	ndata = 0; 
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL){
			i = sscanf(line, "%lf %lf %lf %lf %lf %lf", &x1, &x2, &x3, &x4, &x5, &x6);
			if (i == 6 ){
				data->hc[ndata] = x1;
				data->Ac[ndata] = x2*1e6;
				data->uhc[ndata] = x3;
				data->uAc[ndata] = x4*1e6;
				data->rho[ndata] = x6;
				ndata++;
			}
		}
	}
	printf("ndata %d data->ndata %d \n", ndata, data->ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return ndata;
	}

	return ndata;
}

double chi2val(double *p, INT n)
{
	double sum = 0;
	double *xstar, *ystar;
	double *pc;
	double ndata;
	int i,j;

	pc = p;
	xstar = p+npoly;
	ndata = n - npoly;
	ystar = (double *) calloc(ndata, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = 24.5 *xstar[i]*xstar[i];
		for (j = 0; j < npoly; j++){
			ystar[i] += pc[j]*pow(xstar[i], pow(2,-j));
		}
	}
/*	printf("\n star data \n");
	for (i = 0; i< ndata; i ++){
		printf(" %g %g \n", xstar[i], ystar[i]);
	}
	printf("\n star data \n");
	*/
	

	for (i=0; i < n - npoly; i++)
	{
		/* (x - X)^2/ux^2 + 2 rho (x-X)(y-Y)/ux/uy + (y-Y)^2/uy^2 */
		sum += (px[i] - xstar[i])*(px[i] - xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i]);
		sum += (py[i] - ystar[i])*(py[i] - ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]);
		sum -= 2*r[i]*(px[i] - xstar[i])*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]);
	}

	free(ystar);
	return sum;
}

void chi2grad(double *g, double *p, INT n)
{
	double *xstar, *ystar, **dystar;
	double *pc;
	double ndata;
	int i, j;
	double w;

	pc = p;
	xstar = p+npoly;
	ndata = n - npoly;


	for (i = 0 ;i < n; i++){
		g[i] = 0;
	}

	ystar = (double *) calloc(ndata,sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = 24.5*xstar[i]*xstar[i];
		for (j = 0; j < npoly; j++){
			ystar[i] += pc[j]*pow(xstar[i], pow(2,-j));
		}
	}

	/*dY/dp */
	for (i = 0; i< ndata; i ++){ 
		for (j = 0 ; j< npoly; j++)
			dystar[i][j] += pow(xstar[i], pow(2, -j));
	}
	/* dY/dX */
	for (i = 0; i< ndata; i ++){
		dystar[i][i+npoly] = 24.5*2*xstar[i];
		for (j = 0 ; j< npoly; j++){
			dystar[i][i+npoly] += pc[j]*pow(2, -j) * pow(xstar[i], pow(2, -j) -1);
		}
	}


	for (i=0; i < ndata; i++)
	{
		g[i+npoly] = -2*(px[i]-xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i])
			+2*r[i]*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dxstar */

		w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i])
			+2*r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dystar */
		g[i+npoly] += w*dystar[i][i+npoly]; /* dchi2/dysta * dystar/dxstar */

	}

	for (j=0; j< npoly; j++)
	{
		for (i=0; i < ndata; i++){
			w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i])
				+2*r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	free(ystar);
	for (i=0; i < ndata; i++)
		free(dystar[i]);
	free(dystar);
	return ;
}

double chi2valgrad(double *g, double *p, INT n)
{
	double sum = 0;
	double *xstar, *ystar, **dystar;
	double *pc;
	double ndata;
	int i,j;
	double w;

	pc = p;
	xstar = p+npoly;
	ndata = n - npoly;

	for (i=0; i< n; i++)
		g[i] = 0;

//	printf(" ---------------------------------------------\n");
	ystar = (double *) calloc(ndata,sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = 24.5*xstar[i]*xstar[i];
		for (j = 0; j < npoly; j++){
			ystar[i] += pc[j]*pow(xstar[i], pow(2,-j));
		}
	}
	for (i = 0; i< ndata; i ++){
//		printf("x[%d] %g xstar[%d] %g y[%d] %g ystar[%d] %g \n", i,px[i], i, xstar[i] , i,py[i], i, ystar[i]);
	}
	

	/*dY/dp */
	for (i = 0; i< ndata; i ++){
		for (j = 0 ; j< npoly; j++)
			dystar[i][j] += pow(xstar[i], pow(2, -j));
	}
	/* dY/dX */
	for (i = 0; i< ndata; i ++){
		dystar[i][i+npoly] = 24.5*2*xstar[i];
		for (j = 0 ; j< npoly; j++){
			dystar[i][i+npoly] += pc[j]*pow(2, -j) * pow(xstar[i], pow(2, -j) -1);
		}
	}

	for (i=0; i< ndata; i++){
//		printf("dystar[%d] ", i);
		for (j=0; j< n; j++){
//			printf(" %d %g ", j, dystar[i][j]); 
		}
//		printf("\n");
		
	}


	for (i=0; i < ndata; i++)
	{
//		printf(" x[%d] %g xstar[%d] %g ux[%d] %g  uy[%d] %g r[%d] %g \n", i, px[i], i, xstar[i], i, ux[i], i, uy[i], i, r[i]);
		g[i+npoly] = -2*(px[i]-xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i])
			+2*r[i]*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dxstar */
//		printf("g[%d] %g \n", i+npoly, g[i+npoly]);
//		printf("aux1 %g y-ystar %g 1-r^2 %g  \n", 2*r[i]*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]), py[i]-ystar[i], 1-r[i]*r[i]);

		w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i])
			+2*r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dystar */
		g[i+npoly] += w*dystar[i][i+npoly]; /* dchi2/dysta * dystar/dxstar */

//		printf("g[%d] %g w %g dystar[%d][%d] %g w*dystar %g  \n", i+npoly, g[i+npoly], w, i, i+npoly, dystar[i][i+npoly], w*dystar[i][i+npoly]);
//		printf("aux2  %g \n",-2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]));

//		printf("aux3 %g \n", r[i]/ux[i]*uy[i]);
 
	}

	for (j=0; j< npoly; j++)
	{
		for (i=0; i < ndata; i++){
			w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i])
				+2*r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		//	printf(" w %g dystar[%d][%d] %g \n", w, i,j, dystar[i][j]);
		}
		//printf(" %g ", g[j]);
	}
/*	for (j = npoly; j< n; j++)
		printf(" %g ", g[j]);
	printf("\n");
	*/

	sum = 0;
	for (i=0; i < n - npoly; i++)
	{
		/* (x - X)^2/ux^2 + 2 rho (x-X)(y-Y)/ux/uy + (y-Y)^2/uy^2 */
		sum += (px[i] - xstar[i])*(px[i] - xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i]);
		sum += (py[i] - ystar[i])*(py[i] - ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]);
		sum -= 2*r[i]*(px[i] - xstar[i])*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]);
	}
	free(ystar);
	for (i=0; i < ndata; i++){
		free(dystar[i]);
	}
	free(dystar);
//	printf(" ---------------------------------------------\n");
	return sum;
}


int main (int argc, char *argv[])
{

	int c, i;
	char *fnm = NULL;
	Data data;
	int n, npar;
	double *par;
	double *dpar;
	cg_stats Stats ;
	cg_parameter Parm ;

	if (argc == 1 ){
		print_help();
		exit(0);
	}

	while (( c = getopt (argc , argv, "f:hp:")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'p':
				npoly = atoi(optarg);
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}

	if (npoly > 10){
		printf(" WARNING: polynomial degree more than 10 not defined. \n");
		printf(" If you really want such a high degree of the polynom, modify the code and recompile. \n");
		exit(1);
	}
	n = read_area_data( fnm, &data);
	if (n == -1){
		printf(" Failed to load data from file %s\n", fnm);
		exit(1);
	}
	px = data.hc;
	py = data.Ac;
	ux = data.uhc;
	uy = data.uAc;
	r = data.rho;

	
	npar = data.ndata + npoly;
	par = (double *)malloc(npar*sizeof(double));
	dpar = (double *)malloc(npar*sizeof(double));

	/* initialize fitting structure */
	cg_default(&Parm);
	Parm.PrintLevel = 0;
	Parm.PrintFinal = TRUE;

	/*initial estimate */
	for (i = 0; i< npoly; i++)
		par[i] = 0;

	/*fix first coefficient */
	for (i = 0; i< data.ndata; i++)
		par[i + npoly] = data.hc[i];

	printf(" chi2val %g \n", chi2val(par, npar));
	printf(" chi2valgrad %g \n", chi2valgrad(dpar, par, npar));

	for (i = 0; i< npar; i++)
		printf("dpar[%d] %g \n", i,dpar[i]);

	cg_descent (par, npar, &Stats, &Parm, 1.e-8, chi2val, chi2grad, chi2valgrad, NULL) ;
//	cg_descent (par, npar, &Stats, &Parm, 1.e-8, chi2val, chi2grad, NULL, NULL) ;
	for (i = 0; i< npoly; i++)
		printf(" %g ", par[i]);
	printf("\n");
	return 0;
}
