#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "../mc-utils.h"


void print_help()
{
}

double calc_A(double x, double *coef, int np)
{
	int i;
	double A = 0;


	A = 24.5*x*x;
	for ( i = 0; i< np; i++){
		A+= coef[i]*pow(x, pow(2, -i ));
	}	
	return A;
}

double calc_uA(double x, double *coef, double *unc_coef , double *cov, int np)
{
	int i,j, k;
	double uA = 0;
	double *dAdp; 

	dAdp = (double *)malloc(np*sizeof(double));

	for ( i = 0; i< np; i++){
		dAdp[i] = pow(x, pow(2, -i));
	}

//	printf(" uA init %g \n", uA);
	for ( i = 0; i< np; i++){
		uA+= dAdp[i]*dAdp[i]*unc_coef[i]*unc_coef[i];
//		printf("i %d %g  \n", i, uA);
	}
	k =0;
	for ( i = 0; i< np; i++){
		for ( j = i+1; j< np; j++){
			uA += 2*cov[k++] *dAdp[i]*dAdp[j]*unc_coef[i]*unc_coef[j];	
//		printf("i %d  j %d %g %g \n", i, j,uA, cov[k-1] *unc_coef[i]*unc_coef[j]);
		}
	}
	return sqrt(uA);
}

int assemble_amat(double **covmat, int np, double *amat)
{
	double *p;
	int i,j;

        p = amat;


	for (i=0;i<np;i++){
		for (j=0;j<=i;j++){
//			printf("assign %d %d %g %g \n", i,j, *p, covmat[i][j]);
			*p = covmat[i][j];
			p++;
		}
	}
	p = amat;
	printf("amat \n");
	for (i=0;i< np*(np+1)/2;i++, p++){
		printf("%g ", *p);
	}
	printf("\n");
	printf("\n");


	if (! gwy_math_choleski_decompose(np, amat)){
                fprintf(stderr, "unable to decompose covariance matrix\n");
		return 0;
        }
	p = amat;
	printf("decomposed amat \n");
	for (i=0;i< np*(np+1)/2;i++, p++){
		printf("%g ", *p);
	}
	printf("\n");
	printf("\n");


	return 1;

}

void assemble_covariance(double *cov, double *unc_coef, int np, double **covmat){
	int i,j, k; 

	k =0;
	for ( i = 0; i< np; i++){
		covmat[i][i] = unc_coef[i]*unc_coef[i];
	}
	for ( i = 0; i< np; i++){
		for ( j = i+1; j< np; j++){
			covmat[i][j] = cov[k++]*unc_coef[i]*unc_coef[j];
			covmat[j][i] = covmat[i][j];
		}
	}
}

int read_data(char *fnm ,int np, double *beta, double *unc_beta, double *cov)
{
	FILE *file;
	file = fopen(fnm, "r");
	size_t size; 
	char *line = NULL;
	//char format[200];
	//char junk[100];
	int i;
	char *token;

	if (!file)
		return 0;

	/* beta */
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, " beta ") )
			break;
	}

	token = strtok(line, " ");  //beta

	i = 0;
	token = strtok(NULL, " ");  //beta[0]
	while (token){
		beta[i] = strtod(token, NULL);					
		token = strtok(NULL, " ");  //beta[i]
		i++;
	}

	if (i <= np) {
		printf(" Not enough data in file. \n");
		return  0;
	}

	/* stdev beta */
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, " stdev_beta ") )
			break;
	}

	token = strtok(line, " ");  //stdevbeta

	i = 0;
	token = strtok(NULL, " ");  //stdbeta[0]
	while (token){
		unc_beta[i] = strtod(token, NULL);					
		token = strtok(NULL, " ");  //stdbeta[i]
		i++;
	}

	if (i <= np) {
		printf(" Not enough data in file. \n");
		return  0;
	}



	/* correlation beta */
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "correlation") )
			break;
	}

	token = strtok(line, " ");  //correlation
	token = strtok(NULL, " ");  //beta

	i = 0;
	token = strtok(NULL, " ");  //cov[0]
	while (token){
		cov[i] = strtod(token, NULL);					
		token = strtok(NULL, " ");  //cov[i]
		i++;
	}
	if (i <= np*(np-1)/2) {
		printf(" Not enough data in file. \n");
		return  0;
	}
	return 1;

}

void	mc_rand_coeff_noise( double *amat, double *noise, int np, GRand *rng)
{

	int i, j;
	double *tt, *p;

	for (i = 0; i< np; i++)
		noise[i] = 0;

	tt = (double *) malloc(np*sizeof(double)); 

	for (i = 0; i< np; i++)
	{
		tt[i] = gaussian_random_number(rng);
	}

	p = amat;
	for (i =0; i< np; i++){
		for (j =0; j<= i; j++, p++){
			noise[i] += (*p)*tt[j];
		}
	}
}

int main (int argc, char *argv[])
{
	int c, i, j;
	int np = 0;
	char *fnm = NULL;
	int isok;
	double *beta, *ubeta, *cov, **covmat;

	int NMC = 1;
 	double *amat, *noise;
	double *randbeta; 
	double **mcA, *avgA, *stdA;
	GRand *rng;

	double to = 250;
	int num = 200;
	double dx, A, uA;


	if (argc == 1 ){
		print_help();
		exit(0);
	}

	while (( c = getopt (argc , argv, "f:hp:m:t:")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'p':
				np = atoi(optarg);
				break;
			case 'm':
				NMC = atoi(optarg);
				break;
			case 't':
				to = strtod(optarg, NULL);
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}

	if (np <= 0 ){
		printf("Number of polynomial coefficients must be positive. \n");
		exit(1);
	}

	beta = (double *)malloc( np * sizeof(double));
	ubeta = (double *)malloc( np * sizeof(double));
	cov = (double *)malloc( np*(np-1)/2 * sizeof(double));
	covmat = (double **)malloc( np * sizeof(double *));
	for (i =0; i< np; i++)
		covmat[i] = (double *)malloc( np * sizeof(double ));

	isok = read_data(fnm, np, beta, ubeta, cov);
	if (!isok){
		exit(2);
	}

	printf("beta: ");
	for (i =0; i< np; i++)
		printf(" %g ", beta[i]);
	printf("\n");

	printf("stdev beta: ");
	for (i =0; i< np; i++)
		printf(" %g ", ubeta[i]);
	printf("\n");


	assemble_covariance(cov, ubeta, np, covmat);
	printf("covariance: \n");
	for (i = 0; i< np; i++){
		for (j = 0; j< np; j++){
			printf("%g ", covmat[i][j]);
		}
		printf("\n");
	}
	printf("cov: \n");
	for (i = 0; i< np*(np-1)/2; i++){
			printf("%g ", cov[i]);
	}

	printf("\n");

	A = calc_A(100, beta, np);
	uA = calc_uA(100, beta, ubeta, cov, np);
	printf(" 100  %g %g \n",  A, uA);

	dx = to/ (num-1);
	for ( i =1 ;i< num ;i ++){
		A = calc_A(dx*i, beta, np);
		uA = calc_uA(dx*i, beta, ubeta, cov, np);
	//	printf(" %g  %g %g \n", dx*i, A, uA);
	}
	printf("\n");
	printf("\n");

	amat = (double *)malloc(np*(np+1)/2*sizeof(double));
	if (!assemble_amat(covmat, np, amat))
		exit(0);

/*
printf("amat: \n");
for (i = 0; i< np*(np+1)/2; i++)
printf(" %g ", amat[i]);
printf("\n");
*/

	noise  = (double *)malloc(np*sizeof(double));
	randbeta  = (double *)malloc(np*sizeof(double));
	mcA  = (double **)malloc(NMC*sizeof(double *));
	for (j = 0; j< NMC ;j++){
		mcA[j]  = (double *)malloc(num*sizeof(double));
	}
	avgA  = (double *)malloc(num*sizeof(double));
	stdA  = (double *)malloc(num*sizeof(double));

	rng= g_rand_new();

FILE *fileB, *fileC;
fileB=fopen("coeff.mc", "w");
fileC=fopen("area.mc100", "w");
	for (j = 0; j< NMC ;j++){
		mc_rand_coeff_noise( amat, noise, np, rng);
//		printf("beta noise \n");
		for ( i =0 ;i< np ;i ++){
			randbeta[i] = beta[i] + noise[i];
//			printf("%g ", noise[i]);
			fprintf(fileB, "%g, ", noise[i]);
		}
//		printf("\n");
		fprintf(fileB,"\n");
		for ( i =0 ;i< num ;i ++){
			mcA[j][i] = calc_A(dx*(i+1), randbeta, np);
		}
		A = calc_A(100, beta, np);
		uA = calc_uA(100, beta, ubeta, cov, np);
		fprintf(fileC, "%g, %g, %g \n", calc_A(100, randbeta, np), A, uA);
	}
	fclose(fileB);
	fclose(fileC);
	for ( i =0 ;i< num ;i ++){
		avgA[i] = 0;
		stdA[i] = 0;

		for (j = 0; j< NMC ;j++){
			avgA[i] += mcA[j][i];
			stdA[i] += mcA[j][i]*mcA[j][i];
		}
//		printf(" %g %g \n", avgA[i], stdA[i]);
		avgA[i] /= NMC;
		stdA[i] /= NMC;
//		printf(" %g %g \n", avgA[i], stdA[i]);
		stdA[i] -= avgA[i]*avgA[i];
//		printf(" %g %g \n", avgA[i], stdA[i]);
		stdA[i] = sqrt(stdA[i]);
		A = calc_A(dx*(i+1), beta, np);
		uA = calc_uA(dx*(i+1), beta, ubeta, cov, np);
		printf(" %g %g %g %g %g \n",  dx*(i+1), avgA[i], stdA[i], A ,uA);
	}

/*
printf("-----------------\n");
	
i=0;
	
		avgA[i] = 0;
		stdA[i] = 0;

		for (j = 0; j< NMC ;j++){

			avgA[i] += mcA[j][i];
			stdA[i] += mcA[j][i]*mcA[j][i];
	printf("%g \n", mcA[j][i]);
		}
		avgA[i] /= NMC;
		stdA[i] /= NMC;
		stdA[i] -= avgA[i]*avgA[i];
//		stdA[i] = sqrt(stdA[i]);
		printf("%d  %g %g %g \n", i, dx*(i+1), avgA[i], stdA[i]);

*/

/*
	printf(" analytic A(100) %g  ", calc_A(100, beta, np));
	double avgAA = 0;
	double stdAA = 0;
	double *mcAA;
	mcAA = (double *) malloc(NMC*sizeof(double));
	for (j = 0; j< NMC; j++){
		mc_rand_coeff_noise( amat, noise, np, rng);
		for ( i =0 ;i< np ;i ++){
			randbeta[i] = beta[i] + noise[i];
		}
		mcAA[j] = calc_A(100, randbeta, np);
		avgAA += mcAA[j];
		stdAA += mcAA[j]*mcAA[j];
	}
	avgAA /= NMC;
	stdAA /= NMC;
	stdAA -= avgAA*avgAA;
	stdAA = sqrt(stdAA);

	printf(" MC A(00) %g %g ", avgAA, stdAA);
	FILE *fielA;
	fielA = fopen ("smz.mc", "w");
	for (j = 0; j< NMC; j++){
		fprintf(fielA, "%g\n", mcAA[j]);
	}
	fclose(fielA);
*/

	return 0;
}

