#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "cg_user.h"


#define NPOLY 2
double *px;
double *py;
double *ux;
double *uy;
double *r;

typedef struct {
	int ndata;
	double *hc;
	double *Ac;
	double *uhc;
	double *uAc;
	double *rho;
} Data;	

void print_help()
{
	printf(" fit line deming fit\n");
	printf(" -h    show this help\n");
	printf(" -f    input file \n");
	printf(" -a    initial estimate slope \n");
	printf(" -b    initial estimate offset \n");
}

void	corr_deming_fit(double *x, double *y, int n, double ux, double uy, double rho, double *a, double *b, double *xstar)
{
	double xavg, yavg, sxx, syy, sxy;
        int i;
        double delta, delta2;


	delta = uy/ux;
	delta2 = delta*delta;

	xavg = 0;
	yavg = 0;

        for (i = 0; i < n; i++){
		xavg += x[i];
		yavg += y[i];
	}
	xavg/= n;
	yavg/= n;


        sxx = 0;
        syy = 0;
        sxy = 0;
        for (i = 0; i < n; i++){
                sxx += (x[i] - xavg) * (x[i] - xavg);
                sxy += (x[i] - xavg) * (y[i] - yavg);
                syy += (y[i] - yavg) * (y[i] - yavg);
        }

        sxy /= n;
        sxx /= n;
        syy /= n;



	printf("delta %g ,rho %g , sxx %g \n", delta, rho, sxx);
	printf( "syy %g rho*delta*sxx %g \n", syy, rho*delta*sxx);
	printf("  %g %g %g \n", delta2*sxy*sxy,rho*delta*sxy*(syy-delta2*sxx),- sxx*syy*delta*rho*rho);
        *a = (syy - delta2*sxx + sqrt((syy-delta2*sxx)*(syy-delta2*sxx) + 4*(delta2*sxy*sxy-rho*delta*sxy*(syy+delta2*sxx) + sxx*syy*delta2*rho*rho))) / (2*(sxy-sxx*delta*rho));
        *b = yavg - (*a)*xavg;

	for (i = 0; i < n; i++){
		xstar[i] = x[i] + (*a - delta *rho)/(delta + (*a)*(*a)- 2*delta*rho*(*a))*(y[i] - (*a)*x[i] -(*b));
	}

}
int read_area_data( char *fnm, Data *data)
{
	FILE *file;
	int ndata = 0;
	int i;
	size_t size; 
	char *line = NULL;
	double x1, x2, x3, x4, x5, x6;

	file = fopen(fnm, "r");

	if (!file)
		return -1;
	
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL)
				ndata ++;
	}
	printf("ndata %d \n", ndata);

	data->ndata = ndata;
	data->hc = (double *)malloc(ndata *sizeof(double));
	data->Ac = (double *)malloc(ndata *sizeof(double));
	data->uhc = (double *)malloc(ndata *sizeof(double));
	data->uAc = (double *)malloc(ndata *sizeof(double));
	data->rho = (double *)malloc(ndata *sizeof(double));

	rewind(file);

	ndata = 0; 
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL){
			i = sscanf(line, "%lf %lf %lf %lf %lf %lf", &x1, &x2, &x3, &x4, &x5, &x6);
			if (i == 6 ){
				data->hc[ndata] = x1;
				data->Ac[ndata] = x2;//1e6;
				data->uhc[ndata] = x3;
				data->uAc[ndata] = x4;//1e6;
				data->rho[ndata] = x6;
				ndata++;
			}
		}
	}
	printf("ndata %d data->ndata %d \n", ndata, data->ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return ndata;
	}

	return ndata;
}

double chi2val(double *p, INT n)
{
	double sum = 0;
	double *xstar, *ystar;
	double *pc;
	double ndata;
	int i;

	pc = p;
	xstar = p+NPOLY;
	ndata = n - NPOLY;
	ystar = (double *) malloc(ndata*sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] =  pc[0]*xstar[i] + pc[1] ;
	}
/*	printf("\n star data \n");
	for (i = 0; i< ndata; i ++){
		printf(" %g %g \n", xstar[i], ystar[i]);
	}
	printf("\n star data \n");
	*/
	

	for (i=0; i < ndata; i++)
	{
		/* (x - X)^2/ux^2/(1-r^2)  + (y-Y)^2/uy^2/(1-r^2) -2 r (x-X) (y-Y)/ ux/uy/(1-r^2) */
		sum += (px[i] - xstar[i])*(px[i] - xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i]);
		sum += (py[i] - ystar[i])*(py[i] - ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]);
		sum -= 2*r[i]*(px[i] - xstar[i])*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]);
	}

	free(ystar);
	return sum;
}

void chi2grad(double *g, double *p, INT n)
{
	double *xstar, *ystar, **dystar;
	double *pc;
	double ndata;
	int i, j;
	double w;

	pc = p;
	xstar = p+NPOLY;
	ndata = n - NPOLY;

	for (i = 0 ;i < n; i++){
		g[i] = 0;
	}

	ystar = (double *) malloc(ndata*sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = pc[0]*xstar[i] + pc[1] ;
	}

	/* dystar/dp */
	for (i = 0; i< ndata; i ++){
		dystar[i][0] += xstar[i];
		dystar[i][1] += 1;
	}
	/*partial ystar / partial xstar */
	for (i = 0; i< ndata; i ++){
		dystar[i][i+NPOLY] = pc[0];	
	}


	for (i=0; i < ndata; i++)
	{
		/*  partial chi2/partial X = -2 (x-X) /ux^2 /(1-r^2) + 2 r (y-Y)/ux/uy/(1-r^2) */
		g[i+NPOLY] = -2*(px[i]-xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i])
			+ 2*r[i]*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dxstar */

		/* dchi2 /dY = -2 (y-Y)/uy^2/(1-r^2) + 2 r (x-X)/ux/uy/(1-r^2) */
		w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i])
			+2*r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); 	/* dchi2/dystar */
		/* dchi2 /dX = partical chi2/ partial X + (partial chi2/partial Y) (partial Y/partial X)  */
		g[i+NPOLY] += w*dystar[i][i+NPOLY]; /* dchi2/dystar * dystar/dxstar */

	}

	for (j=0; j< NPOLY; j++)
	{
		for (i=0; i < ndata; i++){
			/* -2 (y- Y) /uy^2 /(1-r^2) + 2 r (x- X)/ux/uy/(1-r^2) */
			w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]) 
				+2 *r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); 	/* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	free(ystar);
	for (i=0; i < ndata; i++)
		free(dystar[i]);
	free(dystar);
	return ;
}

double chi2valgrad(double *g, double *p, INT n)
{
	double sum = 0;
	double *xstar, *ystar, **dystar;
	double *pc;
	double ndata;
	int i,j;
	double w;

	pc = p;
	xstar = p+NPOLY;
	ndata = n - NPOLY;

	for (i = 0; i< n; i++){
		g[i] = 0;
	}

//	printf(" ---------------------------------------------\n");
	ystar = (double *) malloc(ndata*sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = pc[0]*xstar[i] + pc[1]; 
	}
	for (i = 0; i< ndata; i ++){
//		printf("x[%d] %g xstar[%d] %g y[%d] %g ystar[%d] %g r[%d] %g \n", i,px[i], i, xstar[i] , i,py[i], i, ystar[i], i, r[i]);
	}
	

	/* dystar /dp */
	for (i = 0; i< ndata; i ++){
		dystar[i][0] += xstar[i];
		dystar[i][1] += 1;
	}
	/* partial ystar / partial xstar */
	for (i = 0; i< ndata; i ++){
		dystar[i][i+NPOLY] = pc[0];
	}

	for (i=0; i< ndata; i++){
//		printf("dystar[%d] ", i);
		for (j=0; j< n; j++){
//			printf(" %d %g ", j, dystar[i][j]); 
		}
//		printf("\n");
		
	}


	for (i=0; i < ndata; i++)
	{
		/*  partial chi2/partial X = -2 (x-X) /ux^2 /(1-r^2) + 2 r (y-Y)/ux/uy/(1-r^2) */
		g[i+NPOLY] = -2*(px[i]-xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i])
			+ 2*r[i]*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); /* dchi2/dxstar */

		/* dchi2 /dY = -2 (y-Y)/uy^2/(1-r^2) + 2 r (x-X)/ux/uy/(1-r^2) */
		w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i])
			+2*r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); 	/* dchi2/dystar */
		/* dchi2 /dX = partical chi2/ partial X + (partial chi2/partial Y) (partial Y/partial X)  */
		g[i+NPOLY] += w*dystar[i][i+NPOLY]; /* dchi2/dystar * dystar/dxstar */
	}

	for (j=0; j< NPOLY; j++)
	{
		for (i=0; i < ndata; i++){
			/* -2 (y- Y) /uy^2 /(1-r^2) + 2 r (x- X)/ux/uy/(1-r^2) */
			w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]) 
				+2 *r[i]*(px[i] - xstar[i])/ux[i]/uy[i]/(1-r[i]*r[i]); 	/* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	sum = 0;
	for (i=0; i < n - NPOLY; i++)
	{
		/* (x - X)^2/ux^2/(1-r^2)  + (y-Y)^2/uy^2/(1-r^2) -2 r (x-X) (y-Y)/ ux/uy/(1-r^2) */
		sum += (px[i] - xstar[i])*(px[i] - xstar[i])/ux[i]/ux[i]/(1-r[i]*r[i]);
		sum += (py[i] - ystar[i])*(py[i] - ystar[i])/uy[i]/uy[i]/(1-r[i]*r[i]);
		sum -= 2*r[i]*(px[i] - xstar[i])*(py[i] - ystar[i])/ux[i]/uy[i]/(1-r[i]*r[i]);
	}
	free(ystar);
	for (i=0; i < ndata; i++){
		free(dystar[i]);
	}
	free(dystar);
//	printf(" ---------------------------------------------\n");
	return sum;
}


int main (int argc, char *argv[])
{

	int c, i;
	char *fnm = NULL;
	Data data;
	int n, npar;
	double *par;
	double *dpar;
	cg_stats Stats ;
	cg_parameter Parm ;
	double ain = 0 , bin = 0;
	double a, b;
	double *xstar;

	if (argc == 1 ){
		print_help();
		exit(0);
	}

	while (( c = getopt (argc , argv, "f:ha:b:")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'a':
				ain=strtod(optarg, NULL);
				break;
			case 'b':
				bin =strtod(optarg, NULL);
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}

	n = read_area_data( fnm, &data);
	if (n == -1){
		printf(" Failed to load data from file %s\n", fnm);
		exit(1);
	}
	px = data.hc;
	py = data.Ac;
	ux = data.uhc;
	uy = data.uAc;
	r = data.rho;

	npar = data.ndata + NPOLY;
	par = (double *)malloc(npar*sizeof(double));
	dpar = (double *)malloc(npar*sizeof(double));
	/*initial estimate */
	par[0] = ain;
	par[1] = bin;

	for (i = 0; i< data.ndata; i++)
		par[i+NPOLY] = data.hc[i];

	printf("initial conditions %g %g \n", par[0], par[1]);

	cg_default(&Parm);
	Parm.PrintLevel = 1;
	Parm.PrintFinal = TRUE;
	/* snaha najit nejaky prepinac co by to zachranil ale nic moc
	Parm.QuadStep = FALSE; //jeste by zhorsil
	Parm.step = 1e-5; //jak zvolit aby se to jeste nezhorsilo
	Parm.rho = 1.5;// jen zhorsuje
	cg_default(&Parm);
	Parm.AWolfeFac = 0;
	Parm.PrintLevel = 1;
	Parm.PrintFinal = TRUE;
	*/


	printf(" chi2val %g \n", chi2val(par, npar));
	printf(" chi2valgrad %g \n", chi2valgrad(dpar, par, npar));

	for (i = 0; i< npar; i++)
		printf("dpar[%d] %g \n", i,dpar[i]);

	cg_descent (par, npar, &Stats, &Parm, 1.e-8, chi2val, chi2grad, chi2valgrad, NULL) ;
//	cg_descent (par, npar, NULL, NULL, 1.e-1, chi2val, chi2grad, chi2valgrad, NULL) ;
	printf(" Results \n");
	for (i = 0; i< NPOLY+data.ndata; i++)
		printf(" %g ", par[i]);
	printf("\n");
	printf(" final chi2 %g \n", chi2val(par, npar));

	xstar = (double *) malloc(data.ndata*sizeof(double));
	corr_deming_fit(data.hc, data.Ac, data.ndata, data.uhc[0], data.uAc[0], data.rho[0], &a, &b, xstar);

	printf("analytical solution assuming constant uncertainties and correlation: a = %g, b = %g \n", a, b);
	printf(" xstar: ");
	for (i = 0; i< data.ndata; i++)
		printf(" %g ", xstar[i]);
	printf ("\n");


	par[0] = a;
	par[1] = b;
	for (i = 0; i< data.ndata; i++)
		par[i+NPOLY] = xstar[i];


	printf(" theoretical chi2 %g \n", chi2val(par, npar));
	return 0;
}
