#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "cg_user.h"


#define NPOLY 2
double *px;
double *py;
double *ux;
double *uy;
double *r;
double *delta;

typedef struct {
	int ndata;
	double *x;
	double *y;
	double *ux;
	double *uy;
	double *delta; /*uy/ux */
	double *rho;
} Data;	

void print_help()
{
	printf(" fit the area calibration file with an Oliver Polynomial\n");
	printf(" -h    show this help\n");
	printf(" -f    input file \n");
}

int read_area_data( char *fnm, Data *data)
{
	FILE *file;
	int ndata = 0;
	int i;
	size_t size; 
	char *line = NULL;
	double x1, x2, x3, x4, x5, x6;

	file = fopen(fnm, "r");

	if (!file)
		return -1;
	
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL)
				ndata ++;
	}
	printf("ndata %d \n", ndata);

	data->ndata = ndata;
	data->x = (double *)malloc(ndata *sizeof(double));
	data->y = (double *)malloc(ndata *sizeof(double));
	data->ux = (double *)malloc(ndata *sizeof(double));
	data->uy = (double *)malloc(ndata *sizeof(double));
	data->delta = (double *)malloc(ndata *sizeof(double));
	data->rho = (double *)malloc(ndata *sizeof(double));

	rewind(file);

	ndata = 0; 
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL){
			i = sscanf(line, "%lf %lf %lf %lf %lf %lf", &x1, &x2, &x3, &x4, &x5, &x6);
			if (i == 6 ){
				data->x[ndata] = x1;
				data->y[ndata] = x2;
				data->ux[ndata] = x3;
				data->uy[ndata] = x4;
				data->delta[ndata] = x4/x3;
				data->rho[ndata] = x6;
				ndata++;
			}
		}
	}
	printf("ndata %d data->ndata %d \n", ndata, data->ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return ndata;
	}

	return ndata;
}

double chi2val(double *p, INT n)
{
	double sum = 0;
	double *xstar, *ystar;
	double *pc;
	double ndata;
	int i;

	pc = p;
	xstar = p+NPOLY;
	ndata = n - NPOLY;
	ystar = (double *) malloc(ndata*sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] =  pc[0]*xstar[i] + pc[1] ;
	}
/*	printf("\n star data \n");
	for (i = 0; i< ndata; i ++){
		printf(" %g %g \n", xstar[i], ystar[i]);
	}
	printf("\n star data \n");
	*/
	

	for (i=0; i < ndata; i++)
	{
		/* (x - X)^2/ux^2  + (y-Y)^2/uy^2 */
		sum += (px[i] - xstar[i])*(px[i] - xstar[i])/ux[i]/ux[i];
		sum += (py[i] - ystar[i])*(py[i] - ystar[i])/uy[i]/uy[i];
//		printf(" aux1 %g \n", (px[i] - xstar[i])*(px[i] - xstar[i])/ux[i]/ux[i];
//		printf(" aux2 %g \n", (py[i] - ystar[i])*(py[i] - ystar[i])/uy[i]/uy[i];
	}

	free(ystar);
	return sum;
}

void chi2grad(double *g, double *p, INT n)
{
	double *xstar, *ystar, **dystar;
	double *pc;
	double ndata;
	int i, j;
	double w;

	pc = p;
	xstar = p+NPOLY;
	ndata = n - NPOLY;

	ystar = (double *) malloc(ndata*sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = pc[0]*xstar[i] + pc[1] ;
	}

	/* dystar/dp */
	for (i = 0; i< ndata; i ++){
		dystar[i][0] += xstar[i];
		dystar[i][1] += 1;
	}
	/*partial ystar / partial xstar */
	for (i = 0; i< ndata; i ++){
		dystar[i][i+NPOLY] = pc[0];	
	}


	for (i=0; i < ndata; i++)
	{
		g[i+NPOLY] = -2*(px[i]-xstar[i])/ux[i]/ux[i]; /* dchi2/dxstar */

		w =  -2*(py[i]-ystar[i])/uy[i]/uy[i]; /* dchi2/dystar */
		g[i+NPOLY] += w*dystar[i][i+NPOLY]; /* dchi2/dystar * dystar/dxstar */

	}

	for (j=0; j< NPOLY; j++)
	{
		for (i=0; i < ndata; i++){
			w =  -2*(py[i]-ystar[i])/uy[i]/uy[i] ; /* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	free(ystar);
	for (i=0; i < ndata; i++)
		free(dystar[i]);
	free(dystar);
	return ;
}

double chi2valgrad(double *g, double *p, INT n)
{
	double sum = 0;
	double *xstar, *ystar, **dystar;
	double *pc;
	double ndata;
	int i,j;
	double w;

	pc = p;
	xstar = p+NPOLY;
	ndata = n - NPOLY;

//	printf(" ---------------------------------------------\n");
	ystar = (double *) malloc(ndata*sizeof(double));
	dystar = (double **) malloc(ndata*sizeof(double *)); /*dystar[i]/dpar[j]*/

	for (i=0; i < ndata; i++)
		dystar[i] = (double *) calloc(n, sizeof(double));

	for (i = 0; i< ndata; i ++){
		ystar[i] = pc[0]*xstar[i] + pc[1]; 
	}
	for (i = 0; i< ndata; i ++){
		printf("x[%d] %g xstar[%d] %g y[%d] %g ystar[%d] %g r[%d] %g  ux[%d] %g uy[%d] %g delta[%d] %g ", i,px[i], i, xstar[i] , i,py[i], i, ystar[i], i, r[i], i, ux[i], i , uy[i], i, delta[i]);
	}
	

	/* dystar /dp */
	for (i = 0; i< ndata; i ++){
		dystar[i][0] += xstar[i];
		dystar[i][1] += 1;
	}
	/* partial ystar / partial xstar */
	for (i = 0; i< ndata; i ++){
		dystar[i][i+NPOLY] = pc[0];
	}

	for (i=0; i< ndata; i++){
//		printf("dystar[%d] ", i);
		for (j=0; j< n; j++){
//			printf(" %d %g ", j, dystar[i][j]); 
		}
//		printf("\n");
	}


	for (i=0; i < ndata; i++)
	{
		g[i+NPOLY] = -2*(px[i]-xstar[i])*delta[i]*delta[i]; /* dchi2/dxstar */
		w =  -2*(py[i]-ystar[i]) ; /* dchi2/dystar */
		g[i+NPOLY] += w*dystar[i][i+NPOLY]; /* dchi2/dystar * dystar/dxstar */
		g[i+NPOLY] *= 1./uy[i]/uy[i];
	}

	for (j=0; j< NPOLY; j++)
	{
		for (i=0; i < ndata; i++){
			w =  -2*(py[i]-ystar[i])/uy[i]/uy[i] ; /* dchi2/dystar */
			g[j] += w *dystar[i][j]; /* dchi2/dystar * dystar/dpc */
		}
	}

	sum = 0;
	for (i=0; i < n - NPOLY; i++)
	{
		/* (x - X)^2/ux^2 + 2 rho (x-X)(y-Y)/ux/uy + (y-Y)^2/uy^2 */
		w = (py[i] - ystar[i])*(py[i] - ystar[i]) + (px[i] - xstar[i])*(px[i] - xstar[i])*delta[i]*delta[i];
		sum += w/uy[i]/uy[i]; 
	}
	free(ystar);
	for (i=0; i < ndata; i++){
		free(dystar[i]);
	}
	free(dystar);
//	printf(" ---------------------------------------------\n");
	return sum;
}


int main (int argc, char *argv[])
{

	int c, i;
	char *fnm = NULL;
	Data data;
	int n, npar;
	double *par;
	double *dpar;
	cg_stats Stats ;
	cg_parameter Parm ;
	double ain, bin;
	double tol = 1;

	if (argc == 1 ){
		print_help();
		exit(0);
	}

	while (( c = getopt (argc , argv, "f:ha:b:t:")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'a':
				ain=strtod(optarg, NULL);
				break;
			case 'b':
				bin =strtod(optarg, NULL);
				break;
			case 't':
				tol = strtod(optarg, NULL);
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}

	n = read_area_data( fnm, &data);
	if (n == -1){
		printf(" Failed to load data from file %s\n", fnm);
		exit(1);
	}
	px = data.x;
	py = data.y;
	ux = data.ux;
	uy = data.uy;
	delta = data.delta;
	r = data.rho;

	for (i=0; i< data.ndata; i++)
		r[i] = 0;
	
	npar = data.ndata + NPOLY;
	par = (double *)malloc(npar*sizeof(double));
	dpar = (double *)malloc(npar*sizeof(double));
	/*initial estimate */
	par[0] = ain;
	par[1] = bin;

	for (i = 0; i< data.ndata; i++)
		par[i+NPOLY] = data.x[i];

	printf("initial conditions %g %g \n", par[0], par[1]);

	cg_default(&Parm);
	Parm.PrintLevel = 1;
	Parm.PrintFinal = TRUE;


	printf(" chi2val %g \n", chi2val(par, npar));
	printf(" chi2valgrad %g \n", chi2valgrad(dpar, par, npar));

/*	for (i = 0; i< npar; i++)
		printf("dpar[%d] %g \n", i,dpar[i]);
		*/

	cg_descent (par, npar, &Stats, &Parm, tol, chi2val, chi2grad, chi2valgrad, NULL) ;
//	cg_descent (par, npar, &Stats, &Parm, 1.e-8, chi2val, chi2grad, chi2valgrad, NULL) ;
//	cg_descent (par, npar, NULL, NULL, 1.e-1, chi2val, chi2grad, chi2valgrad, NULL) ;
	printf(" Results \n");
	for (i = 0; i< NPOLY; i++)
		printf(" %g ", par[i]);
	printf("\n");
	return 0;
}
