program fit_area_odr_implicit

      use real_precision
      use odrpack95
      
! fit straight line through data from file
        
        implicit none
        character*256 ::  my_file, arg
        real(kind=r8), dimension (:,:), allocatable ::   x,y 
        real(kind=r8), dimension (:,:), allocatable ::   ux, uy 
        real(kind=r8), dimension (:), allocatable ::   rho
        real(kind=r8), dimension (:), allocatable ::   S, uS, covShc 
        real(kind=r8) ::    tmp
        real(kind=r8) ::    Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc
        real(kind=r8) ::    A, uAhc, uApar , uA

        integer ::     iargc,n

        integer ::  i, j, len1, len2, iostatus

        integer :: m, nq, np

        real(kind=r8), dimension(:),  allocatable :: beta(:)
        real(kind=r8), dimension(:),  allocatable  :: stdev_beta(:)
        real(kind=r8), dimension(:),  allocatable  :: cov(:)
        real(kind=r8) :: res_var, res_sum 

        integer :: info

        integer, parameter :: nt = 500
        real(kind = r8), parameter :: up = 500
        real(kind=r8), dimension (:) :: xt(nt)

        m = 2
        nq = 1

        if (iargc().lt.1)  stop './fit_area_odr filename'
        call getarg(1,my_file)

        if (iargc() .eq. 1) then
                np = 5
        else
                call getarg(2, arg)
                read(arg,*)  np
        end if

        allocate(beta(np))
        allocate(stdev_beta(np))
        allocate(cov(np*(np-1)/2))

        len1 = index(my_file,' ') - 1

        open(unit=77, file=my_file(1:len1), status='old')
        n = 0
        read(77, *, IOSTAT=iostatus) my_file
        do  while (iostatus == 0)
          n= n+1
          read(77, *, IOSTAT=iostatus) my_file
        end do

        if (n == 0 ) then
           stop 'Empty file'
        end if 
        
        n = n - 3 ! 3 lines header

        allocate(x(n, m))
        allocate(y(n, nq))
        allocate(ux(n,m))
        allocate(uy(n, nq))
        allocate(rho(n))
        allocate(S(n))
        allocate(uS(n))
        allocate(covShc(n))

        write(*,*) n, m, nq, size(x)

        rewind(unit = 77)
        !skip header
        read(77, *) my_file
        read(77, *) my_file
        read(77, *) my_file

        do i = 1,n
            read(77, *) x(i,1), x(i,2), ux(i,1), ux(i,2), tmp, rho(i), S(i), uS(i), covShc(i)
            x(i,2) = x(i,2)*1e6 !change units to nm
            ux(i,2) = ux(i,2)*1e6
        !    rho(i) = 0
        end do
        close(77)

        
        do i = 1, np
             beta(i) = 1e-1
        end do

!        beta(1) =19572.461697234903       
!        beta(2) =-2071789.1608516034        
!        beta(3) =26065893.188729271
!        beta(4) =-73302423.576737210        
!        beta(5) =50049752.723462105

         
         
 
        
        call odr_wrap_implicit(n,m, np, nq, x, y, beta, ux, uy, rho, stdev_beta, cov, res_var, res_sum, info)

        write(*,*) 'beta', beta
        write(*,*) 'stdev_beta', stdev_beta
        write(*,*) 'relative error', stdev_beta/beta
        write(*,*) 'correlation beta', cov
        write(*,*) 'residual variance', res_var
        write(*,*) 'residual sum of squares', res_sum
        write(*,*) 'info', info


        call getarg(1,my_file)
        write(arg, '("_np_",i1.1)') np
        len2 = index(arg,' ') - 1

        open(unit=88, file='area_odr_implicit_'//my_file(1:len1)//arg(1:len2))
        do i = 1, nt
           xt(i) = i*up/(nt-1)
        end do
        do i = 1, nt
           write( 88, *) xt(i), calc_Ac(xt(i), np, beta), & 
           calc_unc_Ac(xt(i), np, beta, stdev_beta, cov) 

        end do
        close(88)

        open(unit=99, file='backprop_area_odr_implicit'//my_file(1:len1)//arg(1:len2))
        write(99, *) '#hc, Er, u(Er;S), u(Er; Ah), u(Er; Apar), u(Er; cov), u(Er), A, u(A;h), u(A;par), u(A) '
        do i = 1, n
           call calc_unc_analysis(x(i,1), ux(i,1), S(i), uS(i), covShc(i),& 
                    Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc, &
                    A, uAhc, uApar, uA, np, beta, stdev_beta, cov ) 
           write( 99, *) x(i,1), Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc, &
                    A, uAhc, uApar, uA               
        end do
        close(99)




        contains

        real(kind=r8) function calc_Ac(x, np, beta)

        implicit none
        real(kind=r8), intent(in) :: x
        integer, intent(in) :: np
        real(kind=r8), intent(in) :: beta(np)
        integer ::j 

        calc_Ac = 24.5*x**2.0
        do j = 1,np
                calc_Ac  =  calc_Ac + beta(j)*x**(2.0**(-j+1.0))
        end do
        end function calc_Ac


        real(kind=r8) function calc_unc_Ac(x, np, beta, stdev_beta, cov)

        implicit none
        real(kind=r8), intent(in) :: x
        integer, intent(in) :: np
        real(kind=r8), intent(in) :: beta(np), stdev_beta(np), cov(np*(np-1)/2)
        real(kind =r8) :: uA, dAdbeta(np), covariance(np,np)
        integer ::j ,i,k


        uA = 0

        do j = 1,np
                dAdbeta(j) = x**(2.0**(-j+1.0))
        end do
 !       write(*,*) '------------------'
        do j = 1,np
                uA = uA + stdev_beta(j)**2.0*dAdbeta(j)**2.0
!                write(*,*) 'j', j, stdev_beta(j), dAdbeta(j), &
!                dAdbeta(j)*stdev_beta(j), dAdbeta(j)*stdev_beta(j)/calc_Ac(x,np,beta), calc_Ac(x,np,beta)
!        write(*,*) sqrt(uA), sqrt(uA)/calc_Ac(x,np,beta)
        end do
        k= 1
        do i = 1,np
          do j = i+1,np
              covariance(i,j) = cov(k)*stdev_beta(i)*stdev_beta(j)
              uA = uA + 2*covariance(i,j)*dAdbeta(i)*dAdbeta(j)
 !             write(*,*) sqrt(uA), sqrt(uA)/calc_Ac(x,np,beta)
              k= k+1
          end do
        end do

        calc_unc_Ac = sqrt(uA)

        end function calc_unc_Ac

        real(kind=r8) function calc_Ac_der(x, np, beta)
           
           implicit none
           real(kind=r8), intent(in) :: x
           integer, intent(in) :: np
           real(kind=r8), intent(in) :: beta(np)
           integer ::j 
  
           calc_Ac_der = 24.5*x*2
           do j = 1,np
                calc_Ac_der  =  calc_Ac_der + beta(j)*x**(2.0**(-j+1.0) -1.) *(2.0**(-j+1.0))
           end do
        
        end function calc_Ac_der


        subroutine calc_unc_analysis(hc, uhc, S, uS, covShc,& 
                    Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc, &
                    A, uAhc, uApar, uA, np,  beta, stdev_beta, cov ) 

            implicit none
            real(kind=r8), intent(in)  ::    hc, uhc, S, uS, covShc 
            real(kind=r8), intent(out) ::    Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc
            real(kind=r8), intent(out) ::    A, uAhc, uApar , uA
            integer, intent(in) :: np
            real(kind=r8), intent(in) :: beta(np), stdev_beta(np), cov(np*(np-1)/2)
            real(kind=r8) :: dAdh
            real(kind=r8),  parameter :: PI  = 4 * atan (1.0_8)
            
            A = calc_Ac(hc, np, beta)
            uApar = calc_unc_Ac(hc, np, beta, stdev_beta, cov) 
            dAdh = calc_Ac_der(hc, np, beta) 
            uAhc = abs(dAdh*uhc)
            uA = sqrt(uAhc*uAhc + uApar*uApar)

            !conversion factors
            Ercalc = sqrt(PI/A)*S/2.*1e6

            uErcalcS = Ercalc/S*uS
            uErcalcAhc = 1./2*Ercalc/A*uAhc
            uErcalcApar = 1./2*Ercalc/A*uApar
            uErcalccov = 2*Ercalc/S*1./2*Ercalc/A*dAdh*covShc

            uErcalc = sqrt( uErcalcS**2 + uErcalcAhc**2 + uErcalcApar**2 + uErcalccov)

            
        endsubroutine calc_unc_analysis
end program
