    

      subroutine odr_wrap_implicit( n,m,np,nq,x,y,beta,ux,uy, rho, stdev_beta,cov, res_var, res_sum, info)
      use real_precision
      use odrpack95

      implicit none

      !input
      integer, intent(in) :: n,m, np,nq
      real(kind=r8), intent(in) :: x(n,m), y(n,nq)
      real(kind=r8), intent(in) :: uy(n, nq), ux(n,m)
      real(kind=r8), intent(in) :: rho(n)

      ! parameter
      integer :: info, k, l
      integer:: ldwe , ldwd  !nonconstant uncertainties
      integer:: ld2we, ld2wd
      integer:: lwork, liwork, sdi, niteri, vcvi, rvari,wssi


      ! variable 
      integer :: niter
      integer, pointer, dimension(:) :: iwork
      integer, allocatable, target :: tiwork(:)
      integer :: i,j


      real(kind=r8), allocatable, dimension(:,:,:) :: we, wd
      real(kind=r8), pointer, dimension(:) :: work
      real(kind=r8), allocatable, target :: twork(:)

      !output
      real(kind=r8), intent(out) :: beta(np)
      real(kind=r8), intent(out) :: stdev_beta(np)
      real(kind=r8), intent(out) :: cov(np*(np-1)/2)
      real(kind=r8), intent(out) :: res_var, res_sum
      real(kind=r8) :: covariance(np,np)


      lwork = 18 + 13*np + np**2 + m + m**2 + &
               4*n*nq + 6*n*m + 2*n*nq*np + &
               2*n*nq*m + nq**2 + 5*nq + nq*(np+m) + n*nq*nq

      liwork = 20 + 2* np + nq *(np+m)               
      

      ldwe = n
      ldwd = n

      ld2we = nq
      ld2wd = m


      allocate(wd(ldwd, ld2wd, m))
      allocate(we(ldwe, ld2we, nq))
      allocate(twork(lwork))
      allocate(tiwork(liwork))

      work => twork
      iwork => tiwork


          we(1, 1, 1) = -1


      do i = 1, n
          wd(i, 1, 1) = 1/ux(i, 1)**2
          wd(i, 1, 2) = -rho(i)/ux(i,1)/ux(i,2)
          wd(i, 2, 1) = -rho(i)/ux(i,1)/ux(i,2)
          wd(i, 2, 2) = 1/ux(i, 2)**2
      end do
 

!     JOB 000000 auto forward fin. diff., 00010 - auto central fin. diff.; 00020 - user provided + check
! long output, print evrything possible iprint 6666, no output iprint 0
      call odr(fitfunc, &
               n, m, np, nq, &
               beta, &
               y,x, we = we , wd = wd, &
               job = 00021, &
               maxit = 1000, iprint = 0, &
               work = work, iwork = iwork,&
               info=info)


      ! number of iterations
      niteri = nq*np + nq*m + np + 15
      niter = iwork(niteri)

      ! stdev of fitted parameters
      sdi = 2*n*m+2*n*nq+1
      do k =1, np
          stdev_beta(k) = work(sdi-1 +k)
      enddo

      ! covariance matrix befor rescaling
      vcvi = 2*n*m + 2*n*nq + np + 1
      do k = 1,np
         do l = 1,np
             covariance(k,l) = work(vcvi -1 + k + (l-1)*np)
         enddo
      enddo

      !residual variance
      rvari = 2*n*m + 2*n*nq + np + np*np+ 1
      res_var = work(rvari)

      !residual sum of squares
      wssi =  2*n*m + 2*n*nq + np + np*np+ 2
      res_sum = work(wssi)
 
      !rescale covariance matrix
      covariance = covariance*res_var

     !  write(*,*) 'covariance', covariance
      !off-diagonal terms of covariance matrix
      k = 1
      do i = 1, np
        do j = i+1,np
          cov(k) = covariance(i,j)/stdev_beta(i)/stdev_beta(j)
          k=k+1
        enddo
      enddo

!      write(*,*) stdev_beta(1), stdev_beta(2),stdev_beta(3)
!      write(*,*) covariance(1,2)/stdev_beta(1)/stdev_beta(2), &
!                    covariance(1,3)/stdev_beta(1)/stdev_beta(3), &
!                    covariance(2,3)/stdev_beta(2)/stdev_beta(3)



      deallocate(twork)
      deallocate(tiwork)
      deallocate(wd)
      deallocate(we)

      contains

       function fun(x, m, beta, np)

      implicit none
      real(kind=r8) :: fun

      integer, intent(in) :: np, m
      real(kind=r8), intent(in) :: x(m)
      real(kind=r8), intent(in) :: beta(np)

      integer :: j


      fun = 24.5*x(1)**2 - x(2)
      do j = 1, np 
            fun = fun + beta(j)*x(1)**(2.0**(-j+1.0)) 
      end do

      end function fun


      function dfun_dbeta(x, m, beta, np)

      implicit none

      real(kind=r8), dimension (np) :: dfun_dbeta

      integer, intent(in) :: np, m
      real(kind=r8), intent(in) :: x(m)
      real(kind=r8), intent(in) :: beta(np)

      integer :: j

      do j=1,np
         dfun_dbeta(j) = x(1)**(2.0**(-j+1.0))
      end do

      end function dfun_dbeta


      function dfun_ddeltai(x, m, beta, np)

      implicit none

      real(kind=r8), dimension (m) :: dfun_ddeltai

      integer, intent(in) :: np, m
      real(kind=r8), intent(in) :: x(m)
      real(kind=r8), intent(in) :: beta(np)

      integer :: j

      dfun_ddeltai(1) = 24.5*2*x(1)
      dfun_ddeltai(2) = -1 

      do j = 1, np
            dfun_ddeltai(1) = dfun_ddeltai(1) + beta(j)*(2.0**(-j+1.0))*x(1)**(2.0**(-j+1.0)-1.0) 
      end do

      end function dfun_ddeltai

     

      subroutine fitfunc(n,m,np,nq,ldn, ldm, ldnp, beta, xplusd, ifixb, ifixx, &
      &      ldifx, ideval, f, fjacb, fjacd, istop)

      implicit none
      
      integer,intent(in) :: n,m, np, nq
      integer, intent(in) :: ldifx, ideval
      integer, intent(in) ::ldn, ldm, ldnp
      integer,intent(in) :: ifixb(np), ifixx(ldifx,m)
      
      real(kind=r8),intent(in) :: beta(np)
      real(kind=r8),intent(in) ::  xplusd(ldn,m)


      integer ::istop
      real(kind=r8) :: f(ldn, nq), fjacb(ldn, ldnp, nq), fjacd(ldn, ldm, nq) 

      integer :: i
        
      if (istop .ne. 0) then
              return
      end if         

      if (mod(ideval,10) .ge. 1) then
                 do i=1,n
                     f(i,1) = fun(xplusd(i,:), m,  beta, np)
                 end do
      end if         
      if (mod(ideval/10,10) .ge. 1) then
                 do i=1,n
                     fjacb(i,:,1) = dfun_dbeta(xplusd(i,:), m, beta, np)
                 end do
      end if         
      if (mod(ideval/100,10) .ge. 1) then
                 do i=1,n
                      fjacd(i,:,1) = dfun_ddeltai(xplusd(i,:), m, beta, np)
                 end do
      end if         

      end subroutine fitfunc
      end subroutine odr_wrap_implicit

