    

      subroutine odr_wrap( n,m,np,nq,x,y,beta,ux,uy,stdev_beta,cov,info)
      use real_precision
      use odrpack95

      implicit none

      !input
      integer, intent(in) :: n,m, np,nq
      real(kind=r8), intent(in) :: y(n,m), x(n,nq)
      real(kind=r8), intent(in) :: ux(n, nq), uy(n,m)

      ! parameter
      integer :: info, k, l
      integer:: ldwe , ldwd  !nonconstant uncertainties
      integer:: ld2we =1, ld2wd = 1 !one-dimensional
      integer:: lwork, liwork, sdi, niteri, vcvi, rvari


      ! variable 
      integer :: niter
      integer, pointer, dimension(:) :: iwork
      integer, allocatable, target :: tiwork(:)
      integer :: i,j


      real(kind=r8), allocatable, dimension(:,:,:) :: we, wd
      real(kind=r8), pointer, dimension(:) :: work
      real(kind=r8), allocatable, target :: twork(:)

      !output
      real(kind=r8), intent(out) :: beta(np)
      real(kind=r8), intent(out) :: stdev_beta(np)
      real(kind=r8), intent(out) :: cov(np*(np-1)/2)
      real(kind=r8) :: covariance(np,np)
      real(kind=r8) :: res_var


      lwork = 18 + 13*np + np**2 + m + m**2 + &
               4*n*nq + 6*n*m + 2*n*nq*np + &
               2*n*nq*m + nq**2 + 5*nq + nq*(np+m) + n*nq*nq

      liwork = 20 + 2* np + nq *(np+m)               
      

      ldwe = n
      ldwd = n

      allocate(wd(ldwd, ld2wd, m))
      allocate(we(ldwe, ld2we, nq))
      allocate(twork(lwork))
      allocate(tiwork(liwork))

      work => twork
      iwork => tiwork


      do i = 1, n
          wd(i, 1, 1) = 1/ux(i, 1)**2
          we(i, 1, 1) = 1/uy(i, 1)**2
      end do
 

!     JOB 000000 auto forward fin. diff., 00010 - auto central fin. diff.; 00020 - user provided + check
! long output, print evrything possible iprint 6666, no output iprint 0
      call odr(fitfunc, &
               n, m, np, nq, &
               beta, &
               y,x, we = we , wd = wd, &
               job = 00020, &
               maxit = 10000, iprint = 6666, &
               work = work, iwork = iwork,&
               info=info)


      ! number of iterations
      niteri = nq*np + nq*m + np + 15
      niter = iwork(niteri)

      ! stdev of fitted parameters
      sdi = 2*n*m+2*n*nq+1
      do k =1, np
          stdev_beta(k) = work(sdi-1 +k)
      enddo

      ! covariance matrix befor rescaling
      vcvi = 2*n*m + 2*n*nq + np + 1
      do k = 1,np
         do l = 1,np
             covariance(k,l) = work(vcvi -1 + k + (l-1)*np)
         enddo
      enddo

      !residual variance
      rvari = 2*n*m + 2*n*nq + np + np*np+ 1
      res_var = work(rvari)
 
      !rescale covariance matrix
      covariance = covariance*res_var

      !off-diagonal terms of covariance matrix
      k = 1
      do i = 1, np
        do j = i+1,np
          cov(k) = covariance(i,j)/stdev_beta(i)/stdev_beta(j)
          k=k+1
        enddo
      enddo

!      write(*,*) stdev_beta(1), stdev_beta(2),stdev_beta(3)
!      write(*,*) covariance(1,2)/stdev_beta(1)/stdev_beta(2), &
!                    covariance(1,3)/stdev_beta(1)/stdev_beta(3), &
!                    covariance(2,3)/stdev_beta(2)/stdev_beta(3)



      deallocate(twork)
      deallocate(tiwork)
      deallocate(wd)
      deallocate(we)

      contains

      real(kind=r8) function fun(x,beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      fun = 24.5*x**2 + beta(1)*x + beta(2)*sqrt(x) +&
            beta(3)*x**0.25 + beta(4)*x**0.125 + beta(5)*x**0.0625 

      end function fun


      real(kind=r8) function dfun_dbeta1(x, beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      dfun_dbeta1 =  x

      end function dfun_dbeta1

      real(kind=r8) function dfun_dbeta2(x, beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      dfun_dbeta2 =  sqrt(x)

      end function dfun_dbeta2


      real(kind=r8) function dfun_dbeta3(x, beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      dfun_dbeta3 =  x**0.25

      end function dfun_dbeta3


      real(kind=r8) function dfun_dbeta4(x, beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      dfun_dbeta4 =  x**0.125

      end function dfun_dbeta4


      real(kind=r8) function dfun_dbeta5(x, beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      dfun_dbeta5 =  x**0.0625

      end function dfun_dbeta5


      real(kind=r8) function dfun_ddeltai(x, beta, np)

      implicit none
      integer, intent(in) :: np
      real(kind=r8), intent(in) :: x
      real(kind=r8), intent(in) :: beta(np)

      dfun_ddeltai = 24.5*2*x + beta(1) + 0.5*beta(2)*x**(-0.5) +&
            beta(3)*0.25*x**(-0.75) + beta(4)*0.125*x**(-0.875) + &
            beta(5)*0.0625*x**(-0.9375)

      end function dfun_ddeltai

     

      subroutine fitfunc(n,m,np,nq,ldn, ldm, ldnp, beta, xplusd, ifixb, ifixx, &
      &      ldifx, ideval, f, fjacb, fjacd, istop)

      implicit none
      
      integer,intent(in) :: n,m, np, nq
      integer, intent(in) :: ldifx, ideval
      integer, intent(in) ::ldn, ldm, ldnp
      integer,intent(in) :: ifixb(np), ifixx(ldifx,m)
      
      real(kind=r8),intent(in) :: beta(np)
      real(kind=r8),intent(in) ::  xplusd(ldn,m)


      integer ::istop
      real(kind=r8) :: f(ldn, nq), fjacb(ldn, ldnp, nq), fjacd(ldn, ldm, nq) 

      integer :: l,i,j
        
      if (istop .ne. 0) then
              return
      end if         

      if (mod(ideval,10) .ge. 1) then
              do l=1,nq
                 do i=1,n
                     f(i,l) = fun(xplusd(i,l), beta, np)
                 end do
              end do    
      end if         
      if (mod(ideval/10,10) .ge. 1) then
              do l=1,nq
                 do i=1,n
                     fjacb(i,1,l) = dfun_dbeta1(xplusd(i,l), beta, np)
                     fjacb(i,2,l) = dfun_dbeta2(xplusd(i,l), beta, np)
                     fjacb(i,3,l) = dfun_dbeta3(xplusd(i,l), beta, np)
                     fjacb(i,4,l) = dfun_dbeta4(xplusd(i,l), beta, np)
                     fjacb(i,5,l) = dfun_dbeta5(xplusd(i,l), beta, np)
                 end do
              end do    
      end if         
      if (mod(ideval/100,10) .ge. 1) then
          do l=1,nq
             do j=1,m
                 do i=1,n
                      fjacd(i,j,l) = dfun_ddeltai(xplusd(i,j), beta, np)
                 end do
             end do
          end do    
      end if         

      end subroutine fitfunc
      end subroutine odr_wrap

