# - Try to find Gwyddion libs
# Once done this will define
#  GWY_FOUND - System has Gwy libs
#  GWY_INCLUDE_DIRS - The Gwy libs include directories
#  GWY_LIBRARIES - The libraries needed to use Gwy libs
#  GWY_DEFINITIONS - Compiler switches required for using Gwy libs

# udelat jako komponenty?
# kdyz by stacilo jenom gwyprocess pro niget-cmd, netahat GTK

# casem kdyztak vyloucit gwyapp, pokud nebude potreba

if(GWY_INCLUDE_DIR AND GWY_LIBDIR)
  
  message("Using paths to Gwyddion headers and libraries provided by the user ...")

  # gwyddion.h chosen as a representative of Gwy headers presence
  find_path(USER_GWY_INCLUDE_DIR libgwyddion/gwyddion.h
    HINTS ${GWY_INCLUDE_DIR}
    PATH_SUFFIXES gwyddion)
    
  find_path(USER_GWYCONFIG_INCLUDE_DIR gwyconfig.h
    HINTS ${GWY_LIBDIR}
    PATH_SUFFIXES gwyddion/include)
  
  find_library(GWY_LIBRARY_GWYAPP gwyapp2 HINTS ${GWY_LIBDIR})
  find_library(GWY_LIBRARY_GWYDDION gwyddion2 HINTS ${GWY_LIBDIR})
  find_library(GWY_LIBRARY_GWYDGETS gwydgets2 HINTS ${GWY_LIBDIR})
  find_library(GWY_LIBRARY_GWYDRAW gwydraw2 HINTS ${GWY_LIBDIR})
  find_library(GWY_LIBRARY_MODULE gwymodule2 HINTS ${GWY_LIBDIR})
  find_library(GWY_LIBRARY_GWYPROCESS gwyprocess2 HINTS ${GWY_LIBDIR})

  message("i1 ${USER_GWY_INCLUDE_DIR}")
  message("i2 ${USER_GWYCONFIG_INCLUDE_DIR}")
  message("l ${GWY_LIBRARY_GWYDDION}")

  if(USER_GWY_INCLUDE_DIR AND USER_GWYCONFIG_INCLUDE_DIR AND GWY_LIBRARY_GWYDDION)
    message("... success!")
    
    # Gwyddion might have been built with GtkGLExt
    # try to locate it (then Gwyddion should have found and used it as well)
    find_package(PkgConfig)
    pkg_check_modules(GTKGLEXT gtkglext-1.0)
    
    set(GWY_INCLUDE_DIRS ${USER_GWY_INCLUDE_DIR} ${USER_GWYCONFIG_INCLUDE_DIR} ${GTKGLEXT_INCLUDE_DIRS})
    set(GWY_VERSION 2.99) # improve and set the actual version
  else()
    message("... not found.")
  endif()
  
endif(GWY_INCLUDE_DIR AND GWY_LIBDIR)

if(NOT GWY_VERSION)
  
  message("Trying to find Gwyddion headers and libraries using pkg-config ...")
  find_package(PkgConfig)
  pkg_check_modules(PC_GWY gwyddion)

  if(PC_GWY_FOUND)
    message("... success!")
    set(GWY_DEFINITIONS ${PC_GWY_CFLAGS_OTHER})

    # only Gwy libraries are provided, 
    find_library(GWY_LIBRARY_GWYAPP gwyapp2
      HINTS ${PC_GWY_LIBDIR} ${PC_GWY_LIBRARY_DIRS})
    find_library(GWY_LIBRARY_GWYDDION gwyddion2
      HINTS ${PC_GWY_LIBDIR} ${PC_GWY_LIBRARY_DIRS})
    find_library(GWY_LIBRARY_GWYDGETS gwydgets2
      HINTS ${PC_GWY_LIBDIR} ${PC_GWY_LIBRARY_DIRS})
    find_library(GWY_LIBRARY_GWYDRAW gwydraw2
      HINTS ${PC_GWY_LIBDIR} ${PC_GWY_LIBRARY_DIRS})
    find_library(GWY_LIBRARY_MODULE gwymodule2
      HINTS ${PC_GWY_LIBDIR} ${PC_GWY_LIBRARY_DIRS})
    find_library(GWY_LIBRARY_GWYPROCESS gwyprocess2
      HINTS ${PC_GWY_LIBDIR} ${PC_GWY_LIBRARY_DIRS})
    
    ###
    ### # gwyddion.h chosen as a representative of Gwy headers presence
    ### find_path(GWY_INCLUDE_DIR libgwyddion/gwyddion.h
    ###   HINTS ${PC_GWY_INCLUDEDIR} ${PC_GWY_INCLUDE_DIRS}
    ###   PATH_SUFFIXES gwyddion)
    
    ### find_path(GWYCONFIG_INCLUDE_DIR gwyconfig.h
    ###   HINTS ${PC_GWY_LIBDIR}
    ###   PATH_SUFFIXES gwyddion/include)
    
    ### set(GWY_INCLUDE_DIRS ${GWY_INCLUDE_DIR} ${GWYCONFIG_INCLUDE_DIR} ${PC_GWY_INCLUDE_DIRS})

    # at the moment use include dirs as provided by pkg-config
    set(GWY_INCLUDE_DIRS ${PC_GWY_INCLUDE_DIR} ${PC_GWY_INCLUDE_DIRS})
    set(GWY_VERSION ${PC_GWY_VERSION})
    
  else(PC_GWY_FOUND)
    
    message("... not found.")
    message("Trying to download and build from source ...")

    # zkusit v tomto pripade definovat novy target (ev 2 - process a zbytek) ; jestli to teda ma smysl (pak by se kompilovaly az pri kompilaci)
    # prilinkovat staticky?
    
    # add_library(gwy1 STATIC deps/gwyddion/libxxx/yyy.c)
    # target_link_libraries(niget gwy1)
    # target_link_libraries(niget-cmd gwy1)

    #set(GWY_URL "svn://svn.code.sf.net/p/gwyddion/code/tags/gwyddion-${OWN_GWY_VER}")
    #execute_process(COMMAND svn checkout ${OWN_GWY_URL} ${OWN_GWY_SRC_DIR})
    
    # workaround for msys2
    # use CMAKE_BINDIR in execute_process as command parameters; CMAKE_BINARY_DIR when provided by cmake
    if(WIN32)
      string(REPLACE "C:/msys64" "" CMAKE_BINDIR ${CMAKE_BINARY_DIR})
    else()
      set(CMAKE_BINDIR ${CMAKE_BINARY_DIR})
    endif()
    
    set(OWN_GWY_VER 2.49)
    set(OWN_GWY_SRC_FILE "gwyddion-${OWN_GWY_VER}.tar.gz")
    set(OWN_GWY_URL http://gwyddion.net/download/${OWN_GWY_VER}/${OWN_GWY_SRC_FILE})
    set(OWN_GWY_SRC_DIR "${CMAKE_BINARY_DIR}/gwyddion-${OWN_GWY_VER}")
    set(OWN_GWY_INST_DIR "${CMAKE_BINARY_DIR}/gwyddion-${OWN_GWY_VER}-inst")
    
    message("Should be using ${CMAKE_BINDIR}/${OWN_GWY_SRC_FILE}")
    
    if(NOT EXISTS ${CMAKE_BINARY_DIR}/${OWN_GWY_SRC_FILE})
      message("Downloading Gwyddion sources from ${OWN_GWY_URL} to ${CMAKE_BINDIR}/${OWN_GWY_SRC_FILE} ...")
      file(DOWNLOAD ${OWN_GWY_URL} ${CMAKE_BINARY_DIR}/${OWN_GWY_SRC_FILE} STATUS DLSTATUS)
      message("Download finished with status: ${DLSTATUS}")
    endif()

    if(EXISTS ${CMAKE_BINARY_DIR}/${OWN_GWY_SRC_FILE}) # download succeeded or already present
      message("Extracting Gwyddion sources from ${CMAKE_BINDIR}/${OWN_GWY_SRC_FILE} ...")

      execute_process(COMMAND tar xf ${CMAKE_BINDIR}/${OWN_GWY_SRC_FILE} WORKING_DIRECTORY ${CMAKE_BINARY_DIR})
      
      message("Patching Gwyddion to compile just the libraries ...")
      execute_process(COMMAND patch --forward -i ${CMAKE_SOURCE_DIR}/gwyddion/Makefile.in.patch WORKING_DIRECTORY ${OWN_GWY_SRC_DIR})
      # --forward: do not complain when the patch has already been applied

      if(WIN32)
        set(ENV{PKG_CONFIG_PATH} /mingw32/lib/pkgconfig;/mingw32/share/pkgconfig)
      endif()

      message("Configuring Gwyddion ...")
      execute_process(COMMAND sh ./configure --prefix=${OWN_GWY_INST_DIR}
	--disable-gtk-doc --disable-dependency-tracking 
	--disable-schemas-install --disable-desktop-file-update --disable-pygwy
	--without-pascal --without-perl --without-python --without-ruby
	--without-kde4-thumbnailer --without-unique
	# --without-libiconv-prefix --without-libintl-prefix
	--without-x --without-gtksourceview --without-gl --without-zlib --without-bzip2 --without-libxml2
	WORKING_DIRECTORY ${OWN_GWY_SRC_DIR}
	RESULT_VARIABLE RES
	# WORKING_DIRECTORY ${CMAKE_BINDIR}/gwyddion-${OWN_GWY_VER}
	OUTPUT_FILE ${CMAKE_BINARY_DIR}/gwyddion.configure.out.log
	ERROR_FILE ${CMAKE_BINARY_DIR}/gwyddion.configure.err.log)
	message("Configure result: ${RES}")
      
      message("Compiling Gwyddion libraries ...")
      execute_process(COMMAND make CFLAGS=-Ofast
	WORKING_DIRECTORY ${OWN_GWY_SRC_DIR}
	OUTPUT_FILE ${CMAKE_BINARY_DIR}/gwyddion.make.out.log
	ERROR_FILE ${CMAKE_BINARY_DIR}/gwyddion.make.err.log)
      
      message("Installing Gwyddion libraries ...")
      execute_process(COMMAND make install
	WORKING_DIRECTORY ${OWN_GWY_SRC_DIR}
	OUTPUT_FILE ${CMAKE_BINARY_DIR}/gwyddion.install.out.log
	ERROR_FILE ${CMAKE_BINARY_DIR}/gwyddion.install.err.log)

      set(OWN_GWY_INCLUDE_DIR ${OWN_GWY_INST_DIR}/include/gwyddion)
      set(OWN_GWY_LIBDIR ${OWN_GWY_INST_DIR}/lib)

      # gwyddion.h chosen as a representative of Gwy headers presence
      find_path(OWN_GWY_INCLUDE_DIR libgwyddion/gwyddion.h
	HINTS ${OWN_GWY_INCLUDE_DIR}
	PATH_SUFFIXES gwyddion)
      
      find_path(OWN_GWYCONFIG_INCLUDE_DIR gwyconfig.h
	HINTS ${OWN_GWY_LIBDIR}
	PATH_SUFFIXES gwyddion/include)
      
      find_library(GWY_LIBRARY_GWYAPP gwyapp2 HINTS ${OWN_GWY_LIBDIR})
      find_library(GWY_LIBRARY_GWYDDION gwyddion2 HINTS ${OWN_GWY_LIBDIR})
      find_library(GWY_LIBRARY_GWYDGETS gwydgets2 HINTS ${OWN_GWY_LIBDIR})
      find_library(GWY_LIBRARY_GWYDRAW gwydraw2 HINTS ${OWN_GWY_LIBDIR})
      find_library(GWY_LIBRARY_MODULE gwymodule2 HINTS ${OWN_GWY_LIBDIR})
      find_library(GWY_LIBRARY_GWYPROCESS gwyprocess2 HINTS ${OWN_GWY_LIBDIR})

      message("i1 ${OWN_GWY_INCLUDE_DIR}")
      message("i2 ${OWN_GWYCONFIG_INCLUDE_DIR}")
      message("l ${GWY_LIBRARY_GWYDDION}")

      # ^ tohle jeste poladit

      if(OWN_GWY_INCLUDE_DIR AND OWN_GWYCONFIG_INCLUDE_DIR AND GWY_LIBRARY_GWYDDION)
	message("... all the required files are present, success!")
	
	set(GWY_INCLUDE_DIRS ${OWN_GWY_INCLUDE_DIR} ${OWN_GWYCONFIG_INCLUDE_DIR})
	set(GWY_VERSION ${OWN_GWY_VER})
      else()
	message("... something went wrong, some of the required files are missing!")
      endif()
      
    else()
      message("Gwyddion source package ${CMAKE_BINARY_DIR}/${GWY_SRC_FILE} not found, cannot continue")
    endif()
      
  endif(PC_GWY_FOUND)
endif(NOT GWY_VERSION)

mark_as_advanced(GWY_INCLUDE_DIR GWYCONFIG_INCLUDE_DIR
  GWY_LIBRARY_GWYAPP GWY_LIBRARY_GWYDDION GWY_LIBRARY_GWYDGETS
  GWY_LIBRARY_GWYDRAW GWY_LIBRARY_MODULE GWY_LIBRARY_GWYPROCESS)

set(GWY_LIBRARIES
  ${GWY_LIBRARY_GWYAPP} ${GWY_LIBRARY_GWYDDION} ${GWY_LIBRARY_GWYDGETS}
  ${GWY_LIBRARY_GWYDRAW} ${GWY_LIBRARY_MODULE} ${GWY_LIBRARY_GWYPROCESS}
  ${GTKGLEXT_LIBRARIES})

# handle the QUIETLY and REQUIRED arguments and set GWY_FOUND to TRUE if all listed variables are TRUE
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(GWY
  REQUIRED_VARS GWY_INCLUDE_DIRS GWY_LIBRARY_GWYDDION
  VERSION_VAR PC_GWY_VERSION
  HANDLE_COMPONENTS)
