#include "file-utils.h"

/* zrusit a rozdelit mezi fddata a area? */


/* this should be eliminated so that file-utils does not manipulate with data structures (clear_hF_data) */
#include "fddata.h"

#include "indent-toolbox-common.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>

#include <libprocess/gwyprocess.h>

#ifdef G_OS_WIN32
#include "getline.h"
#include <glib/gstdio.h>
#endif

static gchar* get_file_format_string(enum DataFileFormat fileformat, gint *nvars);
static void fprint_fddata(FILE *file, GwyDataLine *x, GwyDataLine *y);

static gint num_data_in_file(FILE *infile, enum DataFileFormat fileformat);
static FileReadStatus read_file_niget2(FILE *infile, gint numdata, GwyDataLine *time, GwyDataLine *hdata, GwyDataLine *Fdata,
				gint *i_contact_load, gint *i_load_hold, gint *i_hold_unload, gint *i_end_unload);
static FileReadStatus read_file_plaintext2(FILE *infile, enum DataFileFormat fileformat,
				    gint numdata, GwyDataLine *time, GwyDataLine *hdata, GwyDataLine *Fdata);


gboolean // unifikovat, dotahnout
file_contains_data(gchar *filename, enum DataFileFormat fileformat, FileReadStatus *filestatus)
{
    FILE *infile;
    gchar *line = NULL;
    gchar *formatstring;
    size_t size;
    gint numdata, nvars, n;
    gdouble hlp1, hlp2, hlp3;

    infile = fopen(filename, "r");
    if (!infile) {
	    *filestatus = READ_FILE_COULD_NOT_OPEN;
	    return FALSE;
    }

    if (fileformat == DATA_FILE_FORMAT_NIGET) {
	    getline(&line, &size, infile);
	    if (sscanf(line, DATAFORMAT_HEADER_FORMAT, &n, &n, &n, &n) != 4) { /* do not care about the values */
	        *filestatus = READ_FILE_FORMAT_MISMATCH;
	        return FALSE;
	    }
    }

    formatstring = get_file_format_string(fileformat, &nvars);

    numdata = 0;
    /* 3 points define an elementary loading-unloading curve */
    while ((getline(&line, &size, infile) != EOF) && (numdata < 3)) {
	localize_decimal_points(line);
	n = sscanf(line, formatstring, &hlp1, &hlp2, &hlp3);
	if (n == nvars)
	    numdata++;
    }
    
    fclose(infile);
    g_free(formatstring);

    if (numdata == 3) {
	    printf("file %s contains at least 3 valid data items, OK\n", filename);
	    *filestatus = READ_FILE_OK;
	    return TRUE;
    } else {
	    printf("file %s contains only %d valid data items, not OK\n", filename, numdata);
	    *filestatus = READ_FILE_NOT_ENOUGH_ENTRIES;
	    return FALSE;
    }
}

/*
read_file_xxx (dle typu souboru - niget, plaintext, ...) jenom primitivni, zaplni tri zadane datalines - t,F,d
dalsi ukony jako uklid, inicializaci, split, zajisti load_data (non-gui verze)
*/

gint
num_data_in_file(FILE *infile, enum DataFileFormat fileformat)
{
    gint numdata, n, nvars;
    size_t size;
    gdouble x1, x2, x3;
    gchar *formatstring, *line;

    formatstring = get_file_format_string(fileformat, &nvars);

    line = NULL;
    numdata = 0;
    while (getline(&line, &size, infile) != EOF) {
	localize_decimal_points(line);
	n = sscanf(line, formatstring, &x1, &x2, &x3);
	if (n == nvars)
	    numdata++;
    }
    rewind(infile);
    printf("numdata: %d\n",numdata);
    g_free(formatstring);

    return numdata;
}

/* have separate function for Niget files to allow future format changes */
FileReadStatus
read_file_niget2(FILE *infile, gint numdata, GwyDataLine *time, GwyDataLine *hdata, GwyDataLine *Fdata,
		gint *i_contact_load, gint *i_load_hold, gint *i_hold_unload, gint *i_end_unload)
{
    gchar *line = NULL;
    size_t size;

    getline(&line, &size, infile);
    localize_decimal_points(line);
    sscanf(line, DATAFORMAT_HEADER_FORMAT,
	   i_contact_load, i_load_hold, i_hold_unload, i_end_unload);

    return read_file_plaintext2(infile, DATA_FILE_FORMAT_NIGET, numdata, time, hdata, Fdata);
}

FileReadStatus
read_file_plaintext2(FILE *infile, enum DataFileFormat fileformat,
		     gint numdata, GwyDataLine *time, GwyDataLine *hdata, GwyDataLine *Fdata)
{
    gint i, n, nvars;
    gchar *formatstring, *line;
    size_t size;
    gdouble x1, x2, x3;

    formatstring = get_file_format_string(fileformat, &nvars);
    line = NULL;
    i = 0;
	    	
    while (getline(&line, &size, infile) != EOF) {
	localize_decimal_points(line);
	n = sscanf(line, formatstring, &x1, &x2, &x3);
	if (n == nvars) {	    	
	    switch (fileformat) {
	    case (DATA_FILE_FORMAT_UNHT_D_NM_F_MN):
		time->data[i] = i;
		hdata->data[i] = x2;
		Fdata->data[i] = x3;
		break;

	    case (DATA_FILE_FORMAT_UNHT_D_NM_F_UN):
		time->data[i] = i;
		hdata->data[i] = x2;
		Fdata->data[i] = x3 / 1000;
		break;

	    case (DATA_FILE_FORMAT_HYSITRON):
		time->data[i] = i;
		hdata->data[i] = x1;
		Fdata->data[i] = x2 / 1000;
		break;
		    
	    case (DATA_FILE_FORMAT_NIGET):
	    case (DATA_FILE_FORMAT_D_NM_F_MN):
		time->data[i] = i;
		hdata->data[i] = x1;
		Fdata->data[i] = x2;
		break;

	    case (DATA_FILE_FORMAT_F_MN_D_NM):
		time->data[i] = i;
		hdata->data[i] = x2;
		Fdata->data[i] = x1;
		break;

	    case (DATA_FILE_FORMAT_F_UN_D_NM):
		time->data[i] = i;
		hdata->data[i] = x2;
		Fdata->data[i] = x1 / 1000;
		break;

	    default:
		break;
	    }
	    i++;
	}
    }

    if (i == numdata)
	return READ_FILE_OK;
    else
	return READ_FILE_NOT_ENOUGH_ENTRIES;
}

FileReadStatus /* find a right place for this function */
load_data_nogui(gchar *filename, enum DataFileFormat fileformat, FDdata *fddata, gboolean *splitok)
{
    FileReadStatus readstatus;
    gint i, numdata, i_contact_load, i_load_hold, i_hold_unload, i_end_unload;
    FILE *infile;
    GwyDataLine *timedata, *hdata, *Fdata;
    
    /* we should check if fddata is not NULL ... */
    /* will be avoided when this function actually creates the fddata */
    /* so now we should clean up existing fddata to prevent leaks */
    /* ale data bych mel likvidovat az tehdy, kdyz vim, ze se neco fakt uspesne nacetlo */

    printf("loading data from %s\n", filename);

    infile = fopen(filename, "r");
    if (!infile)
	return READ_FILE_COULD_NOT_OPEN;

    numdata = num_data_in_file(infile, fileformat);

    timedata = gwy_data_line_new(numdata, numdata, FALSE);
    hdata = gwy_data_line_new(numdata, 1.0, FALSE);
    Fdata = gwy_data_line_new(numdata, 1.0, FALSE);

    switch (fileformat) { /* tady pridavat nove formaty */
    case DATA_FILE_FORMAT_NIGET:
	readstatus = read_file_niget2(infile, numdata, timedata, hdata, Fdata,
				      &i_contact_load, &i_load_hold, &i_hold_unload, &i_end_unload);
    	break;

    default:
	readstatus = read_file_plaintext2(infile, fileformat, numdata, timedata, hdata, Fdata);
    	break;
    }

    fclose(infile);

    if (readstatus != READ_FILE_OK) /* the right number of entries were not read */
	return readstatus;

    /* check if data is not inf or nan */
    for (i = 0; i < numdata; i++) {
	if (isnan(hdata->data[i]) || isnan(Fdata->data[i])) {
	    dataline_clear(timedata);
	    dataline_clear(hdata);
	    dataline_clear(Fdata);
	    return READ_FILE_CONTAINS_NAN;
	}
	if (isinf(hdata->data[i]) || isinf(Fdata->data[i])) {
	    dataline_clear(timedata);
	    dataline_clear(hdata);
	    dataline_clear(Fdata);
	    return READ_FILE_CONTAINS_INF;
	}
    }

    /* everything OK, populate fddata (check whether all fields are complete!) */
    clear_hF_data(fddata);
    init_FDdata(fddata);

    fddata->ndata = numdata;
    fddata->time = gwy_data_line_duplicate(timedata);
    fddata->horig = gwy_data_line_duplicate(hdata);
    fddata->Forig = gwy_data_line_duplicate(Fdata);
    fddata->hc = hdata->data[i_contact_load]; /* VG: use of unitialized value */
    fddata->Fc = Fdata->data[i_contact_load]; /* VG: use of unitialized value */

    fddata->filename = strdup(filename);
    fddata->fileformat = fileformat;

    /* split */
    switch (fileformat) {
    case DATA_FILE_FORMAT_NIGET:

	fddata->i_contact_load = i_contact_load;
	fddata->i_load_hold = i_load_hold;
	fddata->i_hold_unload = i_hold_unload;
	fddata->i_end_unload = i_end_unload;
	
	*splitok = split_hF_data_spec(fddata,
				      fddata->i_contact_load, fddata->i_load_hold,
				      fddata->i_hold_unload, fddata->i_end_unload);
	    
	fddata->status_contact_load = POINT_STATUS_FROM_FILE;
	fddata->status_load_hold = POINT_STATUS_FROM_FILE;
	fddata->status_hold_unload = POINT_STATUS_FROM_FILE;
	fddata->status_end_unload = POINT_STATUS_FROM_FILE;
	break;
	
    default: /* standard data formats */
	fddata->i_contact_load = 0; // musi se vynulovat i_load_hold i_hold_unload?
	fddata->i_load_hold = 0;
	fddata->i_hold_unload = 0;
	fddata->i_end_unload = numdata - 1;
	    
	fddata_find_contact(fddata->horig, fddata->Forig, &(fddata->i_contact_load), &(fddata->hc), &(fddata->Fc));
	fddata->status_contact_load = POINT_STATUS_SET_AUTO;

	*splitok = split_hF_data_auto(fddata);
	break;
    }

    return READ_FILE_OK;
}

gboolean data_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus)
{
    FILE *file;
    FDdata *fddata;

    fddata = &(args->fddata);
	
    file = fopen(filename, "w");
    if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
    }

    fprintf(file, "# %s \n", fddata->filename);
    if (fddata->horig && fddata->Forig) {
	    fprintf(file, "#fddata: horig, Forig\n");
	    fprint_fddata(file, fddata->horig, fddata->Forig);
    }

    if (fddata->hload && fddata->Fload) {
	    fprintf(file, "#fddata: hload, Fload\n");
	    fprint_fddata(file, fddata->hload, fddata->Fload);
    }

    if (fddata->hhold && fddata->Fhold) {
	    fprintf(file, "#fddata: hhold, Fhold\n");
	    fprint_fddata(file, fddata->hhold, fddata->Fhold);
    }

    if (fddata->hunload && fddata->Funload) {
	    fprintf(file, "#fddata: hunload, Funload\n");
	    fprint_fddata(file, fddata->hunload, fddata->Funload);
    }

    fclose(file); 
    *filestatus = SAVE_FILE_OK;
    return TRUE;
}

/* static functions */

static gchar*
get_file_format_string(enum DataFileFormat fileformat, gint *nvars)
{
    switch (fileformat) {
        case (DATA_FILE_FORMAT_UNHT_D_NM_F_MN):
        case (DATA_FILE_FORMAT_UNHT_D_NM_F_UN):
	        *nvars = 3;
	        return g_strdup("%lf %lf %lf");
	        break;
        case (DATA_FILE_FORMAT_NIGET):
        case (DATA_FILE_FORMAT_HYSITRON):
        case (DATA_FILE_FORMAT_D_NM_F_MN):
        case (DATA_FILE_FORMAT_F_MN_D_NM):
        case (DATA_FILE_FORMAT_F_UN_D_NM):
	        *nvars = 2;
	        return g_strdup("%lf %lf");
	        break;
        default:
	        return NULL;
	    break;
    }
}

static void
fprint_fddata(FILE *file, GwyDataLine *x, GwyDataLine *y)
{
    gint i;

    for (i = 0; i < x->res; i++)
	    fprintf(file, "%f\t%f\n", x->data[i], y->data[i]);
    fprintf(file, "\n\n");
}

/* Creates full log filename path from filename.
   Returned value must be deallocated.
*/
gchar*
create_log_filename_path(const gchar* fname)
{
	gchar* res = NULL;
	gint n = 0;

/*
	gchar buf[4]; 
	GRand *rng;
	gint num;

	rng= g_rand_new();
	num = g_rand_int_range(rng,0,1000);
	printf(" num %d \n", num);
	sprintf(buf, "%03d",num);
	printf(" buf %s \n", buf);
*/

#ifdef G_OS_WIN32	
	GStatBuf st;
	res = g_malloc(512*sizeof(gchar));
	strcpy(res, getenv("APPDATA"));
	strncat(res, "\\Niget\\", strlen("\\Niget\\"));
	if(g_stat(res, &st) == -1)
		g_mkdir(res, 0700);
	strncat(res, fname, strlen(fname));
#else
//	n = strlen(fname) + 3;
	n = strlen(fname) ;
	res = g_malloc((n+1)*sizeof(gchar)); 
	strncpy(res, fname,n+1); 
//	strncat(res,buf, 3);
#endif

//	g_rand_free(rng);
	return res;
}

gint get_number_lines(FILE *infile) 
{
	gint n = 0 ;

	gchar *line = NULL;
	size_t size;

	while (getline(&line, &size, infile) > -1)
		n++;

	rewind(infile);
	return n;
}

AreaFileReadStatus read_area_hysitron_file(const gchar *fnm, Area *area)
{
	FILE *infile;
	gchar *line = NULL;
	size_t size;
	gint i,n ;


	infile = fopen(fnm, "r");
	if (!infile)
		return AREA_FILE_COULD_NOT_OPEN;


	n = get_number_lines(infile);

	if (n<14)
		return AREA_FILE_NOT_ENOUGH_ENTRIES;

	getline(&line, &size, infile); // header

	printf("n %d \n", n);
	area->npoly = (n - 22 - 4 )/4;
	printf(" npoly %d \n", area->npoly);
	for (i = 0; i< area->npoly; i++){
		getline(&line, &size, infile); // name of coefficient
		getline(&line, &size, infile); // coefficient
		localize_decimal_points(line);
		if (sscanf(line, "%lf", &(area->polycoef[i]) )!=1){
			return AREA_FILE_FORMAT_MISMATCH;
		}
	}
	fclose(infile);
	g_free(line);

	area->mode = AREA_POLYNOM;
	return AREA_FILE_OK;
}

AreaFileReadStatus read_area_csm_file(const gchar *fnm, Area *area)
{
	FILE *infile;
	gchar *line = NULL;
	size_t size;
	gint i,n ;
	gchar **c;
	gdouble p;


	infile = fopen(fnm, "r");
	if (!infile)
		return AREA_FILE_COULD_NOT_OPEN;

	n = get_number_lines(infile);

	if (n<14)
		return AREA_FILE_NOT_ENOUGH_ENTRIES;

	// TODO add possibility to read h,A data pairs
	
	/* search for keyword FIT */
	getline(&line, &size, infile); 
	while( strstr(line, "FIT") == NULL)
		getline(&line, &size, infile); 

	getline(&line, &size, infile); // degree of polynome
	if (strstr(line,"POLYNOME DEGREE") == NULL)
		return AREA_FILE_FORMAT_MISMATCH;

	c = g_strsplit (line, "=", -1);
	printf("size c len c %ld %ld \n", sizeof(c), sizeof(c[0]));
	area -> npoly = atoi(c[1]) + 1;

	if (area->npoly > 6){
		printf(" Currently only OP polynom up to order 5 is supported. \n");
		return AREA_FILE_POLYNOM;
	}
	
	getline(&line, &size, infile); // OP polynome
	if (strstr(line,"OLIVER") == NULL){
		printf(" Currently only OP polynome up to order 5 is supported. \n");
		return AREA_FILE_POLYNOM;
	}

	getline(&line, &size, infile); // 24.5 coefficient
	if (strstr(line,"24.5") == NULL){
		printf(" Currently only Berkovich tips with coefficient 24.5 are implemented. \n");
		return AREA_FILE_TIP;
	}

	for (i = 1; i<  area->npoly;i++){
		if (getline(&line, &size, infile) > -1){
			c = g_strsplit (line, "=", -1);
			p = g_strtod(c[1], NULL);
			printf(" c %s p %g \n", c[1], p);
			area->polycoef[i] = p*pow(10, 9*(pow(2,i)-1)/pow(2, i-1));
			printf(" %d  %g %g \n", i, p, area->polycoef[i]);
		}
		else{
			printf("Number of coefficients does not correspond to order of polynom.\n");
			return AREA_FILE_FORMAT_MISMATCH;
		}
	}

	fclose(infile);
	g_free(line);

	area->mode = AREA_POLYNOM;
	return AREA_FILE_OK;

}

AreaFileReadStatus read_area_dat_file(const gchar *fnm, Area *area)
{
	FILE *infile;
	gchar *line = NULL;
	size_t size;
	gint i, n, nn ;
	gdouble  x, y;



	infile = fopen(fnm, "r");
	if (!infile)
		return AREA_FILE_COULD_NOT_OPEN;

	n = get_number_lines(infile);

	if (n<10)
		return AREA_FILE_NOT_ENOUGH_ENTRIES;

	if (!area-> xdata){
		area->xdata = gwy_data_line_new(n+1, n+1, FALSE);
	}
	else{
		gwy_data_line_resample(area->xdata, n+1, GWY_INTERPOLATION_NONE);
	}
	if (!area-> ydata){
		area->ydata = gwy_data_line_new(n+1, n+1, FALSE);
	}
	else{
		gwy_data_line_resample(area->ydata, n+1, GWY_INTERPOLATION_NONE);
	}
	

	area->xdata->data[0] = 0; // TODO je to nutne a rozumne?
	area->ydata->data[0] = 0;

	for (i = 0; i < n; i++) {
		getline(&line, &size, infile);
		localize_decimal_points(line);
		nn = sscanf(line, "%lf %lf", &x, &y);
		if (nn < 2) {
			return AREA_FILE_FORMAT_MISMATCH;
		}
		area->xdata->data[i+1] = x;
		area->ydata->data[i+1] = y;
	}
	fclose(infile);
	g_free(line);

	area->mode = AREA_DATA;
	area->xmax = gwy_data_line_get_max(area->xdata);
	return AREA_FILE_OK;
}
