#!/usr/bin/python

import  numpy as np
import  os, sys
import commonfunctions as cf


ncontact  = 3

if (len (sys.argv) < 3):
	print sys.argv[0] + "sample [BK7 | silicaHys | silicaCSM]   device [HYS | CSM] + [blacklist(files that shouldbe skpped), optional] + [contact (include contact point uncertainty), optional]"
	exit(0)

sample = sys.argv[1]	
device = sys.argv[2]	
try:
   blacklistfnm = sys.argv[3]
except:
   blacklistfnm = ""

contact = False
try:
  if sys.argv[3] == "contact" or sys.argv[4] == "contact":
    contact = True
except:
    pass

fniglist=[]
for f  in os.listdir(os.getcwd()):
        if f.endswith(".nig") and f.find("A_LC")==-1 and f.find("B_LC")==-1 and not f.startswith("out"):
           fniglist.append(f)


if (sample == "BK7"):
#BK7
    Eeff = 90.47 #certificate
    nu  = 0.208  #literature
    Eit = (1-nu*nu)*Eeff #86.556
    Ei = 1141 #literature
    nui = 0.07 #literature
    Er = 1/(1/Eeff + (1-nui*nui)/Ei) # 83.854
    uEeff = 0.04*Eeff  #certificate 4 % hardness
    uEr = Er*Er/Eeff/Eeff * uEeff #
elif(sample =="silicaCSM"):
#fused silica CSM
    Eeff = 75.3 #certificate
    nu = 0.16 #certificate - literature
    uEeff = 0.3 #certificate
    Eit = (1-nu*nu)*Eeff #73.372
    Ei = 1141 #literature
    nui = 0.07 #literature
    Er = 1/( 1/Eeff + (1-nui*nui)/Ei) # 70.685
    uEr = Er*Er/Eeff/Eeff * uEeff #0.26435 
elif(sample =="silicaHys"):
#fused silica Hys
    Er = 71.6 #certificate 69.6
    nu = 0.16 #certificate - literature
    uEr = 0.05 * Er #certificate 5 %
else:
    print "Unknown reference sample."
    exit(1)



#fniglist=[f for f in os.listdir(os.getcwd()) if f.endswith(".nig") and cf.check_nig_has_results(f)]
fniglist=[f for f in fniglist if cf.check_nig_has_results(f)]

blacklist = []
if (blacklistfnm):
	with open(blacklistfnm) as fpy:
		for line in fpy.readlines():
			try:
				blacklist.append(line.split()[0])
			except:
				pass
fniglist = [f for f in fniglist if f not in blacklist ]

#nahodne vybereme

low=20
up=98
if (device == "CSM"):
	fniglist.sort(cmp=cf.cmp_fnm_nig_csm)
elif (device == "HYS"):
#	fniglist.sort(cmp=cf.cmp_fnm_nig_hys)
	fniglist.sort()


ndat = len(fniglist)
Alist = np.zeros(ndat)
uAlist = np.zeros(ndat)


print "#area function, rozsah %(l).2g - %(u).2g" %{'u':up, 'l':low}
print "# Er/GPa %(E)g u(Er)/GPa %(u)g" %{'E':Er, 'u':uEr}
print "# hc/nm, area/um^2, u(hc)/nm, u(area)/um^2, cov(hc, area), corr(hc,area), S/mN nm^(-1), u(S), cov(S,hc)"
for f in fniglist:
	fnm=cf.create_op_odr_simple_filename(f,low,up)
	if (not os.path.isfile(fnm)):
		print 'no file' ,fnm
		break
	res=cf.read_op_odr_simple_file(fnm)
	fnm=cf.create_op_odr_unc_filename(f,low,up)
	if (not os.path.isfile(fnm)):
		print 'no file' ,fnm
		break
	resunc=cf.read_op_odr_unc_file(fnm, ncontact)

	F=res['Fmax'] 
	S=res['S']
	m=res['m']
	hc=res['hc']
	hp=res['hp']
	hmax=res['hmax']
	eps=res['eps']
	uhc = resunc['GUMhc']
	hc_contact = resunc['contacthc']
	imiddle = ncontact
	uhc_contact = max([abs(hc_contact[imiddle] - hc_contact[imiddle-1]), abs(hc_contact[imiddle+1] - hc_contact[imiddle])])
	if (contact):
		uhc = np.sqrt(uhc**2 + uhc_contact**2)
	um = resunc['GUMm']
	uhmax = resunc['uh']
	uhp = resunc['GUMhp']
	uS = resunc['GUMS']
	covmhp = resunc['GUMcovmhp']
	depsdm = resunc['GUMdepsdm']
	uF = resunc['uF']

	Acalc = np.pi*(S/2/Er)**2*1e6
	Alist[fniglist.index(f)] = Acalc

	dAdS= 2*Acalc/S
	dAdE = -2*Acalc/Er
	uA = np.sqrt(dAdS*dAdS*uS*uS + dAdE*dAdE * uEr *uEr)
	uAlist[fniglist.index(f)] = uA
	
	dSdm = S/m
	dSdhp = S/(hmax-hp)
	dSdhmax = -S/(hmax-hp)
	dhcdm = (-depsdm/m + eps/m**2)*(hmax-hp)
	dhcdhp = eps/m
	dhcdhmax = 1-eps/m
	covShc = dSdm*dhcdm*um**2 + dSdhmax*dhcdhmax *uhmax**2 + dSdhp*dhcdhp*uhp**2 + (dSdm*dhcdhp + dSdhp*dhcdm)*covmhp
	covhcA = dAdS * covShc
#       S should not affected by the choice of contact point, covariance neither
	print hc, Acalc, uhc, uA, covhcA, covhcA/uhc/uA, S, uS, covShc, F, uF
			


with open("intercorrelation_%s.txt" %blacklistfnm, "w") as fout:
	for i in range(ndat):
		for j in range(i):
			cov = 4 * Alist[i] * Alist[j]/Er/Er *uEr*uEr/uAlist[i]/uAlist[j]
		#	print Er, uEr, Alist[i], Alist[j], uAlist[i], uAlist[j], cov
			fout.write(" %g" %cov)
		fout.write("\n")
			
