#!/usr/bin/python

import sys, os, numpy


#ncontact=5
#Nmc=1000

hertz_modes = { "0":"R", "1":"Er", "2":"Eit"}

def read_op_odr_simple_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		hmax=float(content[14].split()[1])
		Fmax=float(content[15].split()[1])
		hp=float(content[16].split()[1])
		m=float(content[17].split()[1])
		eps=float(content[24].split()[1])
		R2=float(content[21].split()[1])
		chi2=float(content[23].split()[1])
		hc=float(content[25].split()[1])
		S=float(content[26].split()[1])
		Ap=float(content[27].split()[1])
		Hit=float(content[28].split()[1])
		Er=float(content[29].split()[1])
		num=int(content[35].split()[1])
		low=float(content[33].split()[2].replace("[",""))
		up=float(content[33].split()[4].replace("]",""))
		return {'low':low, 'up':up, 'num':num, 'Fmax':Fmax, 'hmax':hmax, 'Er':Er, 'Hit':Hit, 'Aphc':Ap, 'R2':R2, 'chi2':chi2, 'hc':hc, 'S':S, 'm':m, 'hp':hp, 'eps':eps}

def read_op_odr_unc_file(filename, ncontact):
	with open(filename) as fpy:
		content=fpy.readlines()

		uh=float(content[2].split()[2])
		uF=float(content[3].split()[2])

		um=float(content[21].split()[2])
		uhp=float(content[22].split()[2])
		covmhp=float(content[23].split()[2])
		ueps=float(content[24].split()[2])
		depsdm=float(content[25].split()[2])
		uS=float(content[26].split()[2])
		uhc=float(content[27].split()[2])
		uAp=float(content[28].split()[2])
		uHit=float(content[29].split()[2])
		uEr=float(content[30].split()[2])
		uEit=float(content[31].split()[2])

		shiftc=range(-ncontact,ncontact+1)
		Ec=[]
		Hc=[]
		Ac=[]
		hc=[]
		offset=66
		iE0=offset+3+ncontact
		iH0=iE0+5+2*ncontact
		iA0=iH0+5+2*ncontact
		ih0=iA0+5+2*ncontact
		for i in range(-ncontact,ncontact+1):
			aux = float(content[iE0+i].split()[2].replace("not", "nan"))
			Ec.append(aux)		
			aux = float(content[iH0+i].split()[2].replace("not", "nan"))
			Hc.append(aux)		
			aux = float(content[iA0+i].split()[2].replace("not", "nan"))
			Ac.append(aux)		
			aux = float(content[ih0+i].split()[2].replace("not", "nan"))
			hc.append(aux)		

		return {'uh': uh, 'uF':uF,
			'GUMAp':uAp, 'GUMHit': uHit, 'GUMEr': uEr, 'GUMS':uS,
			'GUMhp': uhp, 'GUMm':um, 'GUMcovmhp':covmhp,  'GUMhc':uhc, 'GUMcovmhp':covmhp,
			'GUMeps': ueps, 'GUMdepsdm':depsdm,
			'contactEr':Ec, 'contactHit':Hc, 'contactAp':Ac, 'contacthc':hc}

def read_op_odr_mc_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		Ap=float(content[19].split()[2].replace("(",""))
		uAp=float(content[19].split()[4].replace(")",""))
		Hit=float(content[20].split()[2].replace("(",""))
		uHit=float(content[20].split()[4].replace(")",""))
		Er=float(content[21].split()[2].replace("(",""))
		uEr=float(content[21].split()[4].replace(")",""))
		return {'mcavgAp': Ap, 'mcstdAp': uAp,'mcavgHit': Hit, 'mcstdHit': uHit, 'mcavgEr': Er, 'mcstdEr': uEr}
		
		
def create_op_odr_mc_filename(filename, Nmc, low,up):
	return filename+"__op_odr_mc%(n)d_range_%(l).2g_%(u).2g_%%.dat" %{'n':Nmc,'l':low, 'u':up}
		
def create_op_odr_unc_filename(filename, low,up):
	return filename+"__op_odr_unc_range_%(l).2g_%(u).2g_%%.dat" %{'l':low, 'u':up}
		
def create_op_odr_simple_filename(filename, low,up):
	return filename+"__op_odr_range_%(l).2g_%(u).2g_%%.dat" %{'l':low, 'u':up}
		

def check_nig_has_results(filename):
	for f  in os.listdir(os.getcwd()):
		if ((f.find(filename) > -1) and f.endswith("dat")):
			return True
	return False

def cmp_fnm_nig_csm(fnm1, fnm2):
    ss1=fnm1[:-4].split("_")
    ss2=fnm2[:-4].split("_")
    len1= len(ss1)
    len2= len(ss2)
    for i in range(min(len1, len2)):
        try:
            s1i = int(ss1[i])
            s2i = int(ss2[i])
        except ValueError:
	    s1i = ss1[i]
	    s2i = ss2[i]

        if (s1i < s2i):
            return -1
        elif (s1i> s2i):
	    return 1	

    if (len2 > len1):
            return -1
    elif (len2 < len1): 
	    return 1
    else:
            return 0

def cmp_fnm_nig_hys(fnm1, fnm2): #proc takhle slozite?
    ss1=fnm1[:-4].split("_")
    ss2=fnm2[:-4].split("_")
    if (ss1[0]< ss2[0]):
	return -1	
    elif (ss1[0]> ss2[0]):
	return 1	
    else:    
	si1=ss1[1]
	si2=ss2[1]
	for c in ["A","B", "C"]:
		si1=si1.replace(c,"")
		si2=si2.replace(c,"")
	i1=int(si1)
	i2=int(si2)
	if (i1 < i2):
		return -1
	elif (i1 > i2):
		return 1
	else:
		if (ss1[1] < ss2[1]):
			return -1
		elif (ss1[1] > ss2[1]):
			return 1
		else:
			if (len(ss1) < len(ss2)):
				return -1
			elif (len(ss1) > len(ss2)):
				return 1
			else:
				j1=int(ss1[3])
				j2=int(ss2[3])
				if (j1 < j2):
					return -1
				elif (j1 > j2):
					return 1
				else:
					return 0

def convert_time_format(ss):
	ll=ss.replace("h"," ").replace("m"," ").replace("s"," ").split()
	tt=0
	for i in range(len(ll)):
		tt+=float(ll[i])*pow(60, len(ll)-i-1)
	return tt
	
def read_time_output(fnm):
	try:
		fin=open(fnm).readlines()
	except:	
		return None
	nn = 36 #magic number, number of lines for whole filename 
	nt = 5 #magic number, number of lines per one NMC
	num = len(fin)/nn
	it = (nn-1)/5
	if (len(fin)%nn >0 ):
		return num
	dd={}
	for i in range(num):
		reallist=[]
		userlist=[]
		syslist=[]
		Nmclist=[]
		f=fin[i*nn].strip()
		for j in range(it):
			nmc=int(fin[i*nn+j*nt	+1])
			Nmclist.append(nmc)
			r=fin[i*nn+j*nt +2].split()[1]
			u=fin[i*nn+j*nt +3].split()[1]
			s=fin[i*nn+j*nt +4].split()[1]
			reallist.append(convert_time_format(r))
			userlist.append(convert_time_format(u))
			syslist.append(convert_time_format(s))
		dd[f] = [Nmclist, reallist, userlist,syslist]
	return dd
			
	
def read_hertz_simple_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		Er=float(content[9].split()[1])
		num=int(content[13].split()[1])
		R2=float(content[15].split()[1])
		low=float(content[12].split()[2].replace("[",""))
		up=float(content[12].split()[4].replace("]",""))
		return {'low':low, 'up':up, 'num':num, 'Er':Er, 'R2': R2 }

def read_hertz_unc_file(filename, ncontact):
	with open(filename) as fpy:
		content=fpy.readlines()
		uEr=float(content[16].split()[2])
		shiftc=range(-ncontact,ncontact+1)
		Ec=[]
		Hc=[]
		Ac=[]
		offset=32
		iE0=offset+3+ncontact
		for i in range(-ncontact,ncontact+1):
			Ec.append(float(content[iE0+i].split()[2].replace("not", "nan")))		

		return { 'GUMEr': uEr, 'contactEr':Ec}

def read_hertz_mc_file(filename):
	#NIGET doesn't produce properly
	return
	with open(filename) as fpy:
		content=fpy.readlines()
		Ap=float(content[19].split()[2].replace("(",""))
		uAp=float(content[19].split()[4].replace(")",""))
		Hit=float(content[20].split()[2].replace("(",""))
		uHit=float(content[20].split()[4].replace(")",""))
		Er=float(content[21].split()[2].replace("(",""))
		uEr=float(content[21].split()[4].replace(")",""))
		return {'mcavgAp': Ap, 'mcstdAp': uAp,'mcavgHit': Hit, 'mcstdHit': uHit, 'mcavgEr': Er, 'mcstdEr': uEr}
		
def read_hertz_odr_simple_file(filename):
	if (filename.find("_R_") > -1) :
		return read_hertz_odr_simple_R_file(filename)
	elif (filename.find("_Er_") > -1) : 
		return read_hertz_odr_simple_Er_file(filename)
	elif (filename.find("_Eit_") > -1) : 
		return read_hertz_odr_simple_Eit_file(filename)
	else:
	 	return None


def read_hertz_odr_simple_R_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		R2=float(content[16].split()[1])
		chi2=float(content[18].split()[1])
		Er=float(content[19].split()[1])
		h0=float(content[12].split()[1])
		n=float(content[13].split()[1])
		num=int(content[23].split()[1])
		low=float(content[22].split()[2].replace("[",""))
		up=float(content[22].split()[4].replace("]",""))
		return {'low':low, 'up':up, 'num':num, 'Er':Er, 'R2': R2, 'chi2':chi2, 'h0':h0, 'n':n }

def read_hertz_odr_simple_Er_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		R2=float(content[12].split()[1])
		chi2=float(content[14].split()[1])
		radius=float(content[15].split()[2])
		h0=float(content[9].split()[1])
		n=float(content[10].split()[1])
		num=int(content[18].split()[1])
		low=float(content[17].split()[2].replace("[",""))
		up=float(content[17].split()[4].replace("]",""))
		return {'low':low, 'up':up, 'num':num, 'radius':radius, 'R2': R2, 'chi2':chi2, 'h0':h0, 'n':n }

def read_hertz_odr_simple_Eit_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		R2=float(content[15].split()[1])
		chi2=float(content[17].split()[1])
		radius=float(content[18].split()[2])
		h0=float(content[12].split()[1])
		n=float(content[13].split()[1])
		num=int(content[21].split()[1])
		low=float(content[20].split()[2].replace("[",""))
		up=float(content[20].split()[4].replace("]",""))
		return {'low':low, 'up':up, 'num':num, 'radius':radius, 'R2': R2, 'chi2':chi2, 'h0':h0, 'n':n }



def read_hertz_odr_unc_file(filename, ncontact):
	with open(filename) as fpy:
		content=fpy.readlines()
		uEr=float(content[17].split()[2])
		shiftc=range(-ncontact,ncontact+1)
		offset=33
		iE0=offset+3+ncontact
		Ec=[]
		for i in range(-ncontact,ncontact+1):
			Ec.append(float(content[iE0+i].split()[2].replace("not", "nan")))		

		return { 'GUMEr': uEr, 'contactEr':Ec}


def read_hertz_odr_mc_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		Er=float(content[17].split()[2].replace("(",""))
		uEr=float(content[17].split()[4].replace(")",""))
		return { 'mcavgEr': Er, 'mcstdEr': uEr}
		

def create_hertz_odr_mc_filename(filename, Nmc, low,up, mode,var, fit):
	if (mode == "R"):
		return filename+"__hertz_odr_mc%(n)d_range_%(l).2g_%(u).2g_nm_R_%(r).2g_fit%(f)d.dat" %{'n':Nmc,'l':low, 'u':up, 'r':var, 'f':fit}
	elif (mode == "Er"):
		return filename+"__hertz_odr_mc%(n)d_range_%(l).2g_%(u).2g_nm_Er_%(r).2g_fit%(f)d.dat" %{'n':Nmc,'l':low, 'u':up, 'r':var, 'f':fit}
	elif (mode == "Eit"):
		return filename+"__hertz_odr_mc%(n)d_range_%(l).2g_%(u).2g_nm_Eit_%(r).2g_fit%(f)d.dat" %{'n':Nmc,'l':low, 'u':up, 'r':var, 'f':fit}
	else:
		return None
		
def create_hertz_odr_unc_filename(filename, low,up, mode,var, fit, uvar):
	if (mode == "R"):
		return filename+"__hertz_odr_unc_range_%(l).2g_%(u).2g_nm_R_%(r).2g_fit%(f)d_uR_%(ur).2g.dat" %{'l':low, 'u':up, 'r':var, 'f':fit, 'ur':uvar}
	elif (mode == "Er"):
		return filename+"__hertz_odr_unc_range_%(l).2g_%(u).2g_nm_Er_%(r).2g_fit%(f)d_uEr_%(ur).2g.dat" %{'l':low, 'u':up, 'r':var, 'f':fit, 'ur':uvar}
	elif (mode == "Eit"):
		return filename+"__hertz_odr_unc_range_%(l).2g_%(u).2g_nm_Eit_%(r).2g_fit%(f)d_uEit_%(ur).2g.dat" %{'l':low, 'u':up, 'r':var, 'f':fit, 'ur':uvar}
	else:
		return None
		
def create_hertz_odr_simple_filename(filename, low,up,mode, var, fit):
	if (mode == "R"):
		return filename+"__hertz_odr_range_%(l).2g_%(u).2g_nm_R_%(r).2g_fit%(f)d.dat" %{'l':low, 'u':up, 'r':var, 'f':fit}
	elif (mode == "Er"):
		return filename+"__hertz_odr_range_%(l).2g_%(u).2g_nm_Er_%(r).2g_fit%(f)d.dat" %{'l':low, 'u':up, 'r':var, 'f':fit}
	elif (mode == "Eit"):
		return filename+"__hertz_odr_range_%(l).2g_%(u).2g_nm_Eit_%(r).2g_fit%(f)d.dat" %{'l':low, 'u':up, 'r':var, 'f':fit}
	else:
		return None
	

def create_hertz_mc_filename(filename, Nmc, low,up, mode , var):
	if (mode == "R"):
		return filename+"__hertz_mc%(n)d_range_%(l).2g_%(u).2g_nm_R_%(r).2g.dat" %{'n':Nmc,'l':low, 'u':up, 'r':var}
	elif (mode == "Er"):
		return filename+"__hertz_mc%(n)d_range_%(l).2g_%(u).2g_nm_Er_%(r).2g.dat" %{'n':Nmc,'l':low, 'u':up, 'r':var}
	elif (mode == "Eit"):
		return filename+"__hertz_mc%(n)d_range_%(l).2g_%(u).2g_nm_Eit_%(r).2g.dat" %{'n':Nmc,'l':low, 'u':up, 'r':var}
	else:
		return None
		
def create_hertz_unc_filename(filename, low,up, mode, var, uvar):
	if (mode == "R"):
		return filename+"__hertz_unc_range_%(l).2g_%(u).2g_nm_R_%(r).2g_uR_%(ur).2g.dat" %{'l':low, 'u':up, 'r':var, 'ur':uvar}
	elif (mode == "Er"):
		return filename+"__hertz_unc_range_%(l).2g_%(u).2g_nm_Er_%(r).2g_uEr_%(ur).2g.dat" %{'l':low, 'u':up, 'r':var, 'ur':uvar}
	elif (mode == "Eit"):
		return filename+"__hertz_unc_range_%(l).2g_%(u).2g_nm_Eit_%(r).2g_uEit_%(ur).2g.dat" %{'l':low, 'u':up, 'r':var, 'ur':uvar}
	else:
		return None

		
def create_hertz_simple_filename(filename, low,up, mode, var):
	if (mode == "R"):
		return filename+"__hertz_range_%(l).2g_%(u).2g_nm_R_%(r).2g.dat" %{'l':low, 'u':up, 'r':var}
	elif (mode == "Er"):
		return filename+"__hertz_range_%(l).2g_%(u).2g_nm_Er_%(r).2g.dat" %{'l':low, 'u':up, 'r':var}
	elif (mode == "Eit"):
		return filename+"__hertz_range_%(l).2g_%(u).2g_nm_Eit_%(r).2g.dat" %{'l':low, 'u':up, 'r':var}
	else:
		return None
	
	

def  get_op_odr_range(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[2].split()
		low = float(line[0])
		up = float(line[1])
		return [low, up]


def get_op_odr_ncontact(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		#find lines "Changes of the blabla corresponding to changes in the contact point"
		ii=[content.index(line) for line in content if line.find("Changes")>-1]
		ni=len(ii)
		zi=[ ii[j+1] - ii[j] for j in range(ni-1)]
		if (max(zi)-min(zi))>0:
			return None
		else:
			return (max(zi)-4 -1)/2 #should be separated by 3 lines of text, 1 empty line, 2*ncontact+1 lines of data


def get_hertz_range(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[3].split()
		low = float(line[0])
		up = float(line[1])
		return [low, up]


def get_hertz_mode_val(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[1].split()
		mode = line[0]
		line= content[2].split()
		val = float(line[0])
		return [hertz_modes[mode], val]


def get_hertz_ncontact(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		#find lines "Changes of the blabla corresponding to changes in the contact point"
		ii=[content.index(line) for line in content if line.find("Changes")>-1]
		ni=len(ii)
		zi=[ ii[j+1] - ii[j] for j in range(ni-1)]
		if (max(zi)-min(zi))>0:
			return None
		else:
			return (max(zi)-4 -1)/2 #should be separated by 3 lines of text, 1 empty line, 2*ncontact+1 lines of data

def get_hertz_mode_val_unc(filename, mode):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[3].split()
		if (mode == "R"):
			val = float(line[0])
		elif (mode == "Er"):
			val = float(line[1])
		elif (mode == "Eit"):
			val = float(line[2])
		else:
			val = None
		return val



def get_hertz_odr_range(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[3].split()
		low = float(line[0])
		up = float(line[1])
		return [low, up]


def get_hertz_odr_mode_val_fit(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[1].split()
		mode = line[0]
		line= content[2].split()
		val = float(line[0])
		line = content[4].split()
		fit = int(line[0])
		return [hertz_modes[mode], val, fit]


def get_hertz_odr_ncontact(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		#find lines "Changes of the blabla corresponding to changes in the contact point"
		ii=[content.index(line) for line in content if line.find("Changes")>-1]
		ni=len(ii)
		zi=[ ii[j+1] - ii[j] for j in range(ni-1)]
		if (max(zi)-min(zi))>0:
			return None
		else:
			return (max(zi)-4 -1)/2 #should be separated by 3 lines of text, 1 empty line, 2*ncontact+1 lines of data

def get_hertz_odr_mode_val_unc(filename, mode):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[3].split()
		if (mode == "R"):
			val = float(line[0])
		elif (mode == "Er"):
			val = float(line[1])
		elif (mode == "Eit"):
			val = float(line[2])
		else:
			val = None
		return val

def  get_slopes_ranges_fit(filename):
	with open(filename) as fpy:
		content = fpy.readlines()
		line= content[2].split()
		loadlow = float(line[0])
		loadup = float(line[1])
		line= content[3].split()
		fitloadexp = int(line[0])
		line= content[5].split()
		unloadlow = float(line[0])
		unloadup = float(line[1])
		return [loadlow, loadup, unloadlow, unloadup, fitloadexp]


def create_slopes_simple_filename(filename, loadlow, loadup, fitloadexp, unloadlow, unloadup):
	return filename+"__slopes_loadrange_%(ll).2g_%(lu).2g_%%_fit%(f)d_unloadrange_%(ul).2g_%(uu).2g_%%.dat" %{'ll':loadlow, 'lu':loadup, 'f':fitloadexp, 'ul':unloadlow, 'uu':unloadup }
		

def create_slopes_unc_filename(filename, loadlow, loadup, fitloadexp, unloadlow, unloadup):
	return filename+"__slopes_unc_loadrange_%(ll).2g_%(lu).2g_%%_fit%(f)d_unloadrange_%(ul).2g_%(uu).2g_%%.dat" %{'ll':loadlow, 'lu':loadup, 'f':fitloadexp, 'ul':unloadlow, 'uu':unloadup }
		

def read_slopes_simple_file(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		hmax=float(content[12].split()[1])
		Fmax=float(content[13].split()[1])
		gamma = float(content[15].split()[1])
		h0 = float(content[16].split()[1])
		n = float(content[17].split()[1])
		R2load = float(content[19].split()[1])
		chi2load = float(content[22].split()[1])
		alpha=float(content[24].split()[1])
		hp=float(content[25].split()[1])
		m=float(content[26].split()[1])
		eps=float(content[32].split()[1])
		R2unload=float(content[29].split()[1])
		chi2unload=float(content[31].split()[1])
		Sload=float(content[33].split()[1])
		Sunload=float(content[34].split()[1])
		Ap=float(content[35].split()[1])
		Hit=float(content[36].split()[1])
		Er=float(content[37].split()[1])
		numload=int(content[43].split()[1])
		lowload=float(content[41].split()[3].replace("[",""))
		upload=float(content[41].split()[5].replace("]",""))
		numunload=int(content[43+8+numload].split()[1])
		lowunload=float(content[41+8+numload].split()[3].replace("[",""))
		upunload=float(content[41+8+numload].split()[5].replace("]",""))
		return {'loadlow':lowload, 'loadup':upload, 'numload':numload,'unloadlow':lowunload, 'unloadup':upunload, 'numunload':numunload, 'Fmax':Fmax, 'hmax':hmax, 'Er':Er, 'Hit':Hit, 'Aphc':Ap, 'R2load':R2load, 'chi2load':chi2load,  'Sload':Sload, 'm':m, 'hp':hp, 'eps':eps, 'R2unload':R2unload, 'chi2unload':chi2unload,  'Sunload':Sunload, 'h0':h0, 'gamma':gamma, 'alpha':alpha, 'n':n}

def read_slopes_unc_file(filename, ncontact):
	with open(filename) as fpy:
		content=fpy.readlines()

		uh=float(content[2].split()[2])
		uF=float(content[3].split()[2])

		un=float(content[23].split()[2])
		uSload=float(content[24].split()[2])
		um=float(content[25].split()[2])
		uSunload=float(content[26].split()[2])
		uAp=float(content[27].split()[2])
		uHit=float(content[28].split()[2])
		uEr=float(content[29].split()[2])
		uEit=float(content[30].split()[2])

		shiftc=range(-ncontact,ncontact+1)
		Ec=[]
		Hc=[]
		Ac=[]
		offset=68
		iE0=offset+3+ncontact
		iH0=iE0+5+2*ncontact
		iA0=iH0+5+2*ncontact
		for i in range(-ncontact,ncontact+1):
			Ec.append(float(content[iE0+i].split()[2]))		
			Hc.append(float(content[iH0+i].split()[2]))		
			Ac.append(float(content[iA0+i].split()[2]))		

		return {'uh': uh, 'uF':uF,
			'GUMAp':uAp, 'GUMHit': uHit, 'GUMEr': uEr, 
			'GUMSload':uSload, 'GUMSunload': uSunload, 
			'GUMn': un, 'GUMm':um, 
			'contactEr':Ec, 'contactHit':Hc, 'contactAp':Ac}


def eval_A(x, p):
	A = 24.5*x**2
	for i in range(len(p)): 
	    A += p[i]*numpy.power(x, numpy.power(2., -i ))
        return A

def eval_A_der(x, p):	
	dA = 24.5*2*x
	for i in range(len(p)): 
	    dA += p[i]*numpy.power(2., -i )*numpy.power(x, numpy.power(2., -i )-1)
        return dA

def eval_uA(x, p, ux, covp):
	uAux = eval_A_der(x,p)*ux

	dAdp = numpy.zeros(len(p))
	for i in range(len(p)): 
	    dAdp[i] = numpy.power(x, numpy.power(2., -i ))

#	print dAdp    
#	uAup = numpy.dot(numpy.dot(dAdp, covp), dAdp)     
	uAup = 0 
#	print covp
	for i in range(len(p)): 
	    for j in range(len(p)): 
		    uAup += covp[i][j]*dAdp[i]*dAdp[j]

        return [uAux, uAup]

def read_area_stats(filename):
	with open(filename) as fpy:
		content=fpy.readlines()
		par = numpy.array(map(float, content[1].split()))
		np = len(par)
		covpar = numpy.zeros((np, np))
		for i in range(np):
			covpar[i,:] = map(float, content[4+i].split())
		
		return par, covpar	
