#!/usr/bin/python

import os, subprocess, time, re, numpy, shutil

shutil.copyfile("../niget-evaluation-scripts/commonfunctions.py", "commonfunctions.py")
import commonfunctions as cf

execfnm="../build/niget-cmd"



Nmc=1000
threshold = 0.01
areafile="Bk7AA10061523.ara"
inputfile="niget_format.dat"
opodrsett="settings.opodr"
Rhertzsett="settings.hertz.R"
Rhertzodrsett="settings.hertzodr.R"
slopessett="settings.slopes"
uncsett="settings.unc"

odrbench ="benchmark_op_odr_niget_format_20_98.dat"
Rhertzbench ="benchmark_hertz_niget_format_20_120_R_1e-7.dat"
Rhertzodrbench ="benchmark_hertz_odr_niget_format_20_120_R_1e-7_fit0.dat"
slopesbench ="benchmark_slopes_niget_format_70_98_1_20_98.dat"
odruncbench ="benchmark_op_odr_unc_niget_format_20_98.dat"
Rhertzuncbench ="benchmark_hertz_unc_niget_format_20_120_R_1e-7_uR_2e-8.dat"
Rhertzodruncbench ="benchmark_hertz_odr_unc_niget_format_20_120_R_1e-7_fit0_uR_2e-8.dat"
slopesuncbench ="benchmark_slopes_unc_niget_format_70_98_1_20_98.dat"

def remove_results():
    for f in os.listdir(os.getcwd()):
        if f.find("_op_odr_range_")> -1:
            os.remove(f)
        if f.find("_hertz_odr_range_")> -1:
            os.remove(f)
        if f.find("_hertz_range_")> -1:
            os.remove(f)
        if f.find("_slopes_range_") > -1:
            os.remove(f)
        if f.find("_op_odr_unc_range_")> -1:
            os.remove(f)
        if f.find("_hertz_odr_unc_range_")> -1:
            os.remove(f)
        if f.find("_hertz_unc_range_")> -1:
            os.remove(f)
        if f.find("_op_odr_mc_range_")> -1:
            os.remove(f)
        if f.find("_hertz_odr_mc_range_")> -1:
            os.remove(f)
        if f.find("_hertz_mc_range_")> -1:
            os.remove(f)
        if f.endswith(".rpt"):    
            os.remove(f)
        if f.endswith(".err"):    
            os.remove(f)


def remove_logs():
    [os.remove(f) for f in os.listdir(os.getcwd()) if f.find("fit.log.")> -1]



def run_test_opodr(inputfile, areafile, settfile, uncfile, Nmc):
    cmd=[execfnm, "-f", inputfile, "-d", settfile, "-U", uncfile,"-a", areafile, "-M", str(Nmc)]
    out=open("out_opodr_"+inputfile, "w");
    proc = subprocess.Popen(cmd, stdout=out)
    return proc

def run_test_hertz(inputfile, settfile, uncfile, Nmc):
    cmd=[execfnm, "-f", inputfile, "-z", settfile, "-U", uncfile, "-M", str(Nmc)]
    out=open("out_hertz_"+inputfile, "w");
    proc = subprocess.Popen(cmd, stdout=out)
    return proc

def run_test_hertz_odr(inputfile, settfile, uncfile, Nmc):
    cmd=[execfnm, "-f", inputfile, "-t", settfile, "-U", uncfile, "-M", str(Nmc)]
    out=open("out_hertz_odr_"+inputfile, "w");
    proc = subprocess.Popen(cmd, stdout=out)
    return proc

def run_test_slopes(inputfile, settfile, uncfile, Nmc):
    cmd=[execfnm, "-f", inputfile, "-s", settfile, "-U", uncfile, "-M", str(Nmc)]
    out=open("out_slopes_"+inputfile, "w");
    proc = subprocess.Popen(cmd, stdout=out)
    return proc



def check_did_it_run(outputfile):
    didit = False
    with open(outputfile) as fin:
        txt = fin.readlines()
        if (len(txt) < 10):
            didit = False
        else:
            didit = True
    return didit        

def check_rpt_err(basename):
    if (not os.path.isfile(basename+".err")):
        print basename+".err file was not created"
        return False
    if (not os.path.isfile(basename+".rpt")):
        print basename +".rpt file was not created"
        return False
    return True
    




if __name__ ==  '__main__':
    # remove previous results
    remove_results()

    #test 1  OP ODR
    print '---------------- TEST #1 --------------------------------------'
    print '---------------- OP ODR  --------------------------------------'
    proc = run_test_opodr(inputfile, areafile, opodrsett, uncsett, 0)
    proc.communicate()

    while (True):
        didit = check_did_it_run("out_opodr_"+inputfile)
        if (not didit):
            break

        #check presence of rpt and err file
        rpterr = check_rpt_err("fit.log.op")
        if (not rpterr):
            break

        #check results
        [low, up] = cf.get_op_odr_range(opodrsett)
        fnm = cf.create_op_odr_simple_filename(inputfile, low, up)
        if (os.path.isfile(fnm)):
            outdata = cf.read_op_odr_simple_file(fnm)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(odrbench)):
            benchdata = cf.read_op_odr_simple_file(odrbench)
        else:
            print "Could not find benchmark data " + odrbench
            break
        
        alldiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in benchdata.keys() if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in alldiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(alldiffs) >0):
            break
        
        print "Test #1 OP ODR  successfull"
        break
    print 
    print 
        

        

    #test 2  OP ODR Uncertainties
    print '---------------- TEST #2 --------------------------------------'
    print '---------------- OP ODR uncertainties -------------------------'

    while (True):
        #some tests already part of TEST #1
        if not didit:
            break

        #check results
        fnm = cf.create_op_odr_unc_filename(inputfile, low, up)
        ncontact = cf.get_op_odr_ncontact(fnm)
        if (os.path.isfile(fnm)):
            outdata = cf.read_op_odr_unc_file(fnm, ncontact)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(odruncbench)):
            benchdata = cf.read_op_odr_unc_file(odruncbench, ncontact)
        else:
            print "Could not find benchmark data " + odruncbench
            break
        
        isok = True
        contactkeys = [k for k in benchdata.keys() if  k.startswith("contact")]
        GUMkeys = [ k for k in benchdata.keys() if k not in contactkeys]
        GUMdiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in GUMkeys if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in GUMdiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(GUMdiffs) >0):
            isok = False
        contactdiffs = [ (k,[(benchdata[k][i] - outdata[k][i])/outdata[k][i]  for i in range(-ncontact, ncontact+1)]) for k in contactkeys ]
        for (x,L) in contactdiffs:
            for i in range(-ncontact, ncontact+1):
                y = L[i]
                if abs(y) > threshold:
                    isok = False
                    print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " % at contact shift "+ str(i)
        if not isok:
            break
            
        print "Test #2 OP ODR uncertainties  successfull"
        break
    print 
    print 


    #test 3  Hertz R mode
    print '---------------- TEST #3 --------------------------------------'
    print '---------------- Hertz   --------------------------------------'
    proc = run_test_hertz(inputfile, Rhertzsett, uncsett, 0)
    proc.communicate()

    while (True):
        didit = check_did_it_run("out_hertz_"+inputfile)
        if (not didit):
            break

        #check results
        #find lower and upper range from settings 
        [low, up] = cf.get_hertz_range(Rhertzsett)
        [mode, var] = cf.get_hertz_mode_val(Rhertzsett)
        fnm = cf.create_hertz_simple_filename(inputfile, low, up, mode ,var)
        if (os.path.isfile(fnm)):
            outdata = cf.read_hertz_simple_file(fnm)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(Rhertzbench)):
            benchdata = cf.read_hertz_simple_file(Rhertzbench)
        else:
            print "Could not find benchmark data " + Rhertzbench
            break
        
        alldiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in benchdata.keys() if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in alldiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(alldiffs) >0):
            break
        
        print "Test #3 Hertz  successfull"
        break
    print 
    print 

        

    #test 4  Hertz Uncertainties
    print '---------------- TEST #4 --------------------------------------'
    print '---------------- Hertz uncertainties -------------------------'

    while (True):
        #some tests already part of TEST #3
        if not didit:
            break

        #check results
        uvar = cf.get_hertz_mode_val_unc(uncsett, mode)
        fnm = cf.create_hertz_unc_filename(inputfile, low, up, mode, var, uvar)
        if (os.path.isfile(fnm)):
            outdata = cf.read_hertz_unc_file(fnm, ncontact)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(Rhertzuncbench)):
            benchdata = cf.read_hertz_unc_file(Rhertzuncbench, ncontact)
        else:
            print "Could not find benchmark data " + Rhertzuncbench
            break
        
        isok = True
        contactkeys = [k for k in benchdata.keys() if  k.startswith("contact")]
        GUMkeys = [ k for k in benchdata.keys() if k not in contactkeys]
        GUMdiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in GUMkeys if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in GUMdiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(GUMdiffs) >0):
            isok = False
        contactdiffs = [ (k,[(benchdata[k][i] - outdata[k][i])/outdata[k][i]  for i in range(-ncontact, ncontact+1)]) for k in contactkeys ]
        for (x,L) in contactdiffs:
            for i in range(-ncontact, ncontact+1):
                y = L[i]
                if abs(y) > threshold:
                    isok = False
                    print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " % at contact shift "+ str(i)
        if not isok:
            break
            
        print "Test #4 Hertz uncertainties  successfull"
        break
    print 
    print 


    #test 5  Hertz ODR R mode
    print '---------------- TEST #5 --------------------------------------'
    print '---------------- Hertz ODR  -----------------------------------'
    proc = run_test_hertz_odr(inputfile, Rhertzodrsett, uncsett, 0)
    proc.communicate()

    while (True):
        didit = check_did_it_run("out_hertz_odr_"+inputfile)
        if (not didit):
            break

        #check presence of rpt and err file
        rpterr = check_rpt_err("fit.log.hz")
        if (not rpterr):
            break

        #check results
        #find lower and upper range from settings 
        [low, up] = cf.get_hertz_odr_range(Rhertzodrsett)
        [mode, var, fit] = cf.get_hertz_odr_mode_val_fit(Rhertzodrsett)
        fnm = cf.create_hertz_odr_simple_filename(inputfile, low, up, mode ,var,fit)
        if (os.path.isfile(fnm)):
            outdata = cf.read_hertz_odr_simple_file(fnm)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(Rhertzodrbench)):
            benchdata = cf.read_hertz_odr_simple_file(Rhertzodrbench)
        else:
            print "Could not find benchmark data " + Rhertzodrbench
            break
        
        alldiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in benchdata.keys() if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in alldiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(alldiffs) >0):
            break
        
        print "Test #5 Hertz ODR successfull"
        break
    print 
    print 
        

        

    #test 6  Hertz ODR Uncertainties
    print '---------------- TEST #6 --------------------------------------'
    print '---------------- Hertz ODR uncertainties ----------------------'

    while (True):
        #some tests already part of TEST #5
        if not didit:
            break
        if not outdata:
            break

        #check results
        uvar = cf.get_hertz_odr_mode_val_unc(uncsett, mode)
        fnm = cf.create_hertz_odr_unc_filename(inputfile, low, up, mode, var, fit, uvar)
        if (os.path.isfile(fnm)):
            outdata = cf.read_hertz_odr_unc_file(fnm, ncontact)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(Rhertzodruncbench)):
            benchdata = cf.read_hertz_odr_unc_file(Rhertzodruncbench, ncontact)
        else:
            print "Could not find benchmark data " + Rhertzodruncbench
            break
        
        isok = True
        contactkeys = [k for k in benchdata.keys() if  k.startswith("contact")]
        GUMkeys = [ k for k in benchdata.keys() if k not in contactkeys]
        GUMdiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in GUMkeys if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in GUMdiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(GUMdiffs) >0):
            isok = False
        contactdiffs = [ (k,[(benchdata[k][i] - outdata[k][i])/outdata[k][i]  for i in range(-ncontact, ncontact+1)]) for k in contactkeys ]
        for (x,L) in contactdiffs:
            for i in range(-ncontact, ncontact+1):
                y = L[i]
                if abs(y) > threshold:
                    isok = False
                    print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " % at contact shift "+ str(i)
        if not isok:
            break
            
        print "Test 6 Hertz uncertainties ODR successfull"
        break
    print 
    print 


    #test 7  Two slopes
    print '---------------- TEST #7 --------------------------------------'
    print '---------------- Two slopes -----------------------------------'
    proc = run_test_slopes(inputfile, slopessett, uncsett, 0)
    proc.communicate()

    while (True):
        didit = check_did_it_run("out_slopes_"+inputfile)
        if (not didit):
            break

        #check presence of rpt and err file
        rpterr = check_rpt_err("fit.log.load")
        if (not rpterr):
            break

        #check presence of rpt and err file
        rpterr = check_rpt_err("fit.log.unload")
        if (not rpterr):
            break

        #check results
        #find lower and upper range from settings 
        [loadlow, loadup, unloadlow, unloadup, fitloadexp] = cf.get_slopes_ranges_fit(slopessett)
        fnm = cf.create_slopes_simple_filename(inputfile, loadlow, loadup, fitloadexp, unloadlow, unloadup)
        if (os.path.isfile(fnm)):
            outdata = cf.read_slopes_simple_file(fnm)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(slopesbench)):
            benchdata = cf.read_slopes_simple_file(slopesbench)
        else:
            print "Could not find benchmark data " + slopesbench
            break
        
        alldiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in benchdata.keys() if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in alldiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(alldiffs) >0):
            break
        
        print "Test #7 Two slopes successfull"
        break
    print 
    print 
        

        

    #test 8  Two Slopes Uncertainties
    print '---------------- TEST #8 --------------------------------------'
    print '---------------- Two slopes uncertainties ---------------------'

    while (True):
        #some tests already part of TEST #7
        if not didit:
            break
        if not outdata:
            break

        #check results
        fnm = cf.create_slopes_unc_filename(inputfile, loadlow, loadup, fitloadexp, unloadlow, unloadup)
        if (os.path.isfile(fnm)):
            outdata = cf.read_slopes_unc_file(fnm, ncontact)
        else:
            print "Output file "+fnm+" was not created."
            break

        if (os.path.isfile(slopesuncbench)):
            benchdata = cf.read_slopes_unc_file(slopesuncbench, ncontact)
        else:
            print "Could not find benchmark data " + slopesuncbench
            break
        
        isok = True
        contactkeys = [k for k in benchdata.keys() if  k.startswith("contact")]
        GUMkeys = [ k for k in benchdata.keys() if k not in contactkeys]
        GUMdiffs =[ (k, (benchdata[k] - outdata[k])/outdata[k]) for k in GUMkeys if abs(benchdata[k] - outdata[k])/outdata[k] > threshold ]
        for (x,y) in GUMdiffs:
            print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " %"
        if (len(GUMdiffs) >0):
            isok = False
        contactdiffs = [ (k,[(benchdata[k][i] - outdata[k][i])/outdata[k][i]  for i in range(-ncontact, ncontact+1)]) for k in contactkeys ]
        for (x,L) in contactdiffs:
            for i in range(-ncontact, ncontact+1):
                y = L[i]
                if abs(y) > threshold:
                    isok = False
                    print "Result "+ x + " differs by " + str(numpy.ceil(y*100)) + " % at contact shift "+ str(i)
        if not isok:
            break
            
        print "Test 8  Two slopes uncertainties  successfull"
        break
    print 
    print 

    remove_logs();



