#ifndef NOFORTRAN

#include "tool_hertz_odr_unc.h"
#include "tool_hertz_odr_mc.h"
#include "tool_hertz_odr.h"

#include "controls.h"
#include "datatypes.h"
#include "fddata.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "indent-toolbox-common.h"

#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>


#define CONTACTMAX 3
#define MC_WHEN_WARN 1e2
#define N_FIX 1.5

static const gchar *logfnm = "fit.log.hz.unc";

void hertz_odr_fit_shift_contact(HertzODRdata *hzodrdata, FDdata *fddata, Instdata *instdata, gdouble *ERc, gint j);

void init_shifted_HertzODRdata(HertzODRdata *src, HertzODRdata *dest, gdouble dh)
{
	init_HertzODRdata(dest);

	// shift integration range by dh

	dest->range[0]= src->range[0]-dh;
	dest->range[1]= src->range[1]-dh;
	dest->from = dest->range[0];
	dest->to = dest->range[1];

	dest->mode = src->mode;
	switch(dest->mode){
		case R_MODE:
			dest->radius = src->radius;
			break;
		case ER_MODE:
			dest->Er = src->Er;
			break;
		case EIT_MODE:
			dest->Eit = src->Eit;
			break;
	}

	dest->ifixb[0] = src->ifixb[0];
	dest->ifixb[1] = src->ifixb[1];
	dest->ifixb[2] = src->ifixb[2];

	dest->logfnm = create_log_filename_path(logfnm);
}

void 
hertz_odr_propagate_uncertainties_calc (Args *args)
{
	HertzODRUncdata *unc;
	HertzODRdata *hzodr;
	FDdata *fd;
	Instdata *inst;
	gdouble R,Er,Eit, nu,nui,Ei;
	gdouble uErunu, uErunui, uEruEi, uEruEit;
	gdouble dradiusdEr ;
	gchar *unc_logfnm;
	gdouble beta[3], std_beta[3], covariance[3];
	gint istart, iend,ndata, info, nremove;
	GwyDataLine *x, *y;
	gdouble ux,uy;
	gdouble uh, uF;
	gdouble ugammah, ugammaF, ugamma;
	gdouble unh, unF, un;
	gdouble SSres, R2, R2adj, chi2;


	unc = &(args->hertzodrunc);
	hzodr = &(args->hertzodrdata);
	inst = &(args->instdata);
	fd = &(args->fddata);

	unc_logfnm =  create_log_filename_path(logfnm);

	printf("logfnm %s \n", unc_logfnm);
	if (VERBOSE)
		printf(" \n\nPropagate uncertainties\n");

	uh = unc->uh;
	uF = unc->uF;

	/* prepare odr fit */

	beta[0] = hzodr->gamma;
	beta[1] = hzodr->h0;
	beta[2] = hzodr->n;


	printf(" fraom %g %g \n", hzodr->from, hzodr->to);
	printf("range %g %g \n", hzodr->range[0], hzodr->range[1]);
	//find appropriate data, which should be fitted
	range_to_indices(hzodr->range[0], hzodr->range[1], fd->hload, FALSE, &istart, &iend, &ndata);

	x = gwy_data_line_part_extract(fd->hload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Fload, istart, ndata);
	//remove negative depths from data
	nremove = filter_negative_data(x, y);
	hzodr->nfitdata -= nremove;

	/* fit with only uh */
	ux = uh;
	uy = 0;
	printf(" ux %g uy %g \n", ux,uy);
	info = odr_run_fit(x, y, beta,hzodr->ifixb, unc_logfnm, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	unh = std_beta[2];
	ugammah = std_beta[0];

	if (VERBOSE)
		printf(" fit with only uh:\n info %d gamma %g n %g h0 %g \n u(gamma) %g u(n) %g u(h0) %g \n",
				info,beta[0], beta[2], beta[1], ugammah, unh, std_beta[1]);

	/* fit with only uF */
	ux = 0;
	uy = uF;
	printf(" ux %g uy %g \n", ux,uy);
	info = odr_run_fit(x, y, beta,hzodr->ifixb, unc_logfnm, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	unF = std_beta[2];
	ugammaF = std_beta[0];

	if (VERBOSE)
		printf(" fit with only uF:\n info %d gamma %g n %g h0 %g \n  u(gamma) %g u(n) %g u(h0) %g \n",
			       info, beta[0], beta[2], beta[1], ugammaF, unF,std_beta[1]);	       

	unc->unuh = unh;
	unc->unuF = unF;
	unc->ugammauh = ugammah;
	unc->ugammauF = ugammaF;


	// total uncertainties
	// should take as given by odr_wrap, otherwise doublecounting
	/* fit with both uh and uF */

	ux = uh;
	uy = uF;
	printf(" ux %g uy %g \n", ux,uy);
	info = odr_run_fit(x, y, beta,hzodr->ifixb, unc_logfnm, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	un = std_beta[2];
	ugamma = std_beta[0];

	unc->un = un;
	unc->ugamma = ugamma;

	if (VERBOSE)
		printf(" fit with both uh, uF:\n info %d gamma %g n %g h0 %g \n  u(gamma) %g u(n) %g u(h0) %g \n",
			       info, beta[0], beta[2], beta[1], ugamma, un,std_beta[1]);	       


	switch(hzodr->mode){
		case R_MODE:
			//make copies to simplify code
			// input 
			Ei = inst->Ei;
			nui = inst->nui;
			nu = inst->nu;
			R = hzodr->radius;
			//output
			Eit = hzodr->Eit;
			Er = hzodr->Er;


			unc->uEruh = ugammah/hzodr->gamma*Er*unc->uh;//Er/a uah 
			unc->uEruF = ugammaF/hzodr->gamma*Er*unc->uF; //Er/a uaF 
			unc->uEruradius = 0.5*Er/R*unc->uradius; // -1/2 *Er/R u(R)

			//uEituh = dEit/ dh uh = dEit / dEr * dEr/dh uh = dEit/dEr * uEruh
			//dEit/dEr = Eit^2/Er^2/ (1-nu^2)

			unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; 
			unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; 
			unc->uEituradius = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruradius; 

			unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*inst->unu); // -2nu/(1-nu^2) Eit u(nu)
			unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *inst->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
			unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/ Ei/Ei *inst->uEi)*1e9; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)


			// total uncertainties
			unc->uErtotal = sqrt(unc->uEruh*unc->uEruh + unc->uEruF*unc->uEruF + unc->uEruradius*unc->uEruradius);
			unc->uEittotal = sqrt(unc->uEituh*unc->uEituh + unc->uEituF*unc->uEituF + unc->uEituradius*unc->uEituradius 
					+unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
			break;
		case  ER_MODE:
			//make copies to simplify code
			// input 
			Er = hzodr->Er;
			// output 
			R = hzodr->radius;


			unc->uradiusuh = R*2*ugammah/hzodr->gamma;// 2 R/a uah 
			unc->uradiusuF = R*2*ugammaF/hzodr->gamma;// 2 R/a uaF 
			unc->uradiusuEr = 2*R/Er*unc->uEr; // -2 *R/Er u(Er)

			// total uncertainties
			unc->uradiustotal = sqrt(unc->uradiusuh*unc->uradiusuh + unc->uradiusuF*unc->uradiusuF + unc->uradiusuEr*unc->uradiusuEr);

			break;
		case EIT_MODE :
			//make copies to simplify code
			// input 
			Ei = inst->Ei;
			nui = inst->nui;
			nu = inst->nu;
			Eit = hzodr->Eit;
			//output
			R = hzodr->radius;
			Er = hzodr->Er;

			uErunu =  2*nu*Er*Er/Eit*inst->unu; // 2*nu/EIT Er^2 unu
			uErunui =  2*nui*Er*Er/Ei*inst->unui*1e9; // 2*nui/Ei Er^2 unui
			uEruEi =  (1-nui*nui)/Ei/Ei*Er*Er*inst->uEi*1e9; // (1-nui^2)/Ei^2 Er^2 uEi
			uEruEit = (1-nu*nu)/Eit/Eit*Er*Er*unc->uEit ; // (1-nu^2)/EIT^2 Er^2 uEIT

			unc->uradiusuh = R*2*ugammah/hzodr->gamma;// 2 R/a uah 
			unc->uradiusuF = R*2*ugammaF/hzodr->gamma;// 2 R/a uaF 
			dradiusdEr = 2*R/Er; // dR/dEr = -2 *R/Er 

			unc->uradiusunu = dradiusdEr*uErunu;
			unc->uradiusunui = dradiusdEr*uErunui;
			unc->uradiusuEi = dradiusdEr*uEruEi;
			unc->uradiusuEit = dradiusdEr*uEruEit;

			// total uncertainties
			unc->uradiustotal = sqrt(unc->uradiusuh*unc->uradiusuh + unc->uradiusuF*unc->uradiusuF + unc->uradiusuEit*unc->uradiusuEit
					+ unc->uradiusuEi*unc->uradiusuEi+ unc->uradiusunu*unc->uradiusunu+ unc->uradiusunui*unc->uradiusunui);

			break;
	}

	g_object_unref(x);
	g_object_unref(y);
	g_free(unc_logfnm);
}

void
hertz_odr_propagate_uncertainties (Data *data)
{
	gchar str[30];
	HertzODRUncControls *ctrls;
	HertzODRUncdata *unc;
	HertzODRdata *hzodr;
	gint i,j;
	GtkWidget *label;

        ctrls =	data->controls->hertzodrunccontrols;
	unc = &(data->args->hertzodrunc);
	hzodr = &(data->args->hertzodrdata);

	if ( !hzodr->has_fit )
		return;

	if ( unc->ERc == NULL)
		return;
	
	// calculate the uncertainties
	hertz_odr_propagate_uncertainties_calc (data->args);

	
	/* show */
	switch(hzodr->mode){
		case  R_MODE:
			// radius
			label_set_gdouble_format(ctrls->uEruradius, unc->uEruradius, MODUL);
			label_set_gdouble_format(ctrls->uEituradius, unc->uEituradius, MODUL);

			//other contributions
			label_set_gdouble_format(ctrls->uEitunu, unc->uEitunu, MODUL);
			label_set_gdouble_format(ctrls->uEitunui, unc->uEitunui, MODUL);
			label_set_gdouble_format(ctrls->uEituEi, unc->uEituEi, MODUL);

			// total uncertainties
			label_set_gdouble_format(ctrls->uErtotal, unc->uErtotal, MODUL);
			label_set_gdouble_format(ctrls->uEittotal, unc->uEittotal, MODUL);

			break;
		case ER_MODE:
			/* show */
			label_set_gdouble_format(ctrls->uradiusuh, unc->uradiusuh * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusuF, unc->uradiusuF * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusuEr, unc->uradiusuEr * 1e9, DEPTH);

			// total uncertainties
			label_set_gdouble_format(ctrls->uradiustotal, unc->uradiustotal * 1e9, DEPTH);

			break;
		case EIT_MODE :
			/* show */
			label_set_gdouble_format(ctrls->uradiusuh, unc->uradiusuh * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusuF, unc->uradiusuF * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusuEi, unc->uradiusuEi * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusuEit, unc->uradiusuEit * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusunui, unc->uradiusunui * 1e9, DEPTH);
			label_set_gdouble_format(ctrls->uradiusunu, unc->uradiusunu * 1e9, DEPTH);

			// total uncertainties
			label_set_gdouble_format(ctrls->uradiustotal, unc->uradiustotal * 1e9, DEPTH);

			break;
	}
	/* contact point uncertainties */

	//destroy the labels of the old table_unc_contact
	remove_table_content(ctrls->table_unc_contact);

	//create new table
	i=0;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("Uncertainties due to choice of contact point"), 0, 4, i,i+1, GTK_FILL, 0, 0, 0);
	i++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("shift contact point"), 0, 1, i,i+1, GTK_FILL, 0, 0, 0);

	if (data->args->hertzodrdata.mode == R_MODE)
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), 
				gwy_label_new_header("E<sub>r</sub>/GPa"), 1, 2, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	else 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), 
				gwy_label_new_header("radius/nm"), 1, 2, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	i++;

	for (j = -CONTACTMAX; j<=CONTACTMAX ;j++){
		label = gtk_label_new(NULL);
		label_set_gint_format(label, j, "%d\n");
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		hertz_odr_fit_shift_contact(&(data->args->hertzodrdata), &(data->args->fddata), &(data->args->instdata), unc->ERc, j);
		label = gtk_label_new(NULL);
		if (unc->ERc[j+CONTACTMAX] > 0)
			if (data->args->hertzodrdata.mode == R_MODE)
				g_snprintf(str, sizeof(str),MODUL, unc->ERc[j+CONTACTMAX]); 
			else
				g_snprintf(str, sizeof(str),DEPTH, unc->ERc[j+CONTACTMAX]*1e9); 
		else
			g_snprintf(str, sizeof(str), "not defined");

		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"1/%d", 2*CONTACTMAX+1); /* co s tim dal? */
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
		i++;

	}
	gtk_widget_show_all(ctrls->table_unc_contact);


}

static void mc_iterations_changed(Data *data)
{
	data->args->hertzodrunc.Nmc = (gint)gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->Nmc));
	if (data->args->hertzodrunc.Nmc > MC_WHEN_WARN){
		label_set_gint_format(data->controls->hertzodrunccontrols->mc_warn, data->args->hertzodrunc.Nmc,
				      "Warning: Monte Carlo calculation for %d iterations may take very long!");
	}
	else{
		label_clear(data->controls->hertzodrunccontrols->mc_warn);
	}
}

static void 
unc_depth_changed (Data *data)
{
	data->args->hertzodrunc.uh = (gint)gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->uh));
	hertz_odr_propagate_uncertainties (data);
}

static void 
unc_load_changed (Data *data)
{
	data->args->hertzodrunc.uF = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->uF));
	hertz_odr_propagate_uncertainties (data);
}

static void 
unc_radius_changed (Data *data)
{
	data->args->hertzodrunc.uradius = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->uradius))*1e-9;
	hertz_odr_propagate_uncertainties (data);
}

static void 
unc_nu_changed (Data *data)
{
	data->args->instdata.unu = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->unu));
	hertz_odr_propagate_uncertainties (data);
}

static void 
unc_nui_changed (Data *data)
{
	data->args->instdata.unui = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->unui));
	hertz_odr_propagate_uncertainties (data);
}

static void 
unc_Ei_changed (Data *data)
{
	data->args->instdata.uEi = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->uEi))*1e9;
	hertz_odr_propagate_uncertainties (data);
}

static void 
unc_Er_changed (Data *data)
{
	data->args->hertzodrunc.uEr = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->uEr));
	hertz_odr_propagate_uncertainties (data);
}


static void 
unc_Eit_changed (Data *data)
{
	data->args->hertzodrunc.uEit = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzodrunccontrols->uEit));
	hertz_odr_propagate_uncertainties (data);
}

gboolean hertz_odr_uncertainty_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	HertzODRdata *hzodrdata;
	HertzODRUncdata *hzodrunc;
	Instdata *instdata;
	gint j;

	hzodrdata = &(args->hertzodrdata);
	hzodrunc = &(args->hertzodrunc);
	instdata = &(args->instdata);

	if (args->fddata.hload->res == 0 || args->fddata.Fload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of HertzODR analysis \n");
	fprintf(file, "# u(depth): %g nm\n", hzodrunc->uh);
	fprintf(file, "# u(load): %g nm\n", hzodrunc->uF);
	fprintf(file, "#fitted range:  [%g, %g ] nm \n", hzodrdata->range[0], hzodrdata->range[1]);
	if (hzodrdata->ifixb[2] == 0)
			fprintf(file, "#n: %g fixed \n", N_FIX);

	switch(hzodrdata->mode){
		case R_MODE:
			fprintf(file, "#R_mode \n");
			fprintf(file, "#tip radius: %g nm       u(tip radius): %g nm\n", hzodrdata->radius*1e9, hzodrunc->uradius*1e9);
			fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, instdata->unu);
			fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, instdata->unui);
			fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, instdata->uEi*1e-9); //TODO units
			fprintf(file, "#\n");

			fprintf(file, "#Results of HertzODR analysis \n");
			fprintf(file, "#Er: %g GPa\n", hzodrdata->Er);
			fprintf(file, "#Eit: %g GPa\n", hzodrdata->Eit);
			break;
		case ER_MODE:
			fprintf(file, "#Er_mode \n");
			fprintf(file, "#Er: %g GPa            u(Er) : %g GPa\n", hzodrdata->Er,hzodrunc->uEr);
			fprintf(file, "#\n");

			fprintf(file, "#Results of HertzODR analysis \n");
			fprintf(file, "#radius: %g nm\n", hzodrdata->radius*1e9);
			break;
		case EIT_MODE:
			fprintf(file, "#Eit_mode \n");
			fprintf(file, "#Eit: %g GPa            u(Eit) : %g GPa\n", hzodrdata->Eit,hzodrunc->uEit);
			fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, instdata->unu);
			fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, instdata->unui);
			fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, instdata->uEi*1e-9); //TODO units
			fprintf(file, "#\n");

			fprintf(file, "#Results of HertzODR analysis \n");
			fprintf(file, "#radius: %g nm\n", hzodrdata->radius*1e9);
			break;
	}
	fprintf(file, "#\n");

	/* Gaussian propagation */
	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	switch(hzodrdata->mode){
		case R_MODE:
			fprintf(file, "#u(Er) =  %g GPa \n", hzodrunc->uErtotal);
			fprintf(file, "#u(Eit) =  %g GPa \n", hzodrunc->uEittotal);
			fprintf(file, "#\n");
			fprintf(file, "#Contributions to uncertainty of Er\n");
			fprintf(file, "#u(Er;uh) = %g GPa \n", hzodrunc->uEruh);
			fprintf(file, "#u(Er;uF) = %g GPa \n", hzodrunc->uEruF);
			fprintf(file, "#u(Er;uradius) = %g GPa \n", hzodrunc->uEruradius);
			fprintf(file, "#\n");
			fprintf(file, "#Contributions to uncertainty of Eit\n");
			fprintf(file, "#u(Eit;uh) = %g GPa \n", hzodrunc->uEituh);
			fprintf(file, "#u(Eit;uF) = %g GPa \n", hzodrunc->uEituF);
			fprintf(file, "#u(Eit;uradius) = %g GPa \n", hzodrunc->uEituradius);
			fprintf(file, "#u(Eit;unu) = %g GPa \n", hzodrunc->uEitunu);
			fprintf(file, "#u(Eit;unui) = %g GPa \n", hzodrunc->uEitunui);
			fprintf(file, "#u(Eit;uEi) = %g GPa \n", hzodrunc->uEituEi);
			fprintf(file, "#\n");
			break;
		case ER_MODE:
			fprintf(file, "#u(radius) =  %g nm \n", hzodrunc->uradiustotal*1e9);
			fprintf(file, "#\n");
			fprintf(file, "#Contributions to uncertainty of radius\n");
			fprintf(file, "#u(radius;uh) = %g nm \n", hzodrunc->uradiusuh*1e9);
			fprintf(file, "#u(radius;uF) = %g nm \n", hzodrunc->uradiusuF*1e9);
			fprintf(file, "#u(uradius;uEr) = %g nm \n", hzodrunc->uradiusuEr*1e9);
			fprintf(file, "#\n");
			break;
		case EIT_MODE:
			fprintf(file, "#u(radius) =  %g nm \n", hzodrunc->uradiustotal*1e9);
			fprintf(file, "#\n");
			fprintf(file, "#Contributions to uncertainty of radius\n");
			fprintf(file, "#u(radius;uh) = %g nm \n", hzodrunc->uradiusuh*1e9);
			fprintf(file, "#u(radius;uF) = %g nm \n", hzodrunc->uradiusuF*1e9);
			fprintf(file, "#u(uradius;uEit) = %g nm \n", hzodrunc->uradiusuEit*1e9);
			fprintf(file, "#u(uradius;unu) = %g nm \n", hzodrunc->uradiusunu*1e9);
			fprintf(file, "#u(uradius;unui) = %g nm \n", hzodrunc->uradiusunui*1e9);
			fprintf(file, "#u(uradius;uEi) = %g nm \n", hzodrunc->uradiusuEi*1e9);
			break;
	}
	fprintf(file, "#\n");


	/* contact point uncertainties */
	switch(hzodrdata->mode){
		case R_MODE:
			fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
			fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   ");
			fprintf(file, "Er/GPa \n");
			for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
				if ( hzodrunc->ERc[j+CONTACTMAX] < 0 )
					fprintf(file, "# %d   not defined \n", j);
				else
					fprintf(file, "# %d   %g  \n", j, hzodrunc->ERc[j+CONTACTMAX]);
			}
			break;
		case ER_MODE:
		case EIT_MODE:
			fprintf(file, "#Changes of the radius corresponding to changes in the contact point \n");
			fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   ");
			fprintf(file, "R/nm \n");
			for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
				if ( hzodrunc->ERc[j+CONTACTMAX] < 0 )
					fprintf(file, "# %d   not defined \n", j);
				else
					fprintf(file, "# %d   %g  \n", j, hzodrunc->ERc[j+CONTACTMAX]*1e9);
			}

			break;
	}
	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}



static void hertz_odr_uncertainty_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;
	FileSaveStatus filestatus;
	gboolean file_ok;
	gint response;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	file_ok = FALSE;
	response = gtk_dialog_run (GTK_DIALOG ( dialog));
	while (  response == GTK_RESPONSE_OK && !file_ok)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog)){
			hertz_odr_uncertainty_write_file(filename, args, &filestatus);
			file_ok = (filestatus == SAVE_FILE_OK);
		}
		g_free(filename);
		if (!file_ok){
			show_save_error(filestatus, filename);
			response = gtk_dialog_run (GTK_DIALOG ( dialog));
		}
	}
	gtk_widget_destroy ( dialog);
}

void
hertz_odr_fit_shift_contact(HertzODRdata *hzodrdata, FDdata *fddata, Instdata *instdata, gdouble *ERc, gint j)
{
	HertzODRdata hzodr;
	FDdata fdc;
	gdouble dh;
	gint istart,iend, ndata;
	gint nremove;
	GwyDataLine *x, *y;

	ERc[j+CONTACTMAX] = -1;

	if (fddata->i_contact_load+j < 0)
		return;

	// create shifted FDdata
	if (VERBOSE)
		printf("\n\nCreate shifted data for j %d \n", j);

	dh = create_shifted_FDdata(fddata, &fdc, j);
	/*
	fdc.i_contact_load = fddata->i_contact_load+j;

	fdc.hall_orig = gwy_data_line_duplicate(fddata->hall_orig);
	fdc.Fall_orig = gwy_data_line_duplicate(fddata->Fall_orig);

	
	fdc.hload = gwy_data_line_part_extract(fdc.hall_orig, fdc.i_contact_load, fddata->hload->res-j);
	fdc.Fload = gwy_data_line_part_extract(fdc.Fall_orig, fdc.i_contact_load, fddata->Fload->res-j);
	gwy_data_line_add(fdc.hload, -fdc.hall_orig->data[fdc.i_contact_load]);
	gwy_data_line_add(fdc.Fload, -fdc.Fall_orig->data[fdc.i_contact_load]);

	//difference in shift between different contactpoints
	dh = fdc.hall_orig->data[fdc.i_contact_load]-fddata->hall_orig->data[fddata->i_contact_load];
	*/
	if (VERBOSE)
		printf("shift dh %g  \n", dh);
	

	// create working copy of HertzODRdat
	init_shifted_HertzODRdata(hzodrdata, &hzodr, dh);

	//check range
	printf(" hzodr.from %g to %g \n", hzodr.from, hzodr.to);
	if (hzodr.from ==  hzodr.to){
		return;
	}

	//find appropriate data, which should be fitted
	range_to_indices(hzodr.from, hzodr.to, fdc.hload, FALSE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, fdc.hload->res - istart);

	if (VERBOSE)
		printf("interval has %d points, from %d to %d, hrange [%g, %g ] nm, Frange [%g, %g] mN \n", ndata, istart, iend,
			       fdc.hload->data[istart], fdc.hload->data[iend], fdc.Fload->data[istart], fdc.Fload->data[iend]);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Fload, istart, ndata);
	nremove = filter_negative_data(x, y);
	hzodr.nfitdata = ndata - nremove;

	hertz_odr_fit(x, y, &hzodr , &fdc, instdata, NULL );

	// clean up
	g_free(hzodr.logfnm);
	g_object_unref(x);
	g_object_unref(y);
	destroy_FDdata(&fdc);

	if (hzodr.mode == R_MODE)
		ERc[j+CONTACTMAX] = hzodr.Er;
	else
		ERc[j+CONTACTMAX] = hzodr.radius;

	if (VERBOSE)
		printf("\n\n");
}

void 
hertz_odr_unc_close_window(Data *data)
{
	g_free(data->args->hertzodrunc.ERc);
	data->args->hertzodrunc.ERc = NULL;
	hertz_odr_mc_close_window(data);

	gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_Er), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_Eit), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_R), TRUE);
	gtk_widget_destroy(data->controls->hertzodrunccontrols->window);
}

void
hertz_odr_uncertainty (Data *data)
{
	GtkWidget  *spin;
	gint i;
	HertzODRUncControls *ctrls;
	HertzODRUncdata *unc;
	Instdata *inst;

	if (data->args->hertzodrunc.ERc != NULL){
		gtk_window_present(GTK_WINDOW(data->controls->hertzodrunccontrols->window));
		return;
	}

	/* deactivate radiobutton for mode, so that it cannot be changed while the uncertainty dialog is open
	 * adjustment of value remains active */

	switch(data->args->hertzodrdata.mode){
		case R_MODE:
			gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_Er), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_Eit), FALSE);
			break;
		case ER_MODE:
			gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_R), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_Eit), FALSE);
			break;
		case EIT_MODE:
			gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_R), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(data->controls->hertzodrcontrols->radio_Er), FALSE);
			break;
	}

	unc = &(data->args->hertzodrunc);
        ctrls =	data->controls->hertzodrunccontrols;
	inst = &(data->args->instdata);


	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Uncertainty in HertzODR analysis");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (hertz_odr_unc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));
    gtk_window_set_destroy_with_parent (GTK_WINDOW(ctrls->window), TRUE);

	/* Uncertainties due to uncertainties in input values */
	ctrls->frame_unc_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to uncertainties in input values")));


	ctrls->table_unc_input = gtk_table_new(6, 8, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_unc_input), ctrls->table_unc_input);

	i=0;

	/* create separately for individual modes */

	switch (data->args->hertzodrdata.mode ){
		case R_MODE:
			/*             RMODE           */
			/*    labels   */
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>r</sub>)/GPa"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>it</sub>)/GPa"), 4, 5, i, i+1, GTK_FILL, 0, 0, 0); 

			i++;

			/* u(h) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			i++;

			/* u(F) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);  

			i++;

			/* u(radius) */
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(radius)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uradius = gtk_adjustment_new(unc->uradius*1e9, 0, 1e6, 1, 10,0); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uradius), 0.1, 1);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uradius, "value-changed", G_CALLBACK(unc_radius_changed), data); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uEruradius = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uEruradius), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruradius, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			ctrls->uEituradius = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uEituradius), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituradius, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(nu) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->unu = gtk_adjustment_new(inst->unu, 0., 0.5, 0.01, 0.1, 0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unu), 0.01, 2); 
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->unu, "value-changed", G_CALLBACK(unc_nu_changed), data); 

			ctrls->uEitunu = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunu), 0.0, 0.5); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunu, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(nu_i) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->unui = gtk_adjustment_new(inst->unui, 0, 0.5, 0.01, 0.1, 0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unui), 0.01, 2); 
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->unui, "value-changed", G_CALLBACK(unc_nui_changed), data); 

			ctrls->uEitunui = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunui), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunui, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(E_i) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uEi = gtk_adjustment_new(inst->uEi*1e-9, 0, 1e5, 1, 10, 0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEi), 1, 0); 
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uEi, "value-changed", G_CALLBACK(unc_Ei_changed), data); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uEituEi = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uEituEi), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituEi, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* total */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

			ctrls->uErtotal = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uErtotal), 0.0, 0.5);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uErtotal, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

			ctrls->uEittotal = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uEittotal), 0.0, 0.5);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEittotal, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

			break;
		case ER_MODE:
			/*             ER_MODE           */
			/*    labels   */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(radius)/nm"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0); 

			i++;

			/* u(h) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uradiusuh = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuh), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuh, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(F) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);  

			ctrls->uradiusuF = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuF), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuF, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/*  u(Er) */
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>r</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uEr = gtk_adjustment_new(unc->uEr, 0, 1e6, 1, 10,0); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEr), 0.1, 1);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uEr, "value-changed", G_CALLBACK(unc_Er_changed), data); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uradiusuEr = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuEr), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuEr, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* total */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

			ctrls->uradiustotal = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiustotal), 0.0, 0.5);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiustotal, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
			break;
		case EIT_MODE:
			/*             EIT_MODE           */
			/*    labels   */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(radius)/nm"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0); 

			i++;

			/* u(h) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uradiusuh = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuh), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuh, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(F) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);  

			ctrls->uradiusuF = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuF), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuF, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/*  u(Eit) */
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>IT</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uEit = gtk_adjustment_new(unc->uEit, 0, 1e6, 1, 10,0); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEit), 0.1, 1);
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uEit, "value-changed", G_CALLBACK(unc_Eit_changed), data); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uradiusuEit = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuEit), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuEit, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(nu) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->unu = gtk_adjustment_new(inst->unu, 0., 0.5, 0.01, 0.1, 0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unu), 0.01, 2); 
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->unu, "value-changed", G_CALLBACK(unc_nu_changed), data); 

			ctrls->uradiusunu = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusunu), 0.0, 0.5); 

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusunu, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(nu_i) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->unui = gtk_adjustment_new(inst->unui, 0, 0.5, 0.01, 0.1, 0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unui), 0.01, 2); 
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->unui, "value-changed", G_CALLBACK(unc_nui_changed), data); 

			ctrls->uradiusunui = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusunui), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusunui, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* u(E_i) */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uEi = gtk_adjustment_new(inst->uEi*1e-9, 0, 1e5, 1, 10, 0.); 
			spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEi), 1, 0); 
			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
			g_signal_connect_swapped(ctrls->uEi, "value-changed", G_CALLBACK(unc_Ei_changed), data); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

			ctrls->uradiusuEi = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiusuEi), 0.0, 0.5); 
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiusuEi, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

			i++;

			/* total */

			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

			ctrls->uradiustotal = gtk_label_new(NULL);
			gtk_misc_set_alignment(GTK_MISC(ctrls->uradiustotal), 0.0, 0.5);
			gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uradiustotal, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
			break;
	}

	

	/* Uncertainties due to contact point */

	ctrls->frame_unc_contact = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_contact),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to choice of contact point")));

	ctrls->table_unc_contact = gtk_table_new(2*CONTACTMAX + 1 + 1, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_contact), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add (GTK_CONTAINER (ctrls->frame_unc_contact), ctrls->table_unc_contact);

	unc->ERc = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));

	/* table is filled in propagate_uncertainties, because it is updated at the change of R */
	hertz_odr_propagate_uncertainties (data);

	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (hertz_odr_uncertainty_save_data), data->args);

	/* Monte Carlo calculation */
	
	ctrls->frame_mc = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_mc),
				    GTK_WIDGET (gwy_label_new_header("Monte Carlo calculation of uncertainties")));

	ctrls->table_mc = gtk_table_new(6, 7, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_mc), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_mc), ctrls->table_mc);

	i = 0;
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), gwy_label_new_header("Number of iterations"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->Nmc = gtk_adjustment_new(unc->Nmc, 1, 1e9, 10, 100,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->Nmc), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->Nmc, "value-changed", G_CALLBACK(mc_iterations_changed), data);
	
	i++;

	ctrls->mc_warn = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->mc_warn, 0, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	if (unc->Nmc > MC_WHEN_WARN)
		label_set_gint_format(ctrls->mc_warn, unc->Nmc, "Warning: Monte Carlo calculation for %d iterations may take very long!");

	//no Monte Carlo window without the uncertainty dialog
	data->controls->hertzmccontrols->has_window = FALSE;
	ctrls->button_mc = gtk_button_new_with_mnemonic ("_Monte Carlo");
	gtk_button_set_image (GTK_BUTTON (ctrls->button_mc), gtk_image_new_from_stock (GWY_STOCK_POLYNOM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_mc, "clicked", G_CALLBACK (hertz_odr_uncertainty_montecarlo), data);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_contact, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save,  FALSE, FALSE,0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_mc, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_mc, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
}

#endif
