#ifndef NOFORTRAN

#include "tool_op_odr.h"
#include "tool_op_odr_unc_gui.h"

#include "controls.h"
#include "datatypes.h"
#include "file-utils.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "indent-toolbox-common.h"

#define _USE_MATH_DEFINES
#include <math.h>

#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include <app/file.h>
#include <libgwydgets/gwydgets.h>
#include <libprocess/gwyprocess.h>

#define BETA_DEFAULT 1.0

/* eliminated in favour of standard parameter passing */
/* #define RESCALE FALSE */

static const gchar *logfnm = "fit.log.op";
//static const gchar *logrptfnm = "fit.log.rpt";
//static const gchar *logerrfnm = "fit.log.err";

static void op_odr_show_log(Data *data);

static void update_models_op_odr(OPODRControls *opodrcontrols);
static void op_odr_zoom_in(Data *data);
static void op_odr_zoom_out(Data *data);
void op_odr_fit_and_update(Data *data);

void
init_OPODRdata(OPODRdata *opodrdata)
{
	opodrdata->beta = BETA_DEFAULT;

	opodrdata->hp=0;
	opodrdata->hc=0;
	opodrdata->hr=0;
	opodrdata->eps=0;
	opodrdata->m=0;
	opodrdata->S=0;
	opodrdata->Aphc=0;

	opodrdata->range[0]=0;
	opodrdata->range[1]=0;
	opodrdata->range_pct_Fmax[0]=0;
	opodrdata->range_pct_Fmax[1]=0;
	opodrdata->has_fit=FALSE;
	opodrdata->from=0;
	opodrdata->to=0;
	opodrdata->to_pct_Fmax=0;
	opodrdata->from_pct_Fmax=0;
	opodrdata->xfit=NULL;
	opodrdata->yfit=NULL;

	opodrdata->Er=0;
	opodrdata->Eit=0;
	opodrdata->Hit=0;

	opodrdata->Finput = FALSE;
	opodrdata->nfitdata = 0;

	
	/* quality of fit stuff */
	opodrdata->SSres = -1;
	opodrdata->R2 = -1;
	opodrdata->R2adj = -1;
	opodrdata->chi2 = -1;

	opodrdata->logfnm = create_log_filename_path(logfnm);
	opodrdata->infolog = 0;

}

void
init_OPODRUncdata (OPODRUncdata *opodrunc)
{
	opodrunc->uh = 1;
	opodrunc->uF = 0.001;
/*	opodrunc->unu = 0.05;
	opodrunc->unui = 0.04;
	opodrunc->uEi = 20e9;
	*/
	opodrunc->Nmc = N_MC;

	opodrunc->uhp = 0 ;
	opodrunc->covmhp = 0 ;
	opodrunc->um = 0 ;
	opodrunc->umuh = 0 ;
	opodrunc->umuF = 0 ;

	opodrunc->ueps = 0;
	opodrunc->depsdm = 0;
	opodrunc->uhcuh = 0;
	opodrunc->uhcuF = 0;
	opodrunc->uhc = 0;

	opodrunc->uSuh = 0;
	opodrunc->uSuF = 0;
	opodrunc->uS = 0;

	opodrunc->uAuh = 0;
	opodrunc->uAuF = 0;
	opodrunc->uA = 0;

	opodrunc->uHituh = 0;
	opodrunc->uHituF = 0;
	opodrunc->uHit = 0;

	opodrunc->uEruh = 0;
	opodrunc->uEruF = 0;
	opodrunc->uEr = 0;

	opodrunc->uEituh = 0;
	opodrunc->uEituF = 0;
	opodrunc->uEitunu = 0;
	opodrunc->uEitunui = 0;
	opodrunc->uEituEi = 0;
	opodrunc->uEit = 0;


	opodrunc->Ec=NULL;
	opodrunc->Hc=NULL;
	opodrunc->Ac=NULL;
	opodrunc->hc=NULL;

}


gboolean
op_odr_has_results(OPODRdata *opodr)
{
	return opodr->has_fit;
}

gboolean
op_odr_unc_has_results(OPODRUncdata *opodrunc)
{
	return  (opodrunc->Ec != NULL && opodrunc->Hc !=NULL && opodrunc->Ac !=NULL && opodrunc->hc !=NULL);
}



void
op_odr_remove_all_fit_results(OPODRdata *opodrdata){
	//reset all results associated with a fit to zero
	
	opodrdata->SSres = -1;
	opodrdata->R2 = -1;
	opodrdata->R2adj = -1;
	opodrdata->chi2 = -1;
	opodrdata->hp=0;
	opodrdata->m=0;
	opodrdata->eps=0;
	opodrdata->hc=0;
	opodrdata->S=0;
	opodrdata->Aphc=0;
	opodrdata->range[0]=0;
	opodrdata->range[1]=0;
	opodrdata->range_pct_Fmax[0]=0;
	opodrdata->range_pct_Fmax[1]=0;
	opodrdata->from=0;
	opodrdata->to=0;
	opodrdata->Hit=0;
	opodrdata->Eit=0;
	opodrdata->Er=0;
	opodrdata->infolog = 0;
}

void
op_odr_remove_all_fit_labels(OPODRControls *opodrcontrols)
{
    /* remove all labels that result from a fit */
    
    entry_clear(opodrcontrols->from);
    entry_clear(opodrcontrols->to);
    entry_clear(opodrcontrols->from_pct_Fmax);
    entry_clear(opodrcontrols->to_pct_Fmax);

    /* R currently not available */
    /* label_clear(opodrcontrols->R); */

    label_clear(opodrcontrols->SSres);
    label_clear(opodrcontrols->R2);
    label_clear(opodrcontrols->R2adj);
    label_clear(opodrcontrols->chi2); 
    label_clear(opodrcontrols->fitinfo);
    label_clear(opodrcontrols->S);
    label_clear(opodrcontrols->hp);
    label_clear(opodrcontrols->m);
    label_clear(opodrcontrols->alpha);
    label_clear(opodrcontrols->eps);
    label_clear(opodrcontrols->hc);
    label_clear(opodrcontrols->Aphc);
    label_clear(opodrcontrols->extrapol);

    label_clear(opodrcontrols->Hit);
    label_clear(opodrcontrols->Er);
    label_clear(opodrcontrols->Eit);
    label_clear(opodrcontrols->range);
    label_clear(opodrcontrols->range_pct_Fmax);
}

void
op_odr_remove_all_fit_curves(OPODRControls *opodrcontrols)
{
    gwy_graph_model_remove_curve_by_description(opodrcontrols->graph_model, "Fit");
}



static void recalculate_Eit(Data *data)
{
    gdouble Er;
    gdouble nu, nui, Ei, Eit;

    nu = data->args->instdata.nu;
    nui = data->args->instdata.nui;
    Ei = data->args->instdata.Ei;
    Er = data->args->opodrdata.Er * 1e9;

    // recalc for changed  nu, nui or Ei
    if (Er == 0)
        return;
    
    Eit = (1 - nu*nu) / (1/Er - (1 - nui*nui)/Ei);
    data->args->opodrdata.Eit = Eit * 1e-9;
    label_set_gdouble_format(data->controls->opodrcontrols->Eit,  data->args->opodrdata.Eit, MODUL);
}

void op_odr_active(Data *data)
{
	// update loading curve and Fmax and hmax labels
	if (data->args->fddata.hunload) {
	    gwy_graph_curve_model_set_data(data->controls->opodrcontrols->cmodelunload,
					   data->args->fddata.hunload->data, data->args->fddata.Funload->data,
					   data->args->fddata.hunload->res);

	    label_set_gdouble_format(data->controls->opodrcontrols->hmax, data->args->fddata.hmax, DEPTH);
	    label_set_gdouble_format(data->controls->opodrcontrols->Fmax, data->args->fddata.Fmax, FORCE);

	    if (gwy_graph_model_get_n_curves(data->controls->opodrcontrols->graph_model) == 2){
			gwy_graph_curve_model_set_data(data->controls->opodrcontrols->cmodelSfit, 
					data->args->opodrdata.xfit->data, data->args->opodrdata.yfit->data,  data->args->opodrdata.xfit->res);
			gwy_graph_curve_model_enforce_order (data->controls->opodrcontrols->cmodelSfit);
	    }
	}
}

static void
opodr_recalculate_beta(Data *data)
{
	OPODRdata *opodrdata;
	OPODRControls *opodrcontrols;
	Instdata *instdata;

	opodrdata = &(data->args->opodrdata);
	instdata = &(data->args->instdata);
	opodrcontrols = data->controls->opodrcontrols;
	
	if (!opodrdata->has_fit)
		return;

	opodrdata->Er = sqrt(M_PI / opodrdata->Aphc) * opodrdata->S / (2*opodrdata->beta) * 1e15;
	opodrdata->Eit = (1 - instdata->nu * instdata->nu) / (1 / opodrdata->Er - (1 - instdata->nui * instdata->nui) / instdata->Ei);
	opodrdata->Eit *= 1e-9;
	opodrdata->Er *= 1e-9;

	label_set_gdouble_format(opodrcontrols->Eit, opodrdata->Eit, MODUL);
	label_set_gdouble_format(opodrcontrols->Er, opodrdata->Er, MODUL);
}

static void
opodr_beta_changed(Data *data)
{
	OPODRControls *fcontrols;

	fcontrols = data->controls->opodrcontrols;
	if (beta_changed(fcontrols->beta, &(data->args->opodrdata.beta)))
		opodr_recalculate_beta(data);
}

static void
opodr_reset_beta(Data *data)
{
	data->args->opodrdata.beta = BETA_DEFAULT;
	entry_set_gdouble_format(data->controls->opodrcontrols->beta, data->args->opodrdata.beta, "%.03f");
	opodr_recalculate_beta(data);
}

static void
op_odr_range_from_changed(Data *data)
{
    OPODRControls *fcontrols;
    OPODRdata *fdata;

    fcontrols = data->controls->opodrcontrols;
    fdata = &(data->args->opodrdata);
	
    range_from_changed(fcontrols->from, fcontrols->to, fcontrols->selection, &(fdata->from), &(fdata->from));
}

static void
op_odr_range_to_changed(Data *data)
{
    OPODRControls *tcontrols;
    OPODRdata *tdata;

    tcontrols = data->controls->opodrcontrols;
    tdata = &(data->args->opodrdata);
	
    range_to_changed(tcontrols->from, tcontrols->to, tcontrols->selection, &(tdata->to), &(tdata->to));
}

static gboolean
op_odr_range_from_pct_Fmax_changed(Data *data)
{
    OPODRControls *rcontrols;
    OPODRdata *rdata;

    rcontrols = data->controls->opodrcontrols;
    rdata = &(data->args->opodrdata);

    return range_from_pct_Fmax_changed(rcontrols->from_pct_Fmax, rcontrols->to_pct_Fmax, rcontrols->selection, data->args->fddata.Fmax, data->args->fddata.hunload, data->args->fddata.Funload,
				       &(rdata->from_pct_Fmax), &(rdata->from_pct_Fmax), &(rdata->to_pct_Fmax), &(rdata->to_pct_Fmax), &(rdata->Finput), &(rdata->Finput));
}

static gboolean
op_odr_range_to_pct_Fmax_changed(Data *data)
{
    OPODRControls *rcontrols;
    OPODRdata *rdata;

    rcontrols = data->controls->opodrcontrols;
    rdata = &(data->args->opodrdata);

    return range_to_pct_Fmax_changed(rcontrols->from_pct_Fmax, rcontrols->to_pct_Fmax, rcontrols->selection, data->args->fddata.Fmax, data->args->fddata.hunload, data->args->fddata.Funload,
				     &(rdata->from_pct_Fmax), &(rdata->from_pct_Fmax), &(rdata->to_pct_Fmax), &(rdata->to_pct_Fmax), &(rdata->Finput), &(rdata->Finput));
}

static void
op_odr_graph_selected( GwySelection* selection, gint i,Data *data )
{
    gdouble range[2];
    gint istart, iend, ndata;
    gdouble Fmax;

    g_return_if_fail(i <= 0);

    //take data from selection and put them in the "from" and "to" values
    gwy_selection_get_object(selection, 0, range);

    data->args->opodrdata.from = MIN(range[0], range[1]);
    data->args->opodrdata.to = MAX(range[0], range[1]);

    //update labels
    entry_set_gdouble_format(data->controls->opodrcontrols->from, data->args->opodrdata.from, DEPTH);
    entry_set_gdouble_format(data->controls->opodrcontrols->to, data->args->opodrdata.to, DEPTH);

    Fmax = data->args->fddata.Fmax;
    range_to_indices(data->args->opodrdata.from,data->args->opodrdata.to , data->args->fddata.hunload, TRUE, &istart, &iend, &ndata);
    data->args->opodrdata.from_pct_Fmax = data->args->fddata.Funload->data[iend]/Fmax*100;
    data->args->opodrdata.to_pct_Fmax = data->args->fddata.Funload->data[istart]/Fmax*100;

    entry_set_gdouble_format(data->controls->opodrcontrols->from_pct_Fmax, data->args->opodrdata.from_pct_Fmax, DEPTH);
    entry_set_gdouble_format(data->controls->opodrcontrols->to_pct_Fmax, data->args->opodrdata.to_pct_Fmax, DEPTH);

    data->args->opodrdata.Finput = FALSE;
}

gboolean op_odr_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	gint i;
	OPODRdata *opodrdata;
	Instdata *instdata;
	gchar *areatext;

	opodrdata = &(args->opodrdata);
	instdata = &(args->instdata);

	if (args->fddata.hunload->res == 0 || args->fddata.Funload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	//Input parameters
	fprintf(file, "#Parameters of OP ODR analysis \n");
	fprintf(file, "#beta: %g \n", opodrdata->beta);
	fprintf(file, "#sigma_h: %g nm\n", instdata->sigma_h);
	fprintf(file, "#sigma_F: %g mN\n", instdata->sigma_F);
	fprintf(file, "#delta: %g \n", instdata->delta);
	fprintf(file, "#nu: %g \n", instdata->nu);
	fprintf(file, "#nui: %g \n", instdata->nui);
	fprintf(file, "#Ei: %g GPa\n", instdata->Ei * 1e-9); //TODO units
	// Area
	areatext = write_area_file(&(args->area));
	fprintf(file, "%s \n", areatext);
	g_free(areatext);
	//warning if extrapolation in area
	if (args->area.mode == AREA_DATA && opodrdata->hc > args->area.xmax)
		fprintf(file, "#    WARNING: Extrapolation in area calibration! \n");


	fprintf(file, "\n\n");


	fprintf(file, "#Results of OP ODR analysis \n");

	fprintf(file, "#hmax: %g nm\n", args->fddata.hmax);
	fprintf(file, "#Fmax: %g mN\n", args->fddata.Fmax);

	fprintf(file, "#hp: %g nm\n", opodrdata->hp);
	fprintf(file, "#m: %g nm\n", opodrdata->m);
	fprintf(file, "#α: %g mN.nm^{-%.3f} \n", opodrdata->alpha, opodrdata->m);

	fprintf(file, "#fit info: %d \n", opodrdata->fitinfo);
	fprintf(file, "#SSres: %g \n", opodrdata->SSres);
	fprintf(file, "#R-squared: %g \n", opodrdata->R2);
	fprintf(file, "#adjusted R-squared: %g \n", opodrdata->R2adj);
	fprintf(file, "#chi-squared: %g \n", opodrdata->chi2);

	fprintf(file, "#epsilon: %g \n", opodrdata->eps);
	fprintf(file, "#hc: %g nm\n", opodrdata->hc);
	fprintf(file, "#S: %g mN/nm\n", opodrdata->S);
	fprintf(file, "#Aphc: %g nm^2\n", opodrdata->Aphc);
	fprintf(file, "#Hit: %g MPa\n", opodrdata->Hit);
	fprintf(file, "#Er: %g GPa\n", opodrdata->Er);
	fprintf(file, "#Eit: %g GPa\n", opodrdata->Eit);

	if (args->opodrdata.has_fit) {
		fprintf(file, "\n \n");
		fprintf(file, "#Fitted range: ");
		if (args->opodrdata.Finput)
			fprintf(file, "	[%g , %g ] %% Fmax \n", opodrdata->range_pct_Fmax[0], opodrdata->range_pct_Fmax[1]);
		else
			fprintf(file, "[%g , %g ] nm \n",opodrdata->range[0], opodrdata->range[1]);	
		fprintf(file, "# corresponds : ");
		if (args->opodrdata.Finput)
			fprintf(file, "[%g , %g ] nm \n",opodrdata->range[0], opodrdata->range[1]);	
		else
			fprintf(file, "	[%g , %g ] %% Fmax \n", opodrdata->range_pct_Fmax[0], opodrdata->range_pct_Fmax[1]);
		fprintf(file, "# %d datapoints used.\n", opodrdata->nfitdata);

		fprintf(file,"\n# S Fit \n");
		fprintf(file, "# x/nm y/mN \n");
		for (i = 0; i < args->opodrdata.xfit->res; i++)
			fprintf(file, "%g %g \n", args->opodrdata.xfit->data[i], args->opodrdata.yfit->data[i]);
	}
	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;

}

void op_odr_save_data(Args *args)
{
	GtkWidget *dialog;
	gchar *filename;
	FileSaveStatus filestatus;
	gboolean file_ok;
	gint response;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	file_ok = FALSE;
	response = gtk_dialog_run (GTK_DIALOG ( dialog));
	while (  response == GTK_RESPONSE_OK && !file_ok)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog)){
			op_odr_write_file(filename, args, &filestatus);
			file_ok = (filestatus == SAVE_FILE_OK);
		}
		g_free(filename);
		if (!file_ok){
			show_save_error(filestatus, filename);
			response = gtk_dialog_run (GTK_DIALOG ( dialog));
		}
	}

	gtk_widget_destroy ( dialog);

}

void op_odr_fit_and_update(Data *data)
{
	gchar str[300];
	gint nc, i;
	gint istart, iend, ndata;
	gint fitinfo;
	gboolean fit=TRUE;
	GwyDataLine *x, *y;

	OPODRdata *opodrdata;
	FDdata *fddata;
	OPODRControls *opodrcontrols;
	GdkColor color;


	gdk_color_parse ("red", &color);
	
	// TEMPORARY update GtkEntries for ranges
	/*
	if (data->args->opodrdata.Finput){
		range_from_pct_Fmax_changed(data);
		range_to_pct_Fmax_changed(data);
	}
	else{
		range_from_changed(data);
		range_to_changed(data);
	}
	*/


	//create pointers for easier reading
	opodrdata=&(data->args->opodrdata);
	opodrcontrols=data->controls->opodrcontrols;
	fddata=&(data->args->fddata);

	//if there was a previous fit, invalidate previous fitted curve
	if (opodrdata->has_fit){
		g_object_unref(opodrdata->xfit);
		g_object_unref(opodrdata->yfit);
	}

	// check range
	if (opodrdata->from == opodrdata->to){
		fit = FALSE;
	}
	
	//find appropriate data, which should be fitted
	//default regime is governed by depth, can be switched to load regime by Finput (if the pct_Fmax ranges were used)
	if (opodrdata->Finput)
	{
		range_to_indices(opodrdata->from_pct_Fmax*0.01*fddata->Fmax, opodrdata->to_pct_Fmax*0.01*fddata->Fmax,
			       	fddata->Funload, TRUE, &istart, &iend, &ndata);
	}
	else
		range_to_indices(opodrdata->from, opodrdata->to, fddata->hunload, TRUE, &istart, &iend, &ndata);


	// if there are not enough data in the selected range,no fitting can be performed 
	ndata = MIN(ndata, fddata->hunload->res - istart);
	if (ndata < 2){
		fit = FALSE;
	}

	opodrdata->nfitdata = ndata;
	if (fit){
		//get data to be fitted
		x = gwy_data_line_part_extract(fddata->hunload, istart, ndata);
		y = gwy_data_line_part_extract(fddata->Funload, istart, ndata);

		//fit the data and evaluate
		fitinfo = op_odr_fit(x,y, opodrdata, &(data->args->fddata), &(data->args->instdata), &(data->args->area), NULL);

		printf("fitinfo pre %d infolog %d \n", fitinfo, opodrdata->infolog);
		/* fatal errors end with fitinfo >= 5, infolog first digit, fitinfo last digit  */
		if (fitinfo >= 1e6){
			opodrdata->infolog = fitinfo/1000000;
			fitinfo = fitinfo % 1000000;
		}
		printf("fitinfo post %d infolog %d \n", fitinfo, opodrdata->infolog);
				
		if (fitinfo >= 5) {
			gtk_label_set_text(GTK_LABEL(opodrcontrols->fitinfo), "Warning: Questionable results \n or fatal errors in fitting!");
			gtk_widget_modify_fg (opodrcontrols->fitinfo, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(opodrcontrols->fitinfo), TRUE);
		}
		else if (fitinfo == 4) {
			gtk_label_set_text(GTK_LABEL(opodrcontrols->fitinfo), "Warning: Iteration limit reached!");
			gtk_widget_modify_fg (opodrcontrols->fitinfo, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(opodrcontrols->fitinfo), TRUE);
		}
		else if (opodrdata->m > 2) {
			gtk_label_set_text(GTK_LABEL(opodrcontrols->fitinfo), "Warning: Unphysical fit results!");
			gtk_widget_modify_fg (opodrcontrols->fitinfo, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(opodrcontrols->fitinfo), TRUE);
		}
		else {
			label_clear(opodrcontrols->fitinfo);
		}

		//update range labels
		entry_set_gdouble_format(opodrcontrols->from, opodrdata->from, DEPTH);
		entry_set_gdouble_format(opodrcontrols->to, opodrdata->to, DEPTH);
		entry_set_gdouble_format(opodrcontrols->from_pct_Fmax, opodrdata->from_pct_Fmax, DEPTH);
		entry_set_gdouble_format(opodrcontrols->to_pct_Fmax, opodrdata->to_pct_Fmax, DEPTH);

		//update labels
		label_set_gdouble_format(opodrcontrols->hp, opodrdata->hp, DEPTH);
		label_set_gdouble_format(opodrcontrols->m, opodrdata->m, NUMBER);
		label_set_gdouble_format(opodrcontrols->alpha, opodrdata->alpha, "%.4g");

		label_set_gdouble_format(opodrcontrols->SSres, opodrdata->SSres, "%.4g");
		label_set_gdouble_format(opodrcontrols->R2, opodrdata->R2, "%.4g");
		label_set_gdouble_format(opodrcontrols->R2adj, opodrdata->R2adj, "%.4g");
		label_set_gdouble_format(opodrcontrols->chi2, opodrdata->chi2, "%.4g");

		label_set_gdouble_format(opodrcontrols->eps, opodrdata->eps, NUMBER);
		label_set_gdouble_format(opodrcontrols->S, opodrdata->S, SLOPE);
		label_set_gdouble_format(opodrcontrols->hc, opodrdata->hc, DEPTH);
		label_set_gdouble_format(opodrcontrols->Aphc, opodrdata->Aphc, AREA);

		if (data->args->area.mode == AREA_DATA && opodrdata->hc > data->args->area.xmax){
			gtk_label_set_text(GTK_LABEL(opodrcontrols->extrapol), "Warning: Extrapolation in area calibration!");
			gtk_widget_modify_fg (opodrcontrols->extrapol, GTK_STATE_NORMAL, &color);
		}
		else{
			label_clear(opodrcontrols->extrapol);
		}

		
		g_snprintf(str, sizeof(str), "[%.3f, %.3f]", opodrdata->from, opodrdata->to);
		gtk_label_set_text(GTK_LABEL(opodrcontrols->range), str);
		g_snprintf(str, sizeof(str), "[%.3f, %.3f]", opodrdata->from_pct_Fmax, opodrdata->to_pct_Fmax);
		gtk_label_set_text(GTK_LABEL(opodrcontrols->range_pct_Fmax), str);

		label_set_gdouble_format(opodrcontrols->Hit, opodrdata->Hit, HARD);
		label_set_gdouble_format(opodrcontrols->Eit, opodrdata->Eit, MODUL);
		label_set_gdouble_format(opodrcontrols->Er, opodrdata->Er, MODUL);

		//calculate curve of fit
		opodrdata->xfit = gwy_data_line_duplicate(x);
		opodrdata->yfit = gwy_data_line_new_alike(y,FALSE);

		for (i=0; i< opodrdata->xfit->res; i++){
			opodrdata->yfit->data[i] =  opodrdata->alpha*pow(opodrdata->xfit->data[i]-opodrdata->hp,opodrdata->m);
		}

		// clean up
		g_object_unref(x);
		g_object_unref(y);

		//update fitted curve, if necessary create it
		nc = gwy_graph_model_get_n_curves(opodrcontrols->graph_model);
		if (nc == 2) {
			gwy_graph_curve_model_set_data(opodrcontrols->cmodelSfit, opodrdata->xfit->data, opodrdata->yfit->data, opodrdata->xfit->res);
			gwy_graph_curve_model_enforce_order (opodrcontrols->cmodelSfit);
		}
		else {
			gwy_graph_curve_model_set_data(opodrcontrols->cmodelSfit, opodrdata->xfit->data, opodrdata->yfit->data, opodrdata->xfit->res);
			gwy_graph_curve_model_enforce_order (opodrcontrols->cmodelSfit);
			gwy_graph_model_add_curve(opodrcontrols->graph_model, opodrcontrols->cmodelSfit);
			g_object_set(opodrcontrols->cmodelSfit,
					"mode", GWY_GRAPH_CURVE_LINE,
					"description", "Fit",
					"color", CURVE_COLOR_FIT,
					"line-style", GDK_LINE_SOLID,
					"line-width", 3,
					NULL);
		}
	}
	else{
		nc = gwy_graph_model_get_n_curves(opodrcontrols->graph_model);
		op_odr_remove_all_fit_labels(opodrcontrols);			
		op_odr_remove_all_fit_results(opodrdata);			
		opodrdata->has_fit = FALSE;
		if (nc == 2){
			gwy_graph_model_remove_curve_by_description(opodrcontrols->graph_model, "Fit");
		}
	}
	// log of fitting procedure is available
	if (opodrdata->has_fit)
		gtk_widget_set_sensitive(opodrcontrols->button_log, TRUE);
	else
		gtk_widget_set_sensitive(opodrcontrols->button_log, FALSE);

	// update previous uncertainties if existing, set button 
	if (opodrdata->has_fit){
		 if ( data->args->opodrunc.Ec !=NULL || data->args->opodrunc.Hc != NULL || data->args->opodrunc.Ac != NULL || data->args->opodrunc.hc != NULL)
			 op_odr_unc_close_window(data);
		gtk_widget_set_sensitive(opodrcontrols->button_unc, TRUE);
	}
	else
		gtk_widget_set_sensitive(opodrcontrols->button_unc, FALSE);
}

gint
op_odr_fit(GwyDataLine *x, GwyDataLine *y, OPODRdata *opodrdata, FDdata *fddata, Instdata *instdata, Area *area, gdouble estimate[3])
{
        gdouble beta[3], std_beta[3], covariance[3];
	gint info;
	gint ifix[3];
	gdouble ux, uy;
	gint istart,iend,ndata = 0;
	gdouble r1 = 0.05,r2 = 0.1, rstep = 0.05;
	gdouble xmin;


	//fit the data with the function beta[0]*(h-beta[1])^beta[2]
	// beta [0] --- alpha, gamma
	// beta [1] --- hp, h0
	// beta [2] --- m, n
	if (estimate){
		beta[0]=estimate[0];
		beta[1]=estimate[1];
		beta[2]=estimate[2];
	}
	else{
		beta[2] = 1.5;
		
		while ( ndata ==0 && r2 < 1.0 ){
			range_to_indices(r1*fddata->Fmax, r2*fddata->Fmax, fddata->Funload, TRUE, &istart, &iend, &ndata); 
			r1+=rstep;
			r2+=rstep;
		}
		if (ndata > 0)
			beta[1] = fddata->hunload->data[iend]; 
		else{
			beta[1] = fddata->hmax*0.05; 
		}

		/* beta[1] (hp estimate) should not be smaller than the smallest x */
		/* beta[1] equal to minimum does not work very well either */
		xmin = gwy_data_line_get_min(x);
		beta[1] = (beta[1] < xmin ? beta[1] : xmin*0.9);

		printf(" beta[1] %g xmin %g \n", beta[1], gwy_data_line_get_min(x));
		beta[0] = fddata->Fmax/pow(fddata->hmax - beta[1], beta[2]); 

	}
// TODO keep some parameters fixed 0:fixed 1:unfixed
	ifix[0]=1;
	ifix[1]=1;
	ifix[2]=1;


	// uncertainties given by sigma_h, sigma_F
	ux = instdata->sigma_h;
	uy = instdata->sigma_F;

	printf("beta estimate %g %g %g  \n", beta[0], beta[1], beta[2]);
	info = odr_run_fit(x, y, beta, ifix, opodrdata->logfnm, ux, uy, std_beta, covariance, 
			&(opodrdata->SSres), &(opodrdata->R2), &(opodrdata->R2adj), &(opodrdata->chi2), FALSE); /* do not rescale */
	printf("beta result %g %g %g  \n\n", beta[0], beta[1], beta[2]);

	// calculate results 
	opodrdata->alpha = beta[0];
	opodrdata->hp = beta[1];
	opodrdata->m = beta[2];
	opodrdata->eps = opodrdata->m * (1 - 2*(opodrdata->m-1) * tgamma(opodrdata->m / 2 / (opodrdata->m - 1)) / tgamma(1.0 / 2.0 / (opodrdata->m - 1)) / sqrt(M_PI));

	opodrdata->S = opodrdata->m * fddata->Fmax / (fddata->hmax - opodrdata->hp);
	opodrdata->hc = fddata->hmax - opodrdata->eps * fddata->Fmax / opodrdata->S;
	opodrdata->Aphc = eval_area(area, opodrdata->hc);

	// TODO units 
	// calculate results 
	opodrdata->Hit = fddata->Fmax / opodrdata->Aphc * 1e9; 
	opodrdata->Er = sqrt(M_PI / opodrdata->Aphc) * opodrdata->S / (2*opodrdata->beta) * 1e15;
	opodrdata->Eit = (1 - instdata->nu * instdata->nu) / (1 / opodrdata->Er - (1 - instdata->nui * instdata->nui) / instdata->Ei);
	opodrdata->Eit *= 1e-9;
	opodrdata->Er *= 1e-9;


	//clean up
	opodrdata->range[0] = opodrdata->from;
	opodrdata->range[1] = opodrdata->to;
	opodrdata->has_fit = TRUE;
	opodrdata->range_pct_Fmax[0] = opodrdata->from_pct_Fmax;
	opodrdata->range_pct_Fmax[1] = opodrdata->to_pct_Fmax;

	opodrdata->fitinfo = info;
	
	return info;
}


static void op_odr_show_log(Data *data)
{
	GtkWidget *scrolled_window;
	GtkWidget *textview;
	GtkTextBuffer *buffer;
	gchar *log;
	gint len;
	FILE *logfile;
	gchar *logfnmerr;
	gchar *logfnmrpt;
	OPODRControls *ctrls;
	OPODRdata *opodrdata;

	ctrls = data->controls->opodrcontrols;
	opodrdata = &(data->args->opodrdata);

	if (ctrls->log_error_window  != NULL && GTK_IS_WIDGET(ctrls->log_error_window))
		gtk_widget_destroy(ctrls->log_error_window);


	/* check if there was a log */
	if (opodrdata->infolog == 2 || opodrdata->infolog == 3 ) {
	    show_warning("Fit error log could not be created.");
	}
	else {
		/*open log */
		logfnmerr = g_malloc(sizeof(gchar)*(strlen(opodrdata->logfnm)+5));
		strcpy(logfnmerr, opodrdata->logfnm);
		strcat(logfnmerr, ".err");
		logfile = fopen(logfnmerr, "r");
		if (!logfile) { /* should not get here */
		    show_error("No error log.");
		    g_free(logfnmerr);
		    return;
		}
		else {
			/*find length and read file */
			fseek(logfile, 0, SEEK_END);
			len = ftell(logfile);
			log = (gchar *) g_malloc((len+1)*sizeof(gchar));
			rewind(logfile);
			fread(log, len,1,logfile);
			fclose(logfile);
			log[len] = '\0';
			g_free(logfnmerr);

			/*create buffer, fill it and create textview */
			buffer = gtk_text_buffer_new (NULL);
			gtk_text_buffer_set_text (buffer, log, -1);
			textview = gtk_text_view_new_with_buffer (buffer);
#ifndef G_OS_WIN32
			gtk_widget_modify_font (textview, pango_font_description_from_string ("monospace"));
#endif
			gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
			g_free(log);

			scrolled_window = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_container_add(GTK_CONTAINER(scrolled_window), textview);

			ctrls->log_error_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_window_set_title(GTK_WINDOW(ctrls->log_error_window), "Fit error log");
			gtk_widget_set_size_request(GTK_WIDGET(ctrls->log_error_window), 450, 400);
			g_signal_connect (ctrls->log_error_window, "delete_event", G_CALLBACK (gtk_widget_destroy),NULL);
			gtk_container_add(GTK_CONTAINER(ctrls->log_error_window), scrolled_window);
			gtk_widget_show_all(ctrls->log_error_window);
		}
	}


	/* check whether there was a log */
	if (opodrdata->infolog == 1 || opodrdata->infolog == 3 ) {
	    show_warning("Fit report could not be created.");
	}
	else {
		/*open log */
		if (ctrls->log_report_window != NULL && GTK_IS_WIDGET(ctrls->log_report_window))
			gtk_widget_destroy(ctrls->log_report_window);


		logfnmrpt = g_malloc(sizeof(gchar)*(strlen(opodrdata->logfnm)+5));
		strcpy(logfnmrpt, opodrdata->logfnm);
		strcat(logfnmrpt, ".rpt");
		logfile = fopen(logfnmrpt, "r");
		if (!logfile) { /* should not get here */
		    show_error("No report log.");
		    g_free(logfnmrpt);
		    return;
		}
		else {
			/*find length and read file */
			fseek(logfile, 0, SEEK_END);
			len = ftell(logfile);
			log = (gchar *) g_malloc((len+1)*sizeof(gchar));
			rewind(logfile);
			fread(log, len,1,logfile);
			fclose(logfile);
			log[len] = '\0';
			g_free(logfnmrpt);

			/*create buffer, fill it and create textview */
			buffer = gtk_text_buffer_new (NULL);
			gtk_text_buffer_set_text (buffer, log, -1);
			textview = gtk_text_view_new_with_buffer (buffer);
#ifndef G_OS_WIN32
			gtk_widget_modify_font (textview, pango_font_description_from_string ("monospace"));
#endif
			gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
			g_free(log);

			scrolled_window = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_container_add(GTK_CONTAINER(scrolled_window), textview);

			ctrls->log_report_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_window_set_title(GTK_WINDOW(ctrls->log_report_window), "Fit report log");
			gtk_widget_set_size_request(GTK_WIDGET(ctrls->log_report_window), 450, 400);
			g_signal_connect (ctrls->log_report_window, "delete_event", G_CALLBACK (gtk_widget_destroy),NULL);
			gtk_container_add(GTK_CONTAINER(ctrls->log_report_window), scrolled_window);
			gtk_widget_show_all(ctrls->log_report_window);
		}
	}
}


GtkWidget *
tool_op_odr_create (Data *data)
{
	GwyGraphArea *area;
	Args *args;
	OPODRControls *opodrcontrols;
	gint i;
//	gchar str[300];

	args=data->args;
	opodrcontrols=data->controls->opodrcontrols;

	// GraphArea, two curves can be displayed: loading F-d curve and the resulting fit 
	opodrcontrols->graph_model = gwy_graph_model_new();
	g_object_set(opodrcontrols->graph_model, "title", "Unload", NULL);
	gwy_graph_model_set_axis_label (opodrcontrols->graph_model, GTK_POS_BOTTOM, "h / nm");
	gwy_graph_model_set_axis_label (opodrcontrols->graph_model, GTK_POS_LEFT, "F / mN");

	opodrcontrols->zoomstack = NULL;
	opodrcontrols->zoomstack = g_slist_prepend (opodrcontrols->zoomstack, opodrcontrols->graph_model);

	opodrcontrols->hbox_graph = gtk_hbox_new (FALSE, CTRLS_SPACING);
	
	opodrcontrols->graph = gwy_graph_new(opodrcontrols->graph_model);
	/* gtk_widget_set_size_request(GTK_WIDGET(opodrcontrols->graph), 600, 400); */

	gwy_graph_enable_user_input(GWY_GRAPH(opodrcontrols->graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(opodrcontrols->graph), GWY_GRAPH_STATUS_XSEL);


	area = GWY_GRAPH_AREA(gwy_graph_get_area(GWY_GRAPH(opodrcontrols->graph)));
	opodrcontrols->selection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XSEL);
	gwy_selection_set_max_objects(opodrcontrols->selection, 1);
	g_signal_connect(opodrcontrols->selection, "changed", G_CALLBACK(op_odr_graph_selected), data);

	opodrcontrols->cmodelunload = gwy_graph_curve_model_new();
	opodrcontrols->cmodelSfit = gwy_graph_curve_model_new();

	g_object_set(opodrcontrols->cmodelunload,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Unloading",
			"point-size", 0,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
		        "color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
		        "line-width", 2,
			NULL);
	gwy_graph_model_add_curve(opodrcontrols->graph_model, opodrcontrols->cmodelunload);


	// Table of input data and results
	//opodrcontrols->ctrls = gtk_table_new(13,4,TRUE);
	opodrcontrols->ctrls = gtk_vbox_new (FALSE, CTRLS_SPACING);

	/* Info section */
	
	opodrcontrols->frame_info = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (opodrcontrols->frame_info), GTK_WIDGET (gwy_label_new_header("Info")));
	opodrcontrols->table_info = gtk_table_new (2, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (opodrcontrols->table_info), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (opodrcontrols->table_info), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (opodrcontrols->table_info), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (opodrcontrols->frame_info), opodrcontrols->table_info);

	i = 0;

	/* h_max */

	gtk_table_attach (GTK_TABLE (opodrcontrols->table_info), label_new_with_markup_left ("h<sub>max</sub>"), 0, 1, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	opodrcontrols->hmax = gtk_label_new (NULL);
	label_set_gdouble_format(opodrcontrols->hmax, args->fddata.hmax, DEPTH);
	gtk_misc_set_alignment (GTK_MISC (opodrcontrols->hmax), 0.0, 0.5);  
	gtk_table_attach (GTK_TABLE (opodrcontrols->table_info), opodrcontrols->hmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach (GTK_TABLE (opodrcontrols->table_info), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	/* F_max */

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_info), label_new_with_markup_left("F<sub>max</sub>"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	opodrcontrols->Fmax = gtk_label_new(NULL);
	label_set_gdouble_format(opodrcontrols->Fmax, args->fddata.Fmax, FORCE);
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->Fmax), 0.0, 0.5);  
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_info), opodrcontrols->Fmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_info), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	/* Parameters section */

	opodrcontrols->frame_parameters = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (opodrcontrols->frame_parameters), GTK_WIDGET (gwy_label_new_header("Parameters")));
	opodrcontrols->table_parameters = gtk_table_new (5, 4, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (opodrcontrols->table_parameters), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (opodrcontrols->table_parameters), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (opodrcontrols->table_parameters), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (opodrcontrols->frame_parameters), opodrcontrols->table_parameters);
	i = 0;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), label_new_with_markup_left("Fit range"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->from = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(opodrcontrols->from), "id", (gpointer)"from"); 
	gtk_entry_set_width_chars(GTK_ENTRY(opodrcontrols->from), 8);
        gtk_entry_set_text(GTK_ENTRY(opodrcontrols->from), "");
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), opodrcontrols->from, 1, 2, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(opodrcontrols->from, TRUE);
	g_signal_connect_swapped(opodrcontrols->from, "activate", G_CALLBACK(op_odr_range_from_changed),data);
/*	g_signal_connect_swapped(opodrcontrols->from, "focus-out-event", G_CALLBACK(range_from_changed),data); */

	opodrcontrols->to = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(opodrcontrols->to), "id", (gpointer)"to"); 
	gtk_entry_set_width_chars(GTK_ENTRY(opodrcontrols->to), 8); 
        gtk_entry_set_text(GTK_ENTRY(opodrcontrols->to), "");
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), opodrcontrols->to, 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(opodrcontrols->to, TRUE);
	g_signal_connect_swapped(opodrcontrols->to, "activate", G_CALLBACK(op_odr_range_to_changed),data);
/*	g_signal_connect_swapped(opodrcontrols->to, "focus-out-event", G_CALLBACK(range_to_changed),data); */

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), label_new_with_markup_left("nm"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	opodrcontrols->from_pct_Fmax = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(opodrcontrols->from_pct_Fmax), "id", (gpointer)"from_pct_Fmax"); 
	gtk_entry_set_width_chars(GTK_ENTRY(opodrcontrols->from_pct_Fmax), 8);
        gtk_entry_set_text(GTK_ENTRY(opodrcontrols->from_pct_Fmax), "");
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), opodrcontrols->from_pct_Fmax, 1, 2, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(opodrcontrols->from_pct_Fmax, TRUE);
	g_signal_connect_swapped(opodrcontrols->from_pct_Fmax, "activate", G_CALLBACK(op_odr_range_from_pct_Fmax_changed),data);
/*	g_signal_connect_swapped(opodrcontrols->from_pct_Fmax, "focus-out-event", G_CALLBACK(range_from_pct_Fmax_changed),data); */

	opodrcontrols->to_pct_Fmax = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(opodrcontrols->to_pct_Fmax), "id", (gpointer)"to_pct_Fmax"); 
	gtk_entry_set_width_chars(GTK_ENTRY(opodrcontrols->to_pct_Fmax), 8); 
        gtk_entry_set_text(GTK_ENTRY(opodrcontrols->to_pct_Fmax), "");
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), opodrcontrols->to_pct_Fmax, 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(opodrcontrols->to_pct_Fmax, TRUE);
	g_signal_connect_swapped(opodrcontrols->to_pct_Fmax, "activate", G_CALLBACK(op_odr_range_to_pct_Fmax_changed),data);
/*	g_signal_connect_swapped(opodrcontrols->to_pct_Fmax, "focus-out-event", G_CALLBACK(range_to_pct_Fmax_changed),data); */

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), label_new_with_markup_left("% F<sub>max</sub>"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);
	i++;

	/* beta */
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), label_new_with_markup_left("β"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->beta = gtk_entry_new();
//	g_object_set_data(G_OBJECT(opodrcontrols->beta), "id", (gpointer)"beta");
	gtk_entry_set_width_chars(GTK_ENTRY(opodrcontrols->beta), 8);
	entry_set_gdouble_format(opodrcontrols->beta, data->args->opodrdata.beta, "%.03f");
	gwy_widget_set_activate_on_unfocus(opodrcontrols->beta, TRUE);
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), opodrcontrols->beta, 1,2, i,i+1, GTK_FILL, 0,0,0);
	g_signal_connect_swapped(opodrcontrols->beta, "activate", G_CALLBACK(opodr_beta_changed), data);

	opodrcontrols->reset_beta = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON(opodrcontrols->reset_beta), gtk_image_new_from_stock (GTK_STOCK_REVERT_TO_SAVED, GTK_ICON_SIZE_BUTTON));
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_parameters), opodrcontrols->reset_beta, 3,4,i,i+1, GTK_FILL, 0,0,0);
	g_signal_connect_swapped (opodrcontrols->reset_beta, "clicked", G_CALLBACK (opodr_reset_beta), data);
	i++;

	/* Fit button */
	
	opodrcontrols->button_fit = gtk_button_new_with_mnemonic(gwy_sgettext("verb|_Fit"));
	gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_fit), gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (opodrcontrols->button_fit, "clicked", G_CALLBACK (op_odr_fit_and_update), data);


	/* Results section */

	opodrcontrols->frame_results = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (opodrcontrols->frame_results), GTK_WIDGET (gwy_label_new_header("Results")));

	opodrcontrols->scrolledwindow_results = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (opodrcontrols->scrolledwindow_results), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (opodrcontrols->frame_results), opodrcontrols->scrolledwindow_results);
	
	opodrcontrols->table_results = gtk_table_new (13, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (opodrcontrols->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (opodrcontrols->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (opodrcontrols->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (opodrcontrols->scrolledwindow_results), opodrcontrols->table_results);
	i = 0;

	opodrcontrols->fitinfo = gtk_label_new(NULL);
	gtk_label_set_line_wrap(GTK_LABEL(opodrcontrols->fitinfo), TRUE);
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->fitinfo, 0, 3, i, i+2, GTK_FILL, GTK_FILL, 0, 0); 
	i+=2;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("h<sub>p</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->hp = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->hp), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->hp, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("m"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->m = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->m), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->m, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("α"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->alpha = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->alpha), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->alpha, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
//	g_snprintf(str, sizeof(str), "mN.nm<sup>-%.2g</sup>", data->args->opodrdata.m);
//	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left(str), 2, 3, i, i+1,  GTK_FILL, 0, 0, 0); 
	i++;


	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("SS<sub>res</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->SSres = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->SSres), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->SSres, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("R<sup>2</sup>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->R2 = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->R2), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->R2, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("R<sup>2</sup><sub>adj</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->R2adj = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->R2adj), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->R2adj, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("χ<sup>2</sup> fit"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);  
	opodrcontrols->chi2 = gtk_label_new(NULL);  
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->chi2), 0.0, 0.5);  
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->chi2, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);  
	i++; 

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("ε"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->eps = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->eps), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->eps, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("h<sub>c</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->hc = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->hc), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->hc, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("S"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->S = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->S), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->S, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("mN/nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("A<sub>p</sub>(h<sub>c</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->Aphc = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->Aphc), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->Aphc, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("nm<sup>2</sup>"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	opodrcontrols->extrapol = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->extrapol, 0, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	/* R not available yet */

	/* gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("R fit"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);  */
	/* opodrcontrols->R = gtk_label_new(NULL);  */
	/* gtk_misc_set_alignment(GTK_MISC(opodrcontrols->R), 0.0, 0.5);  */
	/* gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->R, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);  */
	/* i++; */

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("H<sub>IT</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->Hit = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->Hit), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->Hit, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("MPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("E<sub>r</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->Er = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->Er), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->Er, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("E<sub>IT</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->Eit = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->Eit), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->Eit, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("Fitted range"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	opodrcontrols->range = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->range), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->range, 1, 2, i, i+1, GTK_EXPAND |GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	opodrcontrols->range_pct_Fmax = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(opodrcontrols->range_pct_Fmax), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), opodrcontrols->range_pct_Fmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(opodrcontrols->table_results), label_new_with_markup_left("% F<sub>max</sub>"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	/* Log, uncertainties, save buttons */

	opodrcontrols->button_log = gtk_button_new_with_mnemonic("Show _log");
	gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_log), gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (opodrcontrols->button_log, "clicked", G_CALLBACK (op_odr_show_log), data);
	gtk_widget_set_sensitive(opodrcontrols->button_log, FALSE);
	opodrcontrols->log_error_window = NULL;
	opodrcontrols->log_report_window = NULL;

	opodrcontrols->button_unc = gtk_button_new_with_mnemonic("_Uncertainties");
	gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_unc), gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (opodrcontrols->button_unc, "clicked", G_CALLBACK (op_odr_uncertainty), data);
	gtk_widget_set_sensitive(opodrcontrols->button_unc, FALSE);

	opodrcontrols->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (opodrcontrols->button_save, "clicked", G_CALLBACK (op_odr_save_data), args);


	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->frame_info, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->frame_parameters, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->button_fit, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->frame_results, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->button_log, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->button_unc, FALSE, FALSE, 0); 
	gtk_box_pack_start (GTK_BOX (opodrcontrols->ctrls), opodrcontrols->button_save, FALSE, FALSE, 0);

	/* Zooming */

	opodrcontrols->vbox_zoom_buttons = gtk_vbox_new (TRUE, 0);
	opodrcontrols->vbox_zoom_outer = gtk_vbox_new (FALSE, 0);

	opodrcontrols->button_zoom_in = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_zoom_in), gtk_image_new_from_stock (GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (opodrcontrols->vbox_zoom_buttons), opodrcontrols->button_zoom_in, TRUE, TRUE, 0);
	g_signal_connect_swapped(opodrcontrols->button_zoom_in, "clicked", G_CALLBACK (op_odr_zoom_in), data);
	
	opodrcontrols->button_zoom_out = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_zoom_out), gtk_image_new_from_stock (GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (opodrcontrols->vbox_zoom_buttons), opodrcontrols->button_zoom_out, TRUE, TRUE, 0);
	g_signal_connect_swapped(opodrcontrols->button_zoom_out, "clicked", G_CALLBACK (op_odr_zoom_out), data);

	opodrcontrols->button_zoom_restore = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_zoom_restore), gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (opodrcontrols->vbox_zoom_buttons), opodrcontrols->button_zoom_restore, TRUE, TRUE, 0);
	g_signal_connect_swapped(opodrcontrols->button_zoom_restore, "clicked", G_CALLBACK (op_odr_zoom_restore), data);

	gtk_box_pack_start (GTK_BOX (opodrcontrols->vbox_zoom_outer), opodrcontrols->vbox_zoom_buttons, TRUE, FALSE, 0);
  
	gtk_box_pack_start (GTK_BOX (opodrcontrols->hbox_graph), opodrcontrols->graph, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->hbox_graph), opodrcontrols->vbox_zoom_outer, FALSE, FALSE, 0);


	//zoom and unzoom buttons
	/* opodrcontrols->button_zoom_in = gtk_button_new(); */
	/* gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_zoom_in), gtk_image_new_from_stock (GWY_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON)); */
	/* g_signal_connect_swapped(opodrcontrols->button_zoom_in, "clicked", G_CALLBACK (zoom_in), data); */

	/* opodrcontrols->button_zoom_out = gtk_button_new(); */
	/* gtk_button_set_image (GTK_BUTTON (opodrcontrols->button_zoom_out), gtk_image_new_from_stock (GWY_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON)); */
	/* g_signal_connect_swapped(opodrcontrols->button_zoom_out, "clicked", G_CALLBACK (zoom_out), data); */

	/* vbox = gtk_vbox_new(FALSE,FALSE); */
	/* gtk_box_pack_start (GTK_BOX(vbox), opodrcontrols->graph, TRUE, TRUE, 0); */
	/* gtk_box_pack_start (GTK_BOX(vbox), opodrcontrols->button_zoom_in, FALSE, FALSE, 0); */
	/* gtk_box_pack_start (GTK_BOX(vbox), opodrcontrols->button_zoom_out, FALSE, FALSE, 0); */


	//final assembly

	opodrcontrols->opodr_gui = gtk_hbox_new (FALSE, CTRLS_SPACING);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->opodr_gui), opodrcontrols->ctrls , FALSE, FALSE, 0);
//	gtk_box_pack_start (GTK_BOX (opodrcontrols->opodr_gui), opodrcontrols->graph, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (opodrcontrols->opodr_gui), opodrcontrols->hbox_graph, TRUE, TRUE, 0);

	gtk_widget_show(opodrcontrols->opodr_gui);


	return opodrcontrols->opodr_gui;
}

void
op_odr_recalculate(Data *data)
{
    recalculate_Eit(data);
    op_odr_propagate_uncertainties(data);
}

static void
update_models_op_odr(OPODRControls *opodrcontrols)
{
    GwyGraphCurveModel *cmodel;
    
    opodrcontrols->graph_model = gwy_graph_get_model (GWY_GRAPH(opodrcontrols->graph));
    cmodel = gwy_graph_model_get_curve_by_description (opodrcontrols->graph_model, "Unloading");
    if (cmodel)
	opodrcontrols->cmodelunload = cmodel;
    cmodel = gwy_graph_model_get_curve_by_description (opodrcontrols->graph_model, "Fit");
    if (cmodel)
	opodrcontrols->cmodelSfit = cmodel;
}

static void
op_odr_zoom_in(Data *data)
{
    OPODRControls *opodrcontrols;

    opodrcontrols = data->controls->opodrcontrols;
    
    rs_zoom_in (GWY_GRAPH (opodrcontrols->graph), &(opodrcontrols->zoomstack), FALSE); /* do not rescale y */
    update_models_op_odr(opodrcontrols);
}

static void
op_odr_zoom_out(Data *data)
{
    OPODRControls *opodrcontrols;
    
    opodrcontrols = data->controls->opodrcontrols;
    
    rs_zoom_out (GWY_GRAPH (opodrcontrols->graph), &(opodrcontrols->zoomstack));
    update_models_op_odr(opodrcontrols);

}

void
op_odr_zoom_restore(Data *data)
{
    OPODRControls *opodrcontrols;

    opodrcontrols = data->controls->opodrcontrols;
    
    rs_zoom_restore (GWY_GRAPH (opodrcontrols->graph), &(opodrcontrols->zoomstack));
    update_models_op_odr(opodrcontrols);
}

#endif
