#ifndef NOFORTRAN

#include "tool_op_odr_unc.h"
#include "tool_op_odr.h"

#include "controls.h"
#include "datatypes.h"
#include "fddata.h"
#include "file-utils.h"
#include "fit-utils.h"
#include "indent-toolbox-common.h"
#include "mc-utils.h"

#define _USE_MATH_DEFINES
#include <math.h>
#include <string.h>

static const gchar *logfnm = "fit.log.op.unc";
static const gchar *mclogfnm = "fit.log.op.mc";

void init_shifted_OPODRdata(OPODRdata *src, OPODRdata *dest, gdouble dh)
{
	init_OPODRdata(dest);

	// shift fitting range  in h-mode by dh
	// don't shift range for F-mode
	dest->range[0]= src->range[0]-dh;
	dest->range[1]= src->range[1]-dh;
	dest->from = dest->range[0];
	dest->to = dest->range[1];

	dest->from_pct_Fmax = src->from_pct_Fmax;
	dest->to_pct_Fmax = src->to_pct_Fmax;
	dest->range_pct_Fmax[0] = src->range_pct_Fmax[0];
	dest->range_pct_Fmax[1] = src->range_pct_Fmax[1];

	dest->Finput = src->Finput;

	dest->logfnm = create_log_filename_path(logfnm);

	// copy beta
	dest->beta = src->beta;
}

void 
op_odr_propagate_uncertainties_calc (Args *args)
{
	OPODRUncdata *unc;
	OPODRdata *opodr;
	FDdata *fd;
	Instdata *inst;
	gdouble uh, uF;
	gdouble umh, umF, uhph, uhpF;
	gdouble uSh, uSF;
	gdouble z, depsdm, zh, zF;
	gdouble uepsh, uepsF;
	gdouble epsh, epsF, mh, mF, depsdmh, depsdmF;
	gdouble um, uhp, ueps, uS;
	gdouble beta[3], std_beta[3], covariance[3];
	gint ifix[3];
	gint istart, iend,ndata;
	GwyDataLine *x, *y;
	gdouble ux,uy;
	gdouble dAdh;
	gdouble m,S,eps, hc, hp, hmax,Fmax, Aphc,Hit,Er,Eit, nu,nui,Ei;
	gdouble covmhph, covmhpF, covmhp;
	gdouble covShch, covShcF, covShc;
	gchar *unc_logfnm;
	gdouble SSres, R2, R2adj, chi2;

	unc = &(args->opodrunc);
	opodr = &(args->opodrdata);
	fd = &(args->fddata);
	inst = &(args->instdata);


	//make copies to simplify code
	uh = unc->uh;
	uF = unc->uF;
	m = opodr->m;
	eps = opodr->eps;
	S = opodr->S;
	hp = opodr->hp;
	hc = opodr->hc;
	hmax = fd->hmax;
	Fmax = fd->Fmax;
	Aphc = opodr->Aphc;
	Hit = opodr->Hit;
	Eit = opodr->Eit;
	Er = opodr->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;

	unc_logfnm =  create_log_filename_path(logfnm);

	printf("logfnm %s \n", unc_logfnm);
	if (VERBOSE)
		printf(" \n\nPropagate uncertainties\n");


	ifix[0] = 1;
	ifix[1] = 1;
	ifix[2] = 1;

	/*   u(m)    */

	/* prepare odr fit */

	beta[0] = opodr->alpha;
	beta[1] = opodr->hp;
	beta[2] = opodr->m;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (opodr->Finput)
		range_to_indices(opodr->range_pct_Fmax[0]*0.01*fd->Fmax, opodr->range_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(opodr->range[0], opodr->range[1], fd->hunload, TRUE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	/* fit with only uh */
	ux = uh;
	uy = 0;
	beta[0] = opodr->alpha;
	beta[1] = opodr->hp;
	beta[2] = opodr->m;

	odr_run_fit(x, y, beta, ifix, unc_logfnm, ux, uy, std_beta, covariance, &SSres, &R2, &R2adj, &chi2,FALSE); /* do not rescale */
	mh = beta[2];
	umh = std_beta[2];
	uhph = std_beta[1];
	covmhph = covariance[0]; 

	if (VERBOSE)
		printf(" fit with only uh:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umh, uhph, covmhph/umh/uhph);	       
	/*   u(eps)    */
	epsh = mh * (1 - 2*(mh-1) * tgamma(mh / 2 / (mh - 1)) / tgamma(1.0 / 2.0 / (mh - 1)) / sqrt(M_PI));
	zh = mh/(mh-1)/2;
	depsdmh = epsh/mh - mh/(mh -1) * (1 -epsh/mh ) -mh/2/(mh-1)/(mh-1)*(epsh/mh -1)* (psi(zh)-psi(zh-1./2));
	uepsh = fabs(depsdmh) *umh;
	if (VERBOSE){
		printf("depsdm %g  z %g  \n", depsdmh, zh);
		printf(" u(eps;h) %g  urel(eps;h) %g \n", uepsh,  uepsh/eps );
	}


	/* fit with only uF */
	ux = 0;
	uy = uF;
	odr_run_fit(x, y, beta, ifix, unc_logfnm, ux, uy, std_beta, covariance, &SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	mF = beta[2];
	umF = std_beta[2];
	uhpF = std_beta[1];
	covmhpF = covariance[0]; 

	if (VERBOSE)
		printf(" fit with only uF:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umF, uhpF, covmhpF/umF/uhpF);	       

	unc->umuh = umh;
	unc->umuF = umF;

	/*   u(eps)    */
	epsF = mF * (1 - 2*(mF-1) * tgamma(mF / 2 / (mF - 1)) / tgamma(1.0 / 2.0 / (mF - 1)) / sqrt(M_PI));
	zF = mF/(mF-1)/2;
	depsdmF = epsF/mF - mF/(mF -1) * (1 -epsF/mF ) -mF/2/(mF-1)/(mF-1)*(epsF/mF -1)* (psi(zF)-psi(zF-1./2));
	uepsF = fabs(depsdmF) *umF;
	if (VERBOSE){
		printf("depsdm %g  z %g  \n", depsdmF, zF);
		printf(" u(eps;h) %g u(eps;F) %g  urel(eps;h) %g urel(eps;F) %g \n", uepsh, uepsF, uepsh/eps, uepsF/eps);
	}

	/*    u(S)   */
	uSh = sqrt(umh*umh/m/m +
			uh *uh/(hmax-hp)/(hmax-hp) +
			uhph*uhph/(hmax-hp)/(hmax-hp) +
			2*covmhph/m/(hmax-hp)
		  ) * S; 

	uSF = sqrt(umF*umF/m/m +    
		       	uF*uF/Fmax/Fmax  +    
			uhpF*uhpF/(hmax-hp)/(hmax-hp)+
			2*covmhpF/m/(hmax-hp)
		  ) * S; 
	unc->uSuh = uSh;
	unc->uSuF = uSF;
	/* u(S;h) = S *sqrt [ (umh/m)^2 + (uh/(hmax-hp))^2 + (uhph/(hmax-hp))^2  + 2 cov(m,hp;h)/(m*(hmax-hp))]
	 * u(S;F) = S *sqrt [ (umF/m)^2 + (uFmax/Fmax)^2   + (uhpF/(hmax-hp))^2  + 2 cov(m,hp;F)/(m*(hmax-hp))]
	 */

	if (VERBOSE){
		printf(" u(S;h) %g u(S;F) %g  urel(S;h) %g urel(S;F) %g \n", uSh, uSF, uSh/S, uSF/S);
	}

	/*   u(hc)   */
	unc->uhcuh = sqrt((hmax-hp)*(hmax-hp)* (epsh/mh - depsdmh)*(epsh/mh-depsdmh)/mh/mh *umh*umh + 
			(1-epsh/mh)*(1-epsh/mh)*uh*uh + 
			epsh*epsh/mh/mh*uhph*uhph +
			2*(hmax-hp)*epsh/mh/mh*(epsh/mh-depsdmh)*covmhph);
	unc->uhcuF = sqrt((hmax-hp)*(hmax-hp)* (epsF/mF - depsdmF)*(epsF/mF-depsdmF)/mF/mF *umF*umF + 
			epsF*epsF/mF/mF*uhpF*uhpF +
			2*(hmax-hp)*epsF/mF/mF*(epsF/mF-depsdmF)*covmhpF);
	
	/* u(hc;h) ^2 = [(hmax-hp) (eps/m- deps/dm)/m  u(m;h) ]^2 + [(1-eps/m) u(hmax)]^2 + [eps/m u(hp;h)]^2 + 
	                * 2 (hmax-hp)eps/m^2 (eps/m - deps/dm) cov(m,hp;h) */
	/* u(hc;F) ^2 = [(hmax-hp) (eps/m- deps/dm)/m  u(m;F) ]^2 +                         [eps/m u(hp;F)]^2 + 
	                * 2 (hmax-hp)eps/m^2 (eps/m - deps/dm) cov(m,hp;F) */

	if (VERBOSE)
		printf(" u(hc;h) %g u(hc;F) %g  urel(hc;h) %g urel(hc;F) %g \n", unc->uhcuh, unc->uhcuF, unc->uhcuh/hc, unc->uhcuF/hc);

	/*    u(A)   */
	dAdh =  eval_area_derivative(&(args->area), hc) ;
	unc->uAuh = dAdh * unc->uhcuh;
	unc->uAuF = dAdh * unc->uhcuF;
	if (VERBOSE)
		printf(" u(A;h) %g u(A;F) %g  urel(A;h) %g urel(A;F) %g \n", unc->uAuh, unc->uAuF, unc->uAuh/Aphc, unc->uAuF/Aphc);

	/*     u(H_IT)   */
	unc->uHituh = Hit/Aphc*dAdh*unc->uhcuh;
	unc->uHituF = Hit *sqrt(uF*uF/Fmax/Fmax + dAdh*dAdh/Aphc/Aphc*unc->uhcuh*unc->uhcuh);
	if (VERBOSE)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);


	/*     u(E_r)   */

	covShch = - S *(1-eps)/(hmax-hp)*uh*uh  - S*(hmax - hp)/m *depsdmh* umh*umh +
		S*eps/(hmax-hp)*uhph*uhph + S*(eps/m-depsdmh)*covmhph;
	covShcF =  - S*(hmax - hp)/m *depsdmF* umF*umF +
		S*eps/(hmax-hp)*uhpF*uhpF + S*(eps/m-depsdmF)*covmhpF;
	/*cov(S,hc;h) = -S(1-eps)/(hmax-hp)*uh^2 -S(hmax-hp)/m *(deps/dm) *u(m;h)^2 + S*eps/(hmax-hp)*u(hp;h)^2 +S (eps/m-(deps/dm))cov(m,hp;h)
	 *cov(S,hc;F) =                          -S(hmax-hp)/m *(deps/dm) *u(m;F)^2 + S*eps/(hmax-hp)*u(hp;F)^2 +S (eps/m-(deps/dm))cov(m,hp;F)
	 */

	unc->uEruh = Er*sqrt( uSh*uSh/S/S + unc->uAuh*unc->uAuh/Aphc/Aphc/4 - 1/S/Aphc*dAdh*covShch); 
	unc->uEruF = Er*sqrt( uSF*uSF/S/S + unc->uAuF*unc->uAuF/Aphc/Aphc/4 - 1/S/Aphc*dAdh*covShcF); 
	/* u(Er;h) = sqrt[ (Er/S)^2 u(S;h)^2 + (Er/2A)^2 u(A;h)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc;h)]
	 * u(Er;F) = sqrt[ (Er/S)^2 u(S;F)^2 + (Er/2A)^2 u(A;F)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc;F)]
	 */
	if (VERBOSE)
		printf(" u(Er;h) %g u(Er;F) %g  urel(Er;h) %g urel(Er;F) %g \n", unc->uEruh, unc->uEruF, unc->uEruh/Er, unc->uEruF/Er);


	/*     u(E_IT)   */
	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; // Eit^2/Er^2 /(1-nu^2) u(Er;h)
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; // Eit^2/Er^2 /(1-nu^2) u(Er;F)


	if (VERBOSE){
		printf("u(Eit;h) %g  u(Eit;F) %g  urel(Eit;h) %g  urel(Eit;F) %g\n", unc->uEituh, unc->uEituF, unc->uEituh/Eit, unc->uEituF/Eit);
	}


	

	// total uncertainties
	// should take um as given by odr_wrap, otherwise doublecounting
	/* fit with both uh and uF */
	ux = uh;
	uy = uF;
	odr_run_fit(x, y, beta, ifix, unc_logfnm, ux, uy, std_beta, covariance, &SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */

	if (VERBOSE){
		printf(" fit with both ux %g and uy %g \n", ux,uy);
	}
	um = std_beta[2];
	uhp = std_beta[1];
	covmhp = covariance[0]; 

	unc->um = um;
	unc->uhp = uhp;
	unc->covmhp = covmhp;
	if (VERBOSE)
		printf(" fit with both uh and uF:\n  alpha %g hp %g m %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], um, uhp, covmhp/um/uhp);	       

	/*   u(eps)    */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * (1 -eps/m ) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	ueps = fabs(depsdm) *um;
	unc->ueps = ueps;
	unc->depsdm = depsdm;

	if (VERBOSE){
		printf("eps %g    \n",eps);
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps) %g    urel(eps) %g  \n", ueps, ueps/eps);
	gdouble e1,e2, e3; 
         e2 = m * (1 - 2*(m-1) * tgamma(m / 2 / (m - 1)) / tgamma(1.0 / 2.0 / (m - 1)) / sqrt(M_PI));
         e1 = (m-0.1) * (1 - 2*((m-0.1)-1) * tgamma((m-0.1) / 2 / ((m-0.1) - 1)) / tgamma(1.0 / 2.0 / ((m-0.1) - 1)) / sqrt(M_PI));
         e3 = (m+0.1) * (1 - 2*((m+0.1)-1) * tgamma((m+0.1) / 2 / ((m+0.1) - 1)) / tgamma(1.0 / 2.0 / ((m+0.1) - 1)) / sqrt(M_PI));
		printf(" eps(m-dm) %g eps (m) %g eps (m+dm) %g \n", e1, e2,e3);
		printf(" left deps %g right depts %g symm deps %g \n", (e2-e1)/0.1, (e3-e2)/0.1, (e3-e1)/0.2);
	}


	/*    u(S)   */
	uS = sqrt(um*um/m/m  + uh *uh/(hmax-hp)/(hmax-hp) +  uF*uF/Fmax/Fmax +
			uhp*uhp/(hmax-hp)/(hmax-hp) + 2*covmhp/m/(hmax-hp)) * S; 
	unc->uS = uS;
	/* u(S) = S *sqrt [ (umh/m)^2 +  (uhmax/(hmax-hp))^2 + (uFmax/Fmax)^2 +  (uhp/(hmax-hp))^2 + 2 cov(m,hp)/(m*(hmax-hp)) ]
	 */

	if (VERBOSE){
		printf(" u(S) %g    urel(S) %g \n", uS , uS/S);
	}

	/*   u(hc)   */
	unc->uhc = sqrt((hmax-hp)*(hmax-hp)* (eps/m - depsdm)*(eps/m-depsdm)/m/m *um*um + 
			(1-eps/m)*(1-eps/m)*uh*uh + 
			eps*eps/m/m*uhp*uhp +
			2*(hmax-hp)*eps/m/m*(eps/m-depsdm)*covmhp);
	
	/* u(hc) ^2 = [(hmax- hp) (eps/m- deps/dm)/m  u(m) ]^2 + [(1-eps/m) u(hmax)]^2 + [eps/m u(hp)]^2 + 2 (hmax-hp)eps/m^2 (eps/m - deps/dm) cov(m,hp) */

	if (VERBOSE){
		printf(" u(hc) %g    urel(hc) %g \n", unc->uhc, unc->uhc/hc);
		/*printf(" %g %g %g %g sum %g \n", (hmax-hc)*(hmax-hc)* (eps/m - depsdm)*(eps/m-depsdm)/m/m *um*um ,
				(1-eps/m)*(1-eps/m)*uh*uh ,
				eps*eps/m/m*uhp*uhp,
				2*(hmax-hp)*eps/m/m*(eps/m-depsdm)*covmhp,
				((hmax-hc)*(hmax-hc)* (eps/m - depsdm)*(eps/m-depsdm)/m/m *um*um + 
				 (1-eps/m)*(1-eps/m)*uh*uh + 
				 eps*eps/m/m*uhp*uhp +
				 2*(hmax-hp)*eps/m/m*(eps/m-depsdm)*covmhp)
		      );
*/
	}



	/*    u(A)   */
	dAdh =  eval_area_derivative(&(args->area), hc) ;
	unc->uA = dAdh * unc->uhc;
	unc->uA = dAdh * unc->uhc;
	if (VERBOSE)
		printf(" u(A) %g    urel(A) %g  \n", unc->uA, unc->uA/Aphc);

	/*     u(H_IT)   */
	unc->uHit = Hit*sqrt(uF*uF/Fmax/Fmax + unc->uA*unc->uA/opodr->Aphc/opodr->Aphc); 
	if (VERBOSE)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);

	/*     u(E_r)   */

	covShc = - S *(1-eps)/(hmax-hp)*uh*uh  - S*(hmax - hp)/m *depsdm* um*um +
		S*eps/(hmax-hp)*uhp*uhp + S*(eps/m-depsdm)*covmhp;

	/*cov(S,hc;h) = -S(1-eps)/(hmax-hp)*uh^2 -S(hmax-hp)/m *(deps/dm) *u(m)^2 + S*eps/(hmax-hp)*u(hp)^2 +S (eps/m-(deps/dm))cov(m,hp)
	*/
	
	unc->uEr = Er*sqrt( uS*uS/S/S +unc->uA*unc->uA/Aphc/Aphc/4- 1/S/Aphc*dAdh*covShc); 
	/* u(Er) = sqrt[ (Er/S)^2 u(S)^2 + (Er/2A)^2 u(A)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc)]
	 */
	if (VERBOSE)
		printf(" u(Er) %g   urel(Er) %g  \n", unc->uEr, unc->uEr/Er);


	/*     u(E_IT)   */
	//contribution from h,F
	unc->uEit = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEr; // Eit^2/Er^2 /(1-nu^2) uEr

	//contribution from nu, nui, Ei
	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*inst->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *inst->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/Ei/Ei *inst->uEi)*1e9; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (VERBOSE){
		printf("u(Eit;h,F) %g    urel(Eit;hF) %g  \n", unc->uEit, unc->uEit/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}

	unc->uEit = sqrt( unc->uEit*unc->uEit  +unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
	if (VERBOSE)
		printf("u(Eit;hFnuetc) %g    urel(Eit; hF_nu_etc) %g \n", unc->uEit, unc->uEit/Eit);

	if (VERBOSE)
		printf(" \nEnd propagation of uncertainties\n\n");


	g_object_unref(x);
	g_object_unref(y);
	g_free(unc_logfnm);

}

gboolean op_odr_uncertainty_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus ) 
{
	FILE *file;
	OPODRdata *opodrdata;
	OPODRUncdata *opodrunc;
	Instdata *instdata;
	gint j;

	opodrdata = &(args->opodrdata);
	opodrunc = &(args->opodrunc);
	instdata = &(args->instdata);

	if (args->fddata.hunload->res == 0 || args->fddata.Funload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of Oliver Pharr analysis (ODR fit) \n");
	fprintf(file, "# u(depth): %g nm\n", opodrunc->uh);
	fprintf(file, "# u(load): %g nm\n", opodrunc->uF);
	fprintf(file, "#beta: %g \n", opodrdata->beta);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, instdata->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, instdata->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, instdata->uEi*1e-9); //TODO units
	fprintf(file, "#Fitted range: ");
	if (args->opodrdata.Finput)
		fprintf(file, "	[%g , %g ] %% Fmax \n", opodrdata->range_pct_Fmax[0], opodrdata->range_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",opodrdata->range[0], opodrdata->range[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opodrdata.Finput)
		fprintf(file, "[%g , %g ] nm \n",opodrdata->range[0], opodrdata->range[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", opodrdata->range_pct_Fmax[0], opodrdata->range_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", opodrdata->nfitdata);
	fprintf(file, "#\n");

	fprintf(file, "#Results of Oliver Pharr analysis (ODR fit) \n");
	fprintf(file, "#hc: %g nm\n", opodrdata->hc);
	fprintf(file, "#Ap(hc): %g nm^2\n", opodrdata->Aphc);
	fprintf(file, "#Er: %g GPa\n", opodrdata->Er);
	fprintf(file, "#Hit: %g GPa\n", opodrdata->Hit);
	fprintf(file, "#Eit: %g GPa\n", opodrdata->Eit);
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(m) =  %g  \n", opodrunc->um);
	fprintf(file, "#u(hp) =  %g nm \n", opodrunc->uhp);
	fprintf(file, "#cov(m,hp) =  %g nm \n", opodrunc->covmhp);
	fprintf(file, "#u(eps) = %g  \n", opodrunc->ueps); 
	fprintf(file, "#depsdm = %g  \n", opodrunc->depsdm); 
	fprintf(file, "#u(S) =  %g mN/nm \n", opodrunc->uS);
	fprintf(file, "#u(hc) =  %g nm \n", opodrunc->uhc);
	fprintf(file, "#u(Ap(hc)) =  %g nm^2 \n", opodrunc->uA);
	fprintf(file, "#u(Hit) =  %g MPa \n", opodrunc->uHit);
	fprintf(file, "#u(Er) =  %g GPa \n", opodrunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", opodrunc->uEit);
	fprintf(file, "#\n");


	fprintf(file, "#Contributions to uncertainty of m\n");
	fprintf(file, "#u(m;uh) = %g \n", opodrunc->umuh);
	fprintf(file, "#u(m;uF) = %g \n", opodrunc->umuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of S\n");
	fprintf(file, "#u(S;uh) = %g mN/nm \n", opodrunc->uSuh);
	fprintf(file, "#u(S;uF) = %g mN/nm \n", opodrunc->uSuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of hc\n");
	fprintf(file, "#u(hc;uh) = %g nm \n", opodrunc->uhcuh);
	fprintf(file, "#u(hc;uF) = %g nm \n", opodrunc->uhcuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Ap(hc)\n");
	fprintf(file, "#u(Ap(hc);uh) = %g nm^2 \n", opodrunc->uAuh);
	fprintf(file, "#u(Ap(hc);uF) = %g nm^2 \n", opodrunc->uAuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Hit\n");
	fprintf(file, "#u(Hit;uh) = %g MPa \n", opodrunc->uHituh);
	fprintf(file, "#u(Hit;uF) = %g MPa \n", opodrunc->uHituF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", opodrunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", opodrunc->uEruF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", opodrunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", opodrunc->uEituF);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", opodrunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", opodrunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", opodrunc->uEituEi);
	fprintf(file, "#\n");
	fprintf(file, "#\n");
	fprintf(file, "#\n");


	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opodrunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, opodrunc->Ec[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the H corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#H/MPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opodrunc->Hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, opodrunc->Hc[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the area corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Aphc/nm^2 \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opodrunc->Ac[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, opodrunc->Ac[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the contact depth corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#hc/nm \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opodrunc->hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, opodrunc->hc[j+CONTACTMAX]);
	}

	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}

void
op_odr_fit_shift_contact(OPODRdata *opodrdata, FDdata *fddata, Instdata *instdata, Area *area, gdouble *Ec, gdouble *Hc, gdouble *Ac, gdouble *hc, gint j)
{
	OPODRdata opodr;
	FDdata fdc;
	gdouble dh;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;

	Ec[j+CONTACTMAX] = -1;
	Hc[j+CONTACTMAX] = -1;
	Ac[j+CONTACTMAX] = -1;
	hc[j+CONTACTMAX] = -1;


	if (fddata->i_contact_load+j < 0)
		return;

	// create shifted FDdata
	dh = create_shifted_FDdata(fddata, &fdc, j);

	// create working copy of OPODRdata 
	init_shifted_OPODRdata(opodrdata, &opodr, dh);
	printf("logfnm %s \n", opodr.logfnm);

	//check range
	
	if (opodrdata->Finput){
		if (opodr.from_pct_Fmax ==  opodr.to_pct_Fmax){
			return;
		}
	}
	else{
		if (opodr.from ==  opodr.to){
			return;
		}
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (opodrdata->Finput)
		range_to_indices(opodr.from_pct_Fmax*0.01*fdc.Fmax, opodr.to_pct_Fmax*0.01*fdc.Fmax, fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(opodr.from, opodr.to, fdc.hunload, TRUE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, no fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);


	// perform fit and get results
	op_odr_fit(x,y,&opodr, &fdc, instdata, area, NULL);

	printf("j %d Su %g Er %g H %g A %g \n", j, opodr.S, opodr.Er, opodr.Hit, opodr.Aphc);

	// clean up
	g_free(opodr.logfnm);
	g_object_unref(x);
	g_object_unref(y);
	destroy_FDdata(&fdc);

	Ec[j+CONTACTMAX] = opodr.Er;
	Hc[j+CONTACTMAX] = opodr.Hit;
	Ac[j+CONTACTMAX] = opodr.Aphc;
	hc[j+CONTACTMAX] = opodr.hc;

}

#define NPARAM 9  //hc, Aphc, Hit, Er, Eit, S, m, hp, eps

void
init_OPODRMCdata (OPODRMCdata *opodrmc, gint Nmc)
{
	gint i;

	//number of iterations
	opodrmc->Nmc = Nmc;
	printf("initialize for %d iterations \n", opodrmc->Nmc);

	//how much had to be skipped 
	opodrmc->skipmc = 0;

	//dynamically allocate
	opodrmc->opodrmc = (OPODRdata *)g_malloc(opodrmc->Nmc*sizeof(OPODRdata));
	opodrmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	opodrmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	opodrmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	opodrmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	opodrmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	opodrmc->mcnstat = (gint)floor(3.49*cbrt(opodrmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		opodrmc->mcdd[i] = (gdouble *) g_malloc(opodrmc->Nmc*sizeof(gdouble));
		opodrmc->mcxhist[i] = (gdouble *) g_malloc(opodrmc->mcnstat*sizeof(gdouble));
		opodrmc->mcyhist[i] = (gdouble *) g_malloc(opodrmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_OPODRMCdata (OPODRMCdata *opodrmc)
{
	gint i;


	//free everything
	g_free(opodrmc->mcavg);
	g_free(opodrmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(opodrmc->mcdd[i]);
		g_free(opodrmc->mcxhist[i]);
		g_free(opodrmc->mcyhist[i]);
	}

	g_free(opodrmc->mcdd);
	g_free(opodrmc->mcxhist);
	g_free(opodrmc->mcyhist);

	opodrmc->mcnstat = 0;
	opodrmc->skipmc = 0;
	
	for (i = 0; i < opodrmc->Nmc; i++){
		g_free(opodrmc->opodrmc[i].logfnm);
	}

	g_free(opodrmc->opodrmc);

}

/* tady kdyz uz, tak deklarovat nejake konstantni pole a volat pak primo cislem z toho */
static	gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{

	switch (i){
		case 0:
			g_snprintf(str,nstr,"h_c/nm");
			break;
		case 1:
			g_snprintf(str,nstr,"A_p(h_c)/nm^2");
			break;
		case 2:
			g_snprintf(str,nstr,"H_IT/MPa");
			break;
		case 3:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 4:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		case 5:
			g_snprintf(str,nstr,"S/(mN/nm)");
			break;
		case 6:
			g_snprintf(str,nstr,"m");
			break;
		case 7:
			g_snprintf(str,nstr,"h_p/nm");
			break;
		case 8:
			g_snprintf(str,nstr,"eps");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}

gboolean op_odr_mc_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	OPODRdata *opodr;
	gint i,j;
	gchar str[200];
	gboolean is;

	if (args->fddata.hunload->res == 0 || args->fddata.Funload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of Oliver Pharr analysis (ODR fit)\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->opodrunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->opodrunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->opodrmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of Oliver Pharr analysis (ODR fit) \n");
	fprintf(file, "#beta: %g \n", args->opodrdata.beta);
	fprintf(file, "#nu: %g                 \n", args->instdata.nu);
	fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
	fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
	fprintf(file, "#Fitted range: ");
	if (args->opodrdata.Finput)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->opodrdata.range_pct_Fmax[0], args->opodrdata.range_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->opodrdata.range[0], args->opodrdata.range[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opodrdata.Finput)
		fprintf(file, "[%g , %g ] nm \n",args->opodrdata.range[0], args->opodrdata.range[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->opodrdata.range_pct_Fmax[0], args->opodrdata.range_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->opodrdata.nfitdata);

	fprintf(file, "#");
	if (args->opodrmc.skipmc)
		fprintf(file, "Fit failed in %d cases, the corresponding data area NOT included in the  estimates of mean and uncertainty but ARE included in the histograms.", args->opodrmc.skipmc );

	fprintf(file, "\n\n");


	fprintf(file, "#Results \n");
	for (i = 0; i < NPARAM; i++){
		is = nparam_to_string(i, str, sizeof(str));
		fprintf(file, "#%s = (%g ± %g ) \n", str, args->opodrmc.mcavg[i], args->opodrmc.mcstd[i]);
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->opodrmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		for (i=0; i < NPARAM; i++){
			fprintf(file,"#Histogram\n");
			is = nparam_to_string(i, str, sizeof(str));
			if (is){
				fprintf(file,"#%s    count/a.u.  \n", str);
				for (j = 0;j < args->opodrmc.mcnstat; j++)
					fprintf(file, "%g %g \n", args->opodrmc.mcxhist[i][j], args->opodrmc.mcyhist[i][j]);
				fprintf(file, "\n \n");
			}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	for ( j = 0; j< NPARAM; j++){
		is = nparam_to_string(j, str, sizeof(str));
		fprintf(file, "%s ", str);
	}
	fprintf(file,"\n");

	opodr= args->opodrmc.opodrmc;
	for (i = 0; i< args->opodrmc.Nmc; i++){
		fprintf(file, DEPTH, opodr->hc);
		fprintf(file,"  ");
		fprintf(file, AREA, opodr->Aphc);
		fprintf(file,"  ");
		fprintf(file, HARD, opodr->Hit);
		fprintf(file,"  ");
		fprintf(file, MODUL, opodr->Er);
		fprintf(file,"  ");
		fprintf(file, MODUL, opodr->Eit);
		fprintf(file,"  ");
		fprintf(file, SLOPE, opodr->S);
		fprintf(file,"  ");
		fprintf(file, NUMBER, opodr->m);
		fprintf(file,"  \n");
		fprintf(file, DEPTH, opodr->hp);
		fprintf(file,"  ");
		fprintf(file, NUMBER, opodr->eps);
		fprintf(file,"  \n");
		opodr++;
	}


	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}


void op_odr_uncertainty_montecarlo_run_calc(Args *args )
{
	gint istart, iend, ndata;
	gint i, Nmc;
	OPODRdata *opodr;
	OPODRMCdata *mc;
	GwyDataLine *h, *F;
	GwyDataLine *hnew, *Fnew;
	GwyDataLine *hnoise, *Fnoise;
	gdouble hmax_old, Fmax_old;
	GRand *rng;
	gint nn;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;
	gdouble estimate[3];

	//not necessary to check range, couldn't get here in the first place

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (args->opodrdata.Finput)
		range_to_indices(args->opodrdata.from_pct_Fmax*0.01*args->fddata.Fmax, 
				args->opodrdata.to_pct_Fmax*0.01*args->fddata.Fmax,
				args->fddata.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(args->opodrdata.from, args->opodrdata.to, args->fddata.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//back up hmax, Fmax
	hmax_old = args->fddata.hmax;
	Fmax_old = args->fddata.Fmax;

	//use results of previous fit as input estimates
	estimate[0]= args->opodrdata.alpha;
	estimate[1]= args->opodrdata.hp;
	estimate[2]= args->opodrdata.m;

	//get data to be fitted
	h = gwy_data_line_part_extract(args->fddata.hunload, istart, ndata);
	F = gwy_data_line_part_extract(args->fddata.Funload, istart, ndata);



	Nmc = args->opodrmc.Nmc;
	opodr = args->opodrmc.opodrmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		printf("i %d \n",i);
		init_OPODRdata(opodr);
		printf("initialized %s \n", opodr->logfnm);
		g_free(opodr->logfnm);
		opodr->logfnm = create_log_filename_path(mclogfnm);
		opodr++;
	}

	opodr= args->opodrmc.opodrmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
		/* opodr->nu = args->opodrdata.nu; */
		/* opodr->nui = args->opodrdata.nui; */
		/* opodr->Ei = args->opodrdata.Ei; */
		//		opodr->from = args->opodrdata.from;
		//		opodr->to = args->opodrdata.to;
		opodr++;
	}

	/*
	   for (i=0; i< Nmc ;i++){
	   printf ("nu %g \n", args->opodrmc.opodrmc[i].nu);
	   }
	   */

	// initialize random generator and datafield
	rng= g_rand_new();

	//prepare datafield for noise
	hnoise=gwy_data_line_new_alike(h,FALSE);
	Fnoise=gwy_data_line_new_alike(h,FALSE);

	//prepare datafield for varied fields
	hnew=gwy_data_line_new_alike(h,FALSE);
	Fnew=gwy_data_line_new_alike(F,FALSE);

	//prepare auxiliary field
	opodr = args->opodrmc.opodrmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hnoise, args->opodrunc.uh, rng);
		generate_uncorrelated_noise(Fnoise, args->opodrunc.uF, rng);

		//add noise to original h-F data
		if (!sum_data_lines(hnew,h, hnoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(Fnew,F, Fnoise)){
			printf("Should not get here.\n");
			return;
		}

		//add random noise to hmax, Fmax
		args->fddata.hmax = hmax_old + gaussian_random_number(rng)*args->opodrunc.uh;
		args->fddata.Fmax = Fmax_old + gaussian_random_number(rng)*args->opodrunc.uF;

		//run fit
		op_odr_fit(hnew,Fnew,opodr, &(args->fddata), &(args->instdata), &(args->area), estimate);
		opodr++;
	}

	//reset hmax, Fmax to original value
	args->fddata.hmax = hmax_old;
	args->fddata.Fmax = Fmax_old;

	printf("run Monte Carlo \n");	
	g_object_unref(h);
	g_object_unref(F);
	g_object_unref(hnew);
	g_object_unref(Fnew);
	g_object_unref(hnoise);
	g_object_unref(Fnoise);
	g_rand_free(rng);


	for (i=0; i< Nmc; i++){
		//		printf(" %d Er %g Eit %g Hit %g \n", i, args->opodrmc.opodrmc[i].Er, args->opodrmc.opodrmc[i].Eit, args->opodrmc.opodrmc[i].Hit);
	}

	//calculate averages and stdevs
	//hp, Aphc, Hit, Er, Eit, S, m

	mc = &(args->opodrmc);
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] =0;
		mc->mcstd[i] =0;
	}

	opodr = args->opodrmc.opodrmc;
	nn = 0;
	for (i = 0; i< Nmc; i++){
		//check sanity
		if (opodr->hc >0){
			mc->mcavg[0]+=opodr->hc;
			mc->mcavg[1]+=opodr->Aphc;
			mc->mcavg[2]+=opodr->Hit;
			mc->mcavg[3]+=opodr->Er;
			mc->mcavg[4]+=opodr->Eit;
			mc->mcavg[5]+=opodr->S;
			mc->mcavg[6]+=opodr->m;
			mc->mcavg[7]+=opodr->hp;
			mc->mcavg[8]+=opodr->eps;
			mc->mcstd[0]+=opodr->hc*opodr->hc;
			mc->mcstd[1]+=opodr->Aphc*opodr->Aphc;
			mc->mcstd[2]+=opodr->Hit*opodr->Hit;
			mc->mcstd[3]+=opodr->Er*opodr->Er;
			mc->mcstd[4]+=opodr->Eit*opodr->Eit;
			mc->mcstd[5]+=opodr->S*opodr->S;
			mc->mcstd[6]+=opodr->m*opodr->m;
			mc->mcstd[7]+=opodr->hp*opodr->hp;
			mc->mcstd[8]+=opodr->eps*opodr->eps;
			nn++;
		}
		opodr++;
	}

	//use only reasonable data for average and standard deviation
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] /= nn;
		mc->mcstd[i] /= nn;
		mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
		mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
		mc->mcstd[i] = sqrt(mc->mcstd[i]);
	}

	// how many datasets were skipped
	args->opodrmc.skipmc = Nmc-nn;

	/*
	   printf("mean  stdev \n");
	   for (i = 0; i< NPARAM; i++){
	   printf(" %g   %g \n", avg[i], std[i]);
	   }
	   */

	// Histograms
	//create data structure

	dd = args->opodrmc.mcdd;
	opodr = args->opodrmc.opodrmc;
	for (i = 0; i< Nmc; i++){
		dd[0][i] = opodr->hc;
		dd[1][i] = opodr->Aphc;
		dd[2][i] = opodr->Hit;
		dd[3][i] = opodr->Er;
		dd[4][i] = opodr->Eit;
		dd[5][i] = opodr->S;
		dd[6][i] = opodr->m;
		dd[7][i] = opodr->hp;
		dd[8][i] = opodr->eps;
		opodr++;
	}
	//prepare histogram structure


	for (i=0; i < NPARAM; i++){
		xhist = mc->mcxhist[i];
		yhist = mc->mcyhist[i];
		hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
		if (!hist){
			printf("Warning: No histogram could be created.\n");
			mc->mcnstat = -1;
			break;
		}
	}
}


#endif
