#include "tool_op_unc.h"
#include "tool_op.h"

#include "datatypes.h"
#include "fddata.h"
#include "fit-utils.h"
#include "indent-toolbox-common.h"
#include "mc-utils.h"

#include <math.h>

#include <app/file.h>
#include <libprocess/gwyprocess.h>

static gboolean nparam_to_string(gint i, gchar *str, gint nstr);

void init_shifted_OPdata(OPdata *src, OPdata *dest, gdouble dh)
{
	init_OPdata(dest);

	// shift fitting range  in h-mode by dh
	// don't shift range for F-mode
	// both for hp fit and for S fit
	dest->hprange[0]= src->hprange[0]-dh;
	dest->hprange[1]= src->hprange[1]-dh;
	dest->hpfrom = dest->hprange[0];
	dest->hpto = dest->hprange[1];

	dest->hprange_pct_Fmax[0] = src->hprange_pct_Fmax[0];
	dest->hprange_pct_Fmax[1] = src->hprange_pct_Fmax[1];
	dest->hpfrom_pct_Fmax = src->hpfrom_pct_Fmax;
	dest->hpto_pct_Fmax = src->hpto_pct_Fmax;

	dest->Finputhp = src->Finputhp;

	dest->Srange[0]= src->Srange[0]-dh;
	dest->Srange[1]= src->Srange[1]-dh;
	dest->Sfrom = dest->Srange[0];
	dest->Sto = dest->Srange[1];

	dest->Srange_pct_Fmax[0] = src->Srange_pct_Fmax[0];
	dest->Srange_pct_Fmax[1] = src->Srange_pct_Fmax[1];
	dest->Sfrom_pct_Fmax = src->Sfrom_pct_Fmax;
	dest->Sto_pct_Fmax = src->Sto_pct_Fmax;

	dest->FinputS = src->FinputS;

	// copy beta
	dest->beta = src->beta;
}

void 
op_propagate_uncertainties_calc (Args *args)
{
	// TODO
	OPUncdata *unc;
	OPdata *op;
	FDdata *fd;
	Instdata *inst;
	gdouble uh, uF;
	gdouble uSh, uSF ;
	gdouble uhph, uhpF;
	gdouble uah, uaF, ubh, ubF;
	gdouble umh, umF, z, depsdm, uepsh, uepsF;
	gint istart, iend,ndata;
	gint i;
	gdouble covabh, covabF;
	gdouble covShch, covShcF;
	gdouble covmhph, covmhpF;
	gdouble dAdh;
	gdouble m,S,eps, hc, hp, hmax,Fmax, Aphc,Hit,Er,Eit, nu,nui,Ei;
	GwyDataLine *x, *y, *ux, *uy;

	unc = &(args->opunc);
	op = &(args->opdata);
	fd = &(args->fddata);
	inst = &(args->instdata);


	//make copies to simplify code
	uh = unc->uh;
	uF = unc->uF;
	m = op->m;
	eps = op->eps;
	S = op->S;
	hp = op->hp;
	hc = op->hc;
	hmax = fd->hmax;
	Fmax = fd->Fmax;
	Aphc = op->Aphc;
	Hit = op->Hit;
	Eit = op->Eit;
	Er = op->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;
	
	if (VERBOSE)
		printf(" \n\nPropagate uncertainties\n");

	/* constant uncertainty for all data points */
	if (op->Finputhp)
		range_to_indices(op->hprange_pct_Fmax[0]*0.01*fd->Fmax, op->hprange_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(op->hprange[0], op->hprange[1], fd->hunload, TRUE, &istart, &iend, &ndata);

	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	total_least_squares_line_fit_const(x, y, &(op->reghp), uh , uF  );
	g_object_unref(x);
	g_object_unref(y);


	//for hp fit a*x+b
	uah = op->reghp.unc_slope_x;
	uaF = op->reghp.unc_slope_y;
	ubh = op->reghp.unc_intercept_x;
	ubF = op->reghp.unc_intercept_y;
	covabh = op->reghp.cov_slope_intercept_x;
	covabF = op->reghp.cov_slope_intercept_y;

	if (VERBOSE)
		printf(" a %g b %g hp %g  uah %g uaF %g ubh %g ubF %g \n",op->reghp.slope, op->reghp.intercept, -op->reghp.intercept/op->reghp.slope, uah, uaF, ubh, ubF);


	/*   u(hp)   */
	uhph = hp * sqrt( (uah*uah/op->reghp.slope/op->reghp.slope)+(ubh*ubh/op->reghp.intercept/op->reghp.intercept) - 2*covabh );
	uhpF = hp * sqrt( (uaF*uaF/op->reghp.slope/op->reghp.slope)+(ubF*ubF/op->reghp.intercept/op->reghp.intercept) - 2*covabF );

	/* hp = -b/a
	 * uhp^2 = ( -ub/a)^2 + (ua*b/a^2) ^2 + 2 cov(a,b) 1/a^2 = hp^2 * ( (ub/b)^2 + ( ua/a)^2 ) -2 corr(a,b) ua ub hp/(ab)
	 */

	if (VERBOSE)
		printf("u(hp;h) %g u(hp;F) %g      urel(hp;h) %g urel(hp;F) %g \n", uhph, uhpF, uhph/hp, uhpF/hp);


	/*   u(m)    */
	
	if (op->FinputS)
		range_to_indices(op->Srange_pct_Fmax[0]*0.01*fd->Fmax, op->Srange_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(op->Srange[0], op->Srange[1], fd->hunload, TRUE, &istart, &iend, &ndata);

	x = gwy_data_line_new(ndata,1.0, TRUE);
	y = gwy_data_line_new(ndata,1.0, TRUE);
	ux = gwy_data_line_new(ndata,1.0, TRUE);
	uy = gwy_data_line_new(ndata,1.0, TRUE);

	for (i =0; i< ndata; i++){
		x->data[i] = log(fd->hunload->data[istart+i]-hp);
		y->data[i] = log(fd->Funload->data[istart+i]);
		ux->data[i] = sqrt(uh*uh + uhph*uhph)/(fd->hunload->data[istart+i]-hp); //TODO chybi u(hp;F)
		uy->data[i] = uF/fd->Funload->data[istart+i]; 
	}
	total_least_squares_line_fit_nonconst(x, y, &(op->regS), ux, uy );  
	umh = op->regS.unc_slope_x;
	umF = op->regS.unc_slope_y;

	covmhpF = 0;
	covmhph = 0;
	for (i = 0; i < x->res; i ++){
		covmhph -= op->regS.auxtoberemoved_axi[i]/exp(x->data[i]);
	}

	if (VERBOSE)
		printf(" covmhph %g \n", covmhph);

	g_object_unref(x);
	g_object_unref(y);
	g_object_unref(ux);
	g_object_unref(uy);
	if (VERBOSE){
		printf(" new  slope %g intercept %g slope_x %g slope_y %g intercept_x %g intercept_y %g cov_x %g cov_y %g  \n", op->regS.slope, op->regS.intercept,
				op->regS.unc_slope_x, op->regS.unc_slope_y,
				op->regS.unc_intercept_x, op->regS.unc_intercept_y,
				op->regS.cov_slope_intercept_x, op->regS.cov_slope_intercept_y);
		printf(" m %g umh %g umF %g \n", m, umh, umF);
	}


	/*    u(eps)   */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * (1 -eps/m ) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	uepsh = depsdm *umh;
	uepsF = depsdm *umF;
	if (VERBOSE){
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps;h) %g u(eps;F) %g  urel(eps;h) %g urel(eps;F) %g \n", uepsh, uepsF, uepsh/eps, uepsF/eps);
	}

	/*    u(S)   */
	uSh = sqrt(umh*umh/m/m +
			uh *uh/(hmax-hp)/(hmax-hp) +
			uhph*uhph/(hmax-hp)/(hmax-hp) +
			2*covmhph/m/(hmax-hp)
		  ) * S; 

	uSF = sqrt(umF*umF/m/m +    
		       	uF*uF/Fmax/Fmax  +    
			uhpF*uhpF/(hmax-hp)/(hmax-hp) +
			2*covmhpF/m/(hmax-hp)
		  ) * S; 
	unc->uSuh = uSh;
	unc->uSuF = uSF;
	/* u(S;h) = S *sqrt [ (umh/m)^2 + (uh/(hmax-hp))^2 + (uhph/(hmax-hp))^2  + 2 cov(m,hp;h)/(m*(hmax-hp))]
	 * u(S;F) = S *sqrt [ (umF/m)^2 + (uFmax/Fmax)^2   + (uhpF/(hmax-hp))^2  + 2 cov(m,hp;F)/(m*(hmax-hp))]
	 */
	

	if (VERBOSE){
		printf(" u(S;h) %g u(S;F) %g  urel(S;h) %g urel(S;F) %g \n", uSh, uSF, uSh/S, uSF/S);
	}

	/*   u(hc)   */
	unc->uhcuh = sqrt((hmax-hp)*(hmax-hp)* (eps/m - depsdm)*(eps/m-depsdm)/m/m *umh*umh + 
			(1-eps/m)*(1-eps/m)*uh*uh + 
			eps*eps/m/m*uhph*uhph );
	unc->uhcuF = sqrt((hmax-hp)*(hmax-hp)* (eps/m - depsdm)*(eps/m-depsdm)/m/m *umF*umF + 
			eps*eps/m/m*uhpF*uhpF );
	
	/* u(hc;h) ^2 = [(hmax-hp) (eps/m- deps/dm)/m  u(m;h) ]^2 + [(1-eps/m) u(hmax)]^2 + [eps/m u(hp;h)]^2  
	 u(hc;F) ^2 = [(hmax-hp) (eps/m- deps/dm)/m  u(m;F) ]^2 +                         [eps/m u(hp;F)]^2 
	 */

	if (VERBOSE)
		printf(" u(hc;h) %g u(hc;F) %g  urel(hc;h) %g urel(hc;F) %g \n", unc->uhcuh, unc->uhcuF, unc->uhcuh/hc, unc->uhcuF/hc);

	/*    u(A)   */
	dAdh =  eval_area_derivative(&(args->area), hc) ;
	unc->uAuh = dAdh * unc->uhcuh;
	unc->uAuF = dAdh * unc->uhcuF;
	if (VERBOSE)
		printf(" u(A;h) %g u(A;F) %g  urel(A;h) %g urel(A;F) %g \n", unc->uAuh, unc->uAuF, unc->uAuh/Aphc, unc->uAuF/Aphc);

	/*     u(H_IT)   */
	unc->uHituh = Hit/Aphc*dAdh*unc->uhcuh;
	unc->uHituF = Hit *sqrt(uF*uF/Fmax/Fmax + dAdh*dAdh/Aphc/Aphc*unc->uhcuh*unc->uhcuh);
	if (VERBOSE)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);


	/*     u(E_r)   */

	covShch = - S *(1-eps)/(hmax-hp)*uh*uh  - S*(hmax - hp)/m *depsdm* umh*umh +
		S*eps/(hmax-hp)*uhph*uhph;
	covShcF =  - S*(hmax - hp)/m *depsdm* umF*umF +
		S*eps/(hmax-hp)*uhpF*uhpF;

	/*cov(S,hc;h) = -S(1-eps)/(hmax-hp)*uh^2 -S(hmax-hp)/m *(deps/dm) *u(m;h)^2 + S*eps/(hmax-hp)*u(hp;h)^2
	 *cov(S,hc;F) =                          -S(hmax-hp)/m *(deps/dm) *u(m;F)^2 + S*eps/(hmax-hp)*u(hp;F)^2 
	 */

	unc->uEruh = Er*sqrt( uSh*uSh/S/S + unc->uAuh*unc->uAuh/Aphc/Aphc/4 - 1/S/Aphc*dAdh*covShch); 
	unc->uEruF = Er*sqrt( uSF*uSF/S/S + unc->uAuF*unc->uAuF/Aphc/Aphc/4 - 1/S/Aphc*dAdh*covShcF); 
	/* u(Er;h) = sqrt[ (Er/S)^2 u(S;h)^2 + (Er/2A)^2 u(A;h)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc;h) ]
	 * u(Er;F) = sqrt[ (Er/S)^2 u(S;F)^2 + (Er/2A)^2 u(A;F)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc;F) ]
	 */

	if (VERBOSE)
		printf(" u(Er;h) %g u(Er;F) %g  urel(Er;h) %g urel(Er;F) %g \n", unc->uEruh, unc->uEruF, unc->uEruh/Er, unc->uEruF/Er);



	/*     u(E_IT)   */
	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; // Eit^2/Er^2 /(1-nu^2) u(Er;h)
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; // Eit^2/Er^2 /(1-nu^2) u(Er;F)


	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*inst->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *inst->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/ Ei/Ei *inst->uEi)*1e9; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (VERBOSE){
		printf("u(Eit;h) %g  u(Eit;F) %g  urel(Eit;h) %g  urel(Eit;F) %g\n", unc->uEituh, unc->uEituF, unc->uEituh/Eit, unc->uEituF/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}

	if (VERBOSE)
		printf(" \nEnd propagation of uncertainties\n\n");

}


gboolean op_uncertainty_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	OPdata *opdata;
	Instdata *instdata;
	OPUncdata *opunc;
	gint j;

	opdata = &(args->opdata);
	instdata = &(args->instdata);
	opunc = &(args->opunc);

	if (args->fddata.hunload->res == 0 || args->fddata.Funload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of Oliver Pharr analysis  \n");
	fprintf(file, "# u(depth): %g nm\n", opunc->uh);
	fprintf(file, "# u(load): %g nm\n", opunc->uF);
	fprintf(file, "#beta: %g \n", opdata->beta);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, instdata->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, instdata->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, instdata->uEi*1e-9); //TODO units
	fprintf(file, "#Fitted hprange: ");
	if (args->opdata.Finputhp)
		fprintf(file, "	[%g , %g ] %% Fmax \n", opdata->hprange_pct_Fmax[0], opdata->hprange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",opdata->hprange[0], opdata->hprange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opdata.Finputhp)
		fprintf(file, "[%g , %g ] nm \n",opdata->hprange[0], opdata->hprange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", opdata->hprange_pct_Fmax[0], opdata->hprange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", opdata->nfithpdata);

	fprintf(file, "#Fitted S range: ");
	if (args->opdata.FinputS)
		fprintf(file, "	[%g , %g ] %% Fmax \n", opdata->Srange_pct_Fmax[0], opdata->Srange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",opdata->Srange[0], opdata->Srange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opdata.FinputS)
		fprintf(file, "[%g , %g ] nm \n",opdata->Srange[0], opdata->Srange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", opdata->Srange_pct_Fmax[0], opdata->Srange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", opdata->nfitSdata);


	fprintf(file, "#\n");

	fprintf(file, "#Results of Oliver Pharr analysis \n");
	fprintf(file, "#S = %g mN/nm\n", opdata->S);
	fprintf(file, "#hc = %g nm\n", opdata->hc);
	fprintf(file, "#Ap(hc) = %g nm^2\n", opdata->Aphc);
	fprintf(file, "#Er = %g GPa\n", opdata->Er);
	fprintf(file, "#Hit = %g GPa\n", opdata->Hit);
	fprintf(file, "#Eit = %g GPa\n", opdata->Eit);
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(S) =  %g nm \n", opunc->uS);
	fprintf(file, "#u(hc) =  %g nm \n", opunc->uhc);
	fprintf(file, "#u(Ap(hc)) =  %g nm^2 \n", opunc->uA);
	fprintf(file, "#u(Hit) =  %g MPa \n", opunc->uHit);
	fprintf(file, "#u(Er) =  %g GPa \n", opunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", opunc->uEit);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of S\n");
	fprintf(file, "#u(S;uh) = %g mN/nm \n", opunc->uSuh);
	fprintf(file, "#u(S;uF) = %g mN/nm \n", opunc->uSuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of hc\n");
	fprintf(file, "#u(hc;uh) = %g nm \n", opunc->uhcuh);
	fprintf(file, "#u(hc;uF) = %g nm \n", opunc->uhcuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Ap(hc)\n");
	fprintf(file, "#u(Ap(hc);uh) = %g nm^2 \n", opunc->uAuh);
	fprintf(file, "#u(Ap(hc);uF) = %g nm^2 \n", opunc->uAuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Hit\n");
	fprintf(file, "#u(Hit;uh) = %g MPa \n", opunc->uHituh);
	fprintf(file, "#u(Hit;uF) = %g MPa \n", opunc->uHituF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", opunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", opunc->uEruF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", opunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", opunc->uEituF);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", opunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", opunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", opunc->uEituEi);
	fprintf(file, "#\n");
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Changes of the S corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#S/(mN/nm) \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opunc->Sc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
		fprintf(file, "# %d   %g  \n", j, opunc->Sc[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
		fprintf(file, "# %d   %g  \n", j, opunc->Ec[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the H corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#H/MPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opunc->Hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
		fprintf(file, "# %d   %g  \n", j, opunc->Hc[j+CONTACTMAX]);
	}

	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}

void
op_fit_shift_contact(OPdata *opdata, FDdata *fddata, Instdata *instdata, Area *area, gdouble *Sc, gdouble *Ec,gdouble *Hc,gint j)
{
	OPdata op;
	FDdata fdc;
	gdouble dh;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;
	gint i;

	Sc[j+CONTACTMAX] = -1;
	Ec[j+CONTACTMAX] = -1;
	Hc[j+CONTACTMAX] = -1;


	if (fddata->i_contact_load+j < 0)
		return;

	// create shifted FDdata
	dh = create_shifted_FDdata(fddata, &fdc, j);

	if (VERBOSE)
		printf("\n\nCreate shifted data for j %d \n", j);

	// create working copy of OPdata 
	init_shifted_OPdata(opdata, &op, dh);
	// shift fitting range both for hp fit and for S fit
	/*
	op = *opdata;
	op.hprange[0] += -dh;
	op.hprange[1] += -dh;
	op.hpfrom = op.hprange[0];
	op.hpto = op.hprange[1];
	op.Srange[0] += -dh;
	op.Srange[1] += -dh;
	op.Sfrom = op.Srange[0];
	op.Sto = op.Srange[1];
	*/


	/* hp fit */

	/*check range */
	if (opdata->Finputhp){
		printf(" hpfrom %g hpto %g \n", op.hpfrom_pct_Fmax, op.hpto_pct_Fmax);
		if (op.hpfrom_pct_Fmax ==  op.hpto_pct_Fmax){
			return;
		}
	}
	else{
		printf(" hpfrom %g hpto %g \n", op.hpfrom, op.hpto);

		if (op.hpfrom ==  op.hpto){
			return;
		}
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (op.Finputhp)
		range_to_indices(op.hprange_pct_Fmax[0]*0.01*fdc.Fmax, op.hprange_pct_Fmax[1]*0.01*fdc.Fmax,
			       	 fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(op.hprange[0], op.hprange[1], fdc.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, no fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if (VERBOSE)
		printf("interval has %d points, from %d to %d, hrange [%g, %g ] nm, Frange [%g, %g] mN \n", ndata, istart, iend,
			       fdc.hunload->data[iend], fdc.hunload->data[istart], fdc.Funload->data[iend], fdc.Funload->data[istart]);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);


	// perform fit and get results
	op_fit_hp(x,y,&op, &fdc);

	// clean up
	g_object_unref(x);
	g_object_unref(y);


	/*  S fit */

	/*check range */
	if (opdata->FinputS){
		printf(" Sfrom %g Sto %g \n", op.Sfrom_pct_Fmax, op.Sto_pct_Fmax);
		if (op.Sfrom_pct_Fmax ==  op.Sto_pct_Fmax){
			return;
		}
	}
	else{
		printf(" Sfrom %g Sto %g \n", op.Sfrom, op.Sto);

		if (op.Sfrom ==  op.Sto){
			return;
		}
	}


	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (opdata->FinputS)
		range_to_indices(op.Srange_pct_Fmax[0]*0.01*fdc.Fmax, op.Srange_pct_Fmax[1]*0.01*fdc.Fmax,
			       	 fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(op.Srange[0], op.Srange[1], fdc.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, no fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if(VERBOSE)
		printf("interval has %d points, from %d to %d, hrange [%g, %g ] nm, Frange [%g, %g] mN \n", ndata, istart, iend,
			       fdc.hunload->data[iend], fdc.hunload->data[istart], fdc.Funload->data[iend], fdc.Funload->data[istart]);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	if (FIT_S_LR){
		//linear regression 
		x = gwy_data_line_new(ndata,1.0, TRUE);
		y = gwy_data_line_new(ndata,1.0, TRUE);
		for (i =0; i< ndata; i++){
			x->data[i] = log(fdc.hunload->data[istart+i]-op.hp);
			y->data[i] = log(fdc.Funload->data[istart+i]);
		}
	}
	else{
		//nonlinear regression
		x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
		y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);
	}

	// perform fit and get results
	if (FIT_S_LR) {
	    //linear regression
	    op_fit_S_lr(x,y, &op, &fdc, instdata, area);
	}
	else {
	    //nonlinear regression
	    op_fit_S_nl(x,y, &op, &fdc, instdata, area);
	}

	// clean up
	g_object_unref(x);
	g_object_unref(y);
	destroy_FDdata(&fdc);

	Sc[j+CONTACTMAX] = op.S;
	Ec[j+CONTACTMAX] = op.Er;
	Hc[j+CONTACTMAX] = op.Hit;

	if (VERBOSE)
		printf("\n\n");
}

void op_uncertainty_montecarlo_run_calc( Args *args)
{
	gint istart, iend, ndata;
	gint i,j, Nmc;
	OPdata *op;
	OPMCdata *mc;
	GwyDataLine *h, *F;
	GwyDataLine *hnew, *Fnew;
	GwyDataLine *hnoise, *Fnoise;
	gdouble hmax_old, Fmax_old;
	GRand *rng;
	gint nn;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;

	// initialize data

	Nmc = args->opmc.Nmc;
	op = args->opmc.opmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		init_OPdata(op);
		op++;
	}

	op= args->opmc.opmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
	    /* OPdata no longer contains these; use Instdata instead */
		/* op->nu = args->instdata.nu; */
		/* op->nui = args->instdata.nui; */
		/* op->Ei = args->instdata.Ei; */
		//		op->from = args->opdata.from;
		//		op->to = args->opdata.to;
		op++;
	}

	/*
	   for (i=0; i< Nmc ;i++){
	   printf ("nu %g \n", args->opmc.opmc[i].nu);
	   }
	   */

	// initialize random generator and datafield
	rng= g_rand_new();



	/* hp fit */

	//not necessary to check range, couldn't get here in the first place

	//find appropriate data, which should be fitted, must use real fitted range, use same regime as for the main fit
	if (args->opdata.Finputhp)
		range_to_indices(args->opdata.hprange_pct_Fmax[0]*0.01*args->fddata.Fmax, 
			         args->opdata.hprange_pct_Fmax[1]*0.01*args->fddata.Fmax,
			       	 args->fddata.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(args->opdata.hprange[0], args->opdata.hprange[1], args->fddata.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hunload->res - istart);

	if (ndata < 2){
	        g_rand_free(rng);
		return;
	}

	//back up hmax, Fmax
	hmax_old = args->fddata.hmax;
	Fmax_old = args->fddata.Fmax;

	//get data to be fitted
	h = gwy_data_line_part_extract(args->fddata.hunload, istart, ndata);
	F = gwy_data_line_part_extract(args->fddata.Funload, istart, ndata);

	//prepare datafield for noise
	hnoise=gwy_data_line_new_alike(h,FALSE);
	Fnoise=gwy_data_line_new_alike(h,FALSE);

	//prepare datafield for varied fields
	hnew=gwy_data_line_new_alike(h,FALSE);
	Fnew=gwy_data_line_new_alike(F,FALSE);

	op = args->opmc.opmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hnoise, args->opunc.uh, rng);
		generate_uncorrelated_noise(Fnoise, args->opunc.uF, rng);

		//add noise to original h-F data
		if (!sum_data_lines(hnew,h, hnoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(Fnew,F, Fnoise)){
			printf("Should not get here.\n");
			return;
		}

		//add random noise to hmax, Fmax
		args->fddata.hmax = hmax_old + gaussian_random_number(rng)*args->opunc.uh;
		args->fddata.Fmax = Fmax_old + gaussian_random_number(rng)*args->opunc.uF;

		//run fit
	//	printf("i %d \n", i);
	//	printf("-----\n");
	//	for (j =0;j < hnew->res; j++)
	//		printf("%g %g \n", hnew->data[j], Fnew->data[j]);

		op_fit_hp(hnew,Fnew,op, &(args->fddata));
	//	printf(" hp %g \n", op->hp);
		op++;
	}

	// clean up
	g_object_unref(h);
	g_object_unref(F);
	g_object_unref(hnew);
	g_object_unref(Fnew);
	g_object_unref(hnoise);
	g_object_unref(Fnoise);

	/*            S fit */

	//not necessary to check range, couldn't get here in the first place
	
	//find appropriate data, which should be fitted, must use real fitted range, use same regime as for the main fit
	if (args->opdata.FinputS)
		range_to_indices(args->opdata.Srange_pct_Fmax[0]*0.01*args->fddata.Fmax, 
			         args->opdata.Srange_pct_Fmax[1]*0.01*args->fddata.Fmax,
			       	 args->fddata.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(args->opdata.Srange[0], args->opdata.Srange[1], args->fddata.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	h = gwy_data_line_part_extract(args->fddata.hunload, istart, ndata);
	F = gwy_data_line_part_extract(args->fddata.Funload, istart, ndata);

	//prepare datafield for noise
	hnoise=gwy_data_line_new_alike(h,FALSE);
	Fnoise=gwy_data_line_new_alike(h,FALSE);

	//prepare datafield for varied fields
	hnew=gwy_data_line_new_alike(h,FALSE);
	Fnew=gwy_data_line_new_alike(F,FALSE);

	op = args->opmc.opmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hnoise, args->opunc.uh, rng);
		generate_uncorrelated_noise(Fnoise, args->opunc.uF, rng);

		//add noise to original h-F data
		if (!sum_data_lines(hnew,h, hnoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(Fnew,F, Fnoise)){
			printf("Should not get here.\n");
			return;
		}

		//add random noise to hmax, Fmax
		args->fddata.hmax = hmax_old + gaussian_random_number(rng)*args->opunc.uh;
		args->fddata.Fmax = Fmax_old + gaussian_random_number(rng)*args->opunc.uF;

		//if necessary calculate logarithm
		if (FIT_S_LR){
			for (j = 0 ; j < hnew->res; j++){
				hnew->data[j] = log(hnew->data[j]-op->hp);
				Fnew->data[j] = log(Fnew->data[j]);
			}
		}
//		printf("i %d \n", i);
//		printf("-----\n");
//		printf("hnew, Fnew \n");
//		for (j =0;j < hnew->res; j++)
//			printf("%g %g \n", hnew->data[j], Fnew->data[j]);


		//run fit
		op_fit_hp(hnew,Fnew,op, &(args->fddata));
		if (FIT_S_LR) 		//linear regression
		    op_fit_S_lr(hnew,Fnew,op, &(args->fddata), &(args->instdata), &(args->area));
		else 		//nonlinear regression
		    op_fit_S_nl(hnew,Fnew,op, &(args->fddata), &(args->instdata), &(args->area));

//		printf("H %g E %g \n", op->Hit, op->Eit);
		op++;
	}

	//reset hmax, Fmax to original value
	args->fddata.hmax = hmax_old;
	args->fddata.Fmax = Fmax_old;

	printf("run Monte Carlo \n");	
	g_object_unref(h);
	g_object_unref(F);
	g_object_unref(hnew);
	g_object_unref(Fnew);
	g_object_unref(hnoise);
	g_object_unref(Fnoise);
	g_rand_free(rng);

	for (i=0; i< Nmc; i++){
		//		printf(" %d Er %g Eit %g Hit %g \n", i, args->opmc.opmc[i].Er, args->opmc.opmc[i].Eit, args->opmc.opmc[i].Hit);
	}

	//calculate averages and stdevs
	//hp, Aphc, Hit, Er, Eit

	mc = &(args->opmc);
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] =0;
		mc->mcstd[i] =0;
	}

	op = args->opmc.opmc;
	nn = 0;
	for (i = 0; i< Nmc; i++){
		//check sanity
		if (op->hc >0){
			mc->mcavg[0]+=op->hc;
			mc->mcavg[1]+=op->Aphc;
			mc->mcavg[2]+=op->Hit;
			mc->mcavg[3]+=op->Er;
			mc->mcavg[4]+=op->Eit;
			mc->mcavg[5]+=op->S;
			mc->mcstd[0]+=op->hc*op->hc;
			mc->mcstd[1]+=op->Aphc*op->Aphc;
			mc->mcstd[2]+=op->Hit*op->Hit;
			mc->mcstd[3]+=op->Er*op->Er;
			mc->mcstd[4]+=op->Eit*op->Eit;
			mc->mcstd[5]+=op->S*op->S;
			nn++;
		}
		op++;
	}

	//use only reasonable data for average and standard deviation
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] /= nn;
		mc->mcstd[i] /= nn;
		mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
		mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
		mc->mcstd[i] = sqrt(mc->mcstd[i]);
	}

	// how many datasets were skipped
	args->opmc.skipmc = Nmc-nn;

	/*
	   printf("mean  stdev \n");
	   for (i = 0; i< NPARAM; i++){
	   printf(" %g   %g \n", avg[i], std[i]);
	   }
	   */

	// Histograms
	//create data structure

	dd = args->opmc.mcdd;
	op = args->opmc.opmc;
	for (i = 0; i< Nmc; i++){
		dd[0][i] = op->hc;
		dd[1][i] = op->Aphc;
		dd[2][i] = op->Hit;
		dd[3][i] = op->Er;
		dd[4][i] = op->Eit;
		dd[5][i] = op->S;
		op++;
	}
	//prepare histogram structure


	for (i=0; i < NPARAM; i++){
		xhist = mc->mcxhist[i];
		yhist = mc->mcyhist[i];
		hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
		if (!hist){
			printf("Warning: No histogram could be created.\n");
			mc->mcnstat = -1;
			break;
		}
	}
}

gboolean
op_mc_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	OPdata *op;
	gint i,j;
	gchar str[200];
	gboolean is;

	if (args->fddata.hunload->res == 0 || args->fddata.Funload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of Oliver Pharr analysis\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->opunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->opunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->opmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of Oliver Pharr analysis  \n");
	fprintf(file, "#beta: %g \n", args->opdata.beta);
	fprintf(file, "#nu: %g                 \n", args->instdata.nu);
	fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
	fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
	fprintf(file, "#Fitted hprange: ");
	if (args->opdata.Finputhp)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->opdata.hprange_pct_Fmax[0], args->opdata.hprange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->opdata.hprange[0], args->opdata.hprange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opdata.Finputhp)
		fprintf(file, "[%g , %g ] nm \n",args->opdata.hprange[0], args->opdata.hprange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->opdata.hprange_pct_Fmax[0], args->opdata.hprange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->opdata.nfithpdata);


	fprintf(file, "#Fitted S range: ");
	if (args->opdata.FinputS)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->opdata.Srange_pct_Fmax[0], args->opdata.Srange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->opdata.Srange[0], args->opdata.Srange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opdata.FinputS)
		fprintf(file, "[%g , %g ] nm \n",args->opdata.Srange[0], args->opdata.Srange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->opdata.Srange_pct_Fmax[0], args->opdata.Srange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->opdata.nfitSdata);

	fprintf(file, "\n\n");


	fprintf(file, "#Results \n");
	for (i = 0; i < NPARAM; i++){
		is = nparam_to_string(i, str, sizeof(str));
		fprintf(file, "#%s = (%g ± %g ) \n", str, args->opmc.mcavg[i], args->opmc.mcstd[i]);
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->opmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		for (i=0; i < NPARAM; i++){
			fprintf(file,"#Histogram\n");
			is = nparam_to_string(i, str, sizeof(str));
			if (is){
				fprintf(file,"#%s    count/a.u.  \n", str);
				for (j = 0;j < args->opmc.mcnstat; j++)
					fprintf(file, "%g %g \n", args->opmc.mcxhist[i][j], args->opmc.mcyhist[i][j]);
				fprintf(file, "\n \n");
			}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	for ( j = 0; j< NPARAM; j++){
		is = nparam_to_string(j, str, sizeof(str));
		fprintf(file, "%s ", str);
	}
	fprintf(file,"\n");

	op= args->opmc.opmc;
	for (i = 0; i< args->opmc.Nmc; i++){
		fprintf(file, DEPTH, op->hc);
		fprintf(file,"  ");
		fprintf(file, AREA, op->Aphc);
		fprintf(file,"  ");
		fprintf(file, HARD, op->Hit);
		fprintf(file,"  ");
		fprintf(file, MODUL, op->Er);
		fprintf(file,"  ");
		fprintf(file, MODUL, op->Eit);
		fprintf(file,"  ");
		fprintf(file, SLOPE, op->S);
		fprintf(file,"  \n");
		op++;
	}
	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}

void
init_OPMCdata (OPMCdata *opmc, gint Nmc)
{
	gint i;

	//number of iterations
	opmc->Nmc = Nmc;
	printf("initialize for %d iterations \n", opmc->Nmc);

	//how much had to be skipped 
	opmc->skipmc = 0;

	//dynamically allocate
	opmc->opmc = (OPdata *)g_malloc(opmc->Nmc*sizeof(OPdata));
	opmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	opmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	opmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	opmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	opmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	opmc->mcnstat = (gint)floor(3.49*cbrt(opmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		opmc->mcdd[i] = (gdouble *) g_malloc(opmc->Nmc*sizeof(gdouble));
		opmc->mcxhist[i] = (gdouble *) g_malloc(opmc->mcnstat*sizeof(gdouble));
		opmc->mcyhist[i] = (gdouble *) g_malloc(opmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_OPMCdata (OPMCdata *opmc)
{
	gint i;

	//free everything
	g_free(opmc->mcavg);
	g_free(opmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(opmc->mcdd[i]);
		g_free(opmc->mcxhist[i]);
		g_free(opmc->mcyhist[i]);
	}

	g_free(opmc->mcdd);
	g_free(opmc->mcxhist);
	g_free(opmc->mcyhist);

	opmc->mcnstat = 0;
	opmc->skipmc = 0;
	g_free(opmc->opmc);
}

static gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{

	switch (i){
		case 0:
			g_snprintf(str,nstr,"h_c/nm");
			break;
		case 1:
			g_snprintf(str,nstr,"A_p(h_c)/nm^2");
			break;
		case 2:
			g_snprintf(str,nstr,"H_IT/MPa");
			break;
		case 3:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 4:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		case 5:
			g_snprintf(str,nstr,"S/(mN/nm)");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}
