#ifndef NOFORTRAN

#include "tool_twoslopes_unc.h"
#include "tool_twoslopes.h"

#include "controls.h"
#include "datatypes.h"
#include "fddata.h"
#include "fit-utils.h"
#include "file-utils.h"
#include "gui-utils.h"
#include "indent-toolbox-common.h"
#include "mc-utils.h"

#define _USE_MATH_DEFINES
#include <math.h>
#include <string.h>

#include <app/file.h>


static const gchar *logfnmload = "fit.log.load.unc";
static const gchar *logfnmunload = "fit.log.unload.unc";

static const gchar *mclogfnmload = "fit.log.load.mc";
static const gchar *mclogfnmunload = "fit.log.unload.mc";

static gboolean nparam_to_string(gint i, gchar *str, gint nstr);

void init_shifted_Slopesdata(Slopesdata *src, Slopesdata *dest, gdouble dh)
{
	init_Slopesdata(dest);

	// shift fitting range  in h-mode by dh
	// don't shift range for F-mode
	
	dest->Finputload = src->Finputload;

	dest->loadrange[0]= src->loadrange[0]-dh;
	dest->loadrange[1]= src->loadrange[1]-dh;
	dest->loadfrom = dest->loadrange[0];
	dest->loadto = dest->loadrange[1];

	dest->loadrange_pct_Fmax[0] = src->loadrange_pct_Fmax[0];
	dest->loadrange_pct_Fmax[1] = src->loadrange_pct_Fmax[1];
	dest->loadfrom_pct_Fmax = src->loadfrom_pct_Fmax;
	dest->loadto_pct_Fmax = src->loadto_pct_Fmax;

	dest->loadifixb[0] = src->loadifixb[0];
	dest->loadifixb[1] = src->loadifixb[1];
	dest->loadifixb[2] = src->loadifixb[2];


	dest->Finputunload = src->Finputunload;
	
	dest->unloadrange[0]= src->unloadrange[0]-dh;
	dest->unloadrange[1]= src->unloadrange[1]-dh;
	dest->unloadfrom = dest->unloadrange[0];
	dest->unloadto = dest->unloadrange[1];

	dest->unloadrange_pct_Fmax[0] = src->unloadrange_pct_Fmax[0];
	dest->unloadrange_pct_Fmax[1] = src->unloadrange_pct_Fmax[1];
	dest->unloadfrom_pct_Fmax = src->unloadfrom_pct_Fmax;
	dest->unloadto_pct_Fmax = src->unloadto_pct_Fmax;

	g_free(dest->logfnmload);
	g_free(dest->logfnmunload);
	dest->logfnmload = create_log_filename_path(logfnmload);
	dest->logfnmunload = create_log_filename_path(logfnmunload);

	// copy beta
	dest->beta = src->beta;
}

void 
slopes_propagate_uncertainties_calc (Args *args)
{
	SlopesUncdata *unc;
	Slopesdata *sldata;
	FDdata *fd;
	Instdata *inst;
	gdouble uh, uF;
	gdouble mh, mF, epsh, epsF, zh, zF;
	gdouble umh, umF, uhph, uhpF, um, uhp;
	gdouble unh, unF, uh0h, uh0F, un, uh0;
	gdouble uSuh, uSuF, uSu;
	gdouble uSlh, uSlF ,uSl;
	gdouble z, depsdm, depsdmh, depsdmF;
	gdouble uepsh, uepsF;
	gdouble ueps;
	gdouble beta[3], std_beta[3], covariance[3];
	gint istart, iend,ndata;
	GwyDataLine *x, *y;
	gdouble ux,uy;
	gdouble n,Sl, h0, m, Su, hp, eps, hmax,Fmax, Aphc,Hit,Er,Eit, nu,nui,Ei;
	gdouble covnh0h, covnh0F, covnh0;
	gdouble covmhph, covmhpF, covmhp;
	gdouble K, uKuh, uKuF, uK;
	gdouble covKSuh, covKSuF, covKSu;
	gdouble covSumh, covSumF, covSum;
	gdouble slbeta; 
	gchar *unc_logfnmload, *unc_logfnmunload;
	gdouble SSres, R2, R2adj, chi2;

	unc = &(args->slopesunc);
	sldata = &(args->slopesdata);
	fd = &(args->fddata);
	inst = &(args->instdata);

	//make copies to simplify code
	uh = unc->uh;
	uF = unc->uF;
	n = sldata->n;
	h0 = sldata->h0;
	Sl = sldata->Sload;
	m = sldata->m;
	hp = sldata->hp;
	Su = sldata->Sunload;
	eps = sldata->eps;
	slbeta = sldata->beta; 
	hmax = fd->hmax;
	Fmax = fd->Fmax;
	Aphc = sldata->Aphc;
	Hit = sldata->Hit;
	Eit = sldata->Eit;
	Er = sldata->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;

	unc_logfnmload =  create_log_filename_path(logfnmload);
	unc_logfnmunload =  create_log_filename_path(logfnmunload);

	if (VERBOSE)
		printf(" \n\nPropagate uncertainties\n");


	/*   u(n)    */

	/* prepare odr fit */

	beta[0] = sldata->gamma;
	beta[1] = sldata->h0;
	beta[2] = sldata->n;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputload)
		range_to_indices(sldata->loadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->loadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->loadrange[0], sldata->loadrange[1], fd->hload, FALSE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Fload, istart, ndata);

	/* fit with only uh */
	ux = uh;
	uy = 0;
	beta[0] = sldata->gamma;
	beta[1] = sldata->h0;
	beta[2] = sldata->n;
	odr_run_fit(x, y, beta,sldata->loadifixb, unc_logfnmload, ux, uy, std_beta, covariance, &SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	unh = std_beta[2];
	uh0h = std_beta[1];
	covnh0h = covariance[0]; 

	if (VERBOSE)
		printf(" fit with only uh:\n  gamma %g n %g h0 %g \n  u(n) %g u(h0) %g corr(n,h0) %g \n",
			       beta[0], beta[1], beta[2], unh, uh0h, covnh0h/unh/uh0h);	       

	/* fit with only uF */
	ux = 0;
	uy = uF;
	odr_run_fit(x, y, beta,sldata->loadifixb, unc_logfnmload, ux, uy, std_beta, covariance, &SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	unF = std_beta[2];
	uh0F = std_beta[1];
	covnh0F = covariance[0]; 

	if (VERBOSE)
		printf(" fit with only uF:\n  gamma %g n %g h0 %g \n  u(n) %g u(h0) %g corr(n,h0) %g \n",
			       beta[0], beta[1], beta[2], unF, uh0F, covnh0F/unF/uh0F);	       

	unc->unuh = unh;
	unc->unuF = unF;



	/*   u(m)    */

	/* prepare odr fit */

	beta[0] = sldata->alpha;
	beta[1] = sldata->hp;
	beta[2] = sldata->m;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputunload)
		range_to_indices(sldata->unloadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->unloadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->unloadrange[0], sldata->unloadrange[1], fd->hunload, TRUE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	/* fit with only uh */
	ux = uh;
	uy = 0;
	beta[0] = sldata->alpha;
	beta[1] = sldata->hp;
	beta[2] = sldata->m;
	odr_run_fit(x, y, beta,sldata->unloadifixb, unc_logfnmunload, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	mh = beta[2];
	umh = std_beta[2];
	uhph = std_beta[1];
	covmhph = covariance[0]; 
	unc->umuh = umh;

	if (VERBOSE)
		printf(" fit with only uh:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
				beta[0], beta[1], beta[2], umh, uhph, covmhph/umh/uhph);	       

	/*   u(eps)    */   
	epsh = mh * (1 - 2*(mh-1) * tgamma(mh / 2 / (mh - 1)) / tgamma(1.0 / 2.0 / (mh - 1)) / sqrt(M_PI));
	zh = mh/(mh-1)/2;
	depsdmh = epsh/mh - mh/(mh -1) * (1 -epsh/mh ) -mh/2/(mh-1)/(mh-1)*(epsh/mh -1)* (psi(zh)-psi(zh-1./2));
	uepsh = depsdmh *umh;

	if (VERBOSE){
		printf(" uh fit \n");
		printf("depsdm %g  z %g  \n", depsdmh,zh);
		printf(" u(eps;h) %g urel(eps;h)  %g \n", uepsh, uepsh/eps);
	}


	/* fit with only uF */
	ux = 0;
	uy = uF;
	odr_run_fit(x, y, beta, sldata->unloadifixb, unc_logfnmunload, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	mF = beta[2];
	umF = std_beta[2];
	uhpF = std_beta[1];
	covmhpF = covariance[0]; 

	if (VERBOSE)
		printf(" fit with only uF:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umF, uhpF, covmhpF/umF/uhpF);	       

	unc->umuF = umF;

	
	/*   u(eps)    */   
	epsF = mF * (1 - 2*(mF-1) * tgamma(mF / 2 / (mF - 1)) / tgamma(1.0 / 2.0 / (mF - 1)) / sqrt(M_PI));
	zF = mF/(mF-1)/2;
	depsdmF = epsF/mF - mF/(mF -1) * (1 -epsF/mF ) -mF/2/(mF-1)/(mF-1)*(epsF/mF -1)* (psi(zF)-psi(zF-1./2));
	uepsF = depsdmF *umF;

	if (VERBOSE){
		printf(" uF fit \n");
		printf("depsdm %g  z %g  \n", depsdmF,zF);
		printf(" u(eps;F)  %g urel(eps;F) %g \n", uepsF, uepsF/eps);
	}



	/*    u(Sl)   */
	uSlh = sqrt(unh*unh/n/n +
			uh *uh/(hmax-h0)/(hmax-h0) +
			uh0h*uh0h/(hmax-h0)/(hmax-h0) +
			2*covnh0h/n/(hmax-h0)
		  ) * Sl; 

	uSlF = sqrt(unF*unF/n/n +    
		       	uF*uF/Fmax/Fmax  +    
			uh0F*uh0F/(hmax-h0)/(hmax-h0)+
			2*covnh0F/n/(hmax-h0)
		  ) * Sl; 
	unc->uSluh = uSlh;
	unc->uSluF = uSlF;
	/* u(Sl;h) = Sl *sqrt [ (unh/n)^2 + (uh/(hmax-h0))^2 + (uh0h/(hmax-h0))^2  + 2 cov(n,h0;h)/(n*(hmax-h0))]
	 * u(Sl;F) = Sl *sqrt [ (unF/n)^2 + (uFmax/Fmax)^2   + (uh0F/(hmax-h0))^2  + 2 cov(n,h0;F)/(n*(hmax-h0))]
	 */

	if (VERBOSE){
		printf(" u(Sl;h) %g u(Sl;F) %g  urel(Sl;h) %g urel(Sl;F) %g \n", uSlh, uSlF, uSlh/Sl, uSlF/Sl);
	}

	/*    u(Su)   */
	uSuh = sqrt(umh*umh/m/m +
			uh *uh/(hmax-hp)/(hmax-hp) +
			uhph*uhph/(hmax-hp)/(hmax-hp) +
			2*covmhph/m/(hmax-hp)
		  ) * Su; 

	uSuF = sqrt(umF*umF/m/m +    
		       	uF*uF/Fmax/Fmax  +    
			uhpF*uhpF/(hmax-hp)/(hmax-hp)+
			2*covmhpF/m/(hmax-hp)
		  ) * Su; 

	unc->uSuuh = uSuh;
	unc->uSuuF = uSuF;
	/* u(Su;h) = Su *sqrt [ (umh/m)^2 + (uh/(hmax-hp))^2 + (uhph/(hmax-hp))^2  + 2 cov(m,hp;h)/(m*(hmax-hp))]
	 * u(Su;F) = Su *sqrt [ (umF/m)^2 + (uFmax/Fmax)^2   + (uhpF/(hmax-hp))^2  + 2 cov(m,hp;F)/(m*(hmax-hp))]
	 */

	covSumh =  m*Fmax/(hmax-hp)/(hmax-hp)*covmhph + Fmax/(hmax-hp)*umh*umh;
	covSumF =  m*Fmax/(hmax-hp)/(hmax-hp)*covmhpF + Fmax/(hmax-hp)*umF*umF;

	/* cov(Su,m;h) = m Fmax/(hmax-hp)^2  cov(hp,m;h) + Fmax/(hmax-hp) u(m;h)^2
	 * cov(Su,m;F) = m Fmax/(hmax-hp)^2  cov(hp,m;F) + Fmax/(hmax-hp) u(m;F)^2
	 * */

	if (VERBOSE){
		printf(" u(Su;h) %g u(Su;F) %g  urel(Su;h) %g urel(Su;F) %g \n", uSuh, uSuF, uSuh/Su, uSuF/Su);
		printf(" cov(Su,m;h) %g cov(Su,m;F) %g \n", covSumh, covSumF);
	}

	/*    u(K)   */
	K= (2*Su - Sl*eps*slbeta)/Su/Sl;

	uKuh = sqrt( 4*uSlh*uSlh/Sl/Sl/Sl/Sl + eps *slbeta*eps*slbeta/Su/Su/Su/Su*uSuh*uSuh + slbeta*slbeta/Su/Su*uepsh*uepsh + slbeta*eps/Su/Su*depsdmh*covSumh);
	uKuF = sqrt( 4*uSlF*uSlF/Sl/Sl/Sl/Sl + eps *slbeta*eps*slbeta/Su/Su/Su/Su*uSuF*uSuF + slbeta*slbeta/Su/Su*uepsF*uepsF + slbeta*eps/Su/Su*depsdmF*covSumF);

	covKSuh = eps*slbeta/Su/Su *uSuh*uSuh - slbeta/Su *depsdmh* covSumh;
	covKSuF = eps*slbeta/Su/Su *uSuh*uSuF - slbeta/Su *depsdmF* covSumF;
	/*  K = 2 /Sl - eps*BETA/Su
	 *  u(K;h) = sqrt [ (-2/S1^2 u(S1;h))^2 + ( eps*BETA/Su^2 u(Su;h))^2  + ( BETA/Su u(eps;h))^2 + BETA eps/Su^2 depsdm(h) cov(S,m;h) ]
	 *  u(K;F) = sqrt [ (-2/S1^2 u(S1;F))^2 + ( eps*BETA/Su^2 u(Su;F))^2  + ( BETA/Su u(eps;F))^2 + BETA eps/Su^2 depsdm(F) cov(S,m;F) ] 
	 *  cov(K,Su; h) = eps*BETA/Su^2 u(Su;h)^2 - BETA/Su deps/dm cov(Su,m;h)
	 *  cov(K,Su; F) = eps*BETA/Su^2 u(Su;F)^2 - BETA/Su deps/dm cov(Su,m;F)   
	 *  */



	/*    u(A)   */
	unc->uAuh = 2 * Aphc * uKuh/K;
	unc->uAuF = 2 * Aphc * sqrt( uKuF*uKuF/K/K + uF*uF/Fmax/Fmax);
	/* u(A; h) =  2A *(uK;h)/K ]
	   u(A; F) =  2A * sqrt [ (uFmax/Fmax)^2 + ((uK;F)/K)^2 ] */
	if (VERBOSE)
		printf(" u(A;h) %g u(A;F) %g  urel(A;h) %g urel(A;F) %g \n", unc->uAuh, unc->uAuF, unc->uAuh/Aphc, unc->uAuF/Aphc);


	/*     u(H_IT)   */
	unc->uHituh = 2 * Hit * uKuh/K;
	unc->uHituF = Hit *sqrt ( uF*uF/Fmax/Fmax + 4*uKuF*uKuF/K/K );
	/* u(H; h ) = 2 H_IT u(K;h)/ K
	 * u(H; F)  = H_IT sqrt [ (uFmax/Fmax)^2 + (2 u(K;F)/K)^2 ] */
	if (VERBOSE)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);


	/*     u(E_r)   */
	unc ->uEruh = Er * sqrt( uKuh*uKuh/K/K + uSuh*uSuh/Su/Su - covKSuh/K/Su);
	unc ->uEruF = Er * sqrt( uF*uF/Fmax/Fmax+ uKuF*uKuF/K/K + uSuF*uSuF/Su/Su - covKSuF/K/Su);
	/* u(Er;h) = Er sqrt [  [u(K;h)/ K]^2 + [u(Su;h)/Su]^2 - cov(K,Su;h)/K/Su ] 
	 * u(Er;F) = Er sqrt [ (uFmax/Fmax)^2 +  [u(K;h)/ K]^2 + [u(Su;h)/Su]^2 - cov(K,Su;F)/K/Su] */


	if (VERBOSE)
		printf(" u(Er;h) %g u(Er;F) %g  urel(Er;h) %g urel(Er;F) %g \n", unc->uEruh, unc->uEruF, unc->uEruh/Er, unc->uEruF/Er);


	/*     u(E_IT)   */
	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; // Eit^2/Er^2 /(1-nu^2) u(Er;h)
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; // Eit^2/Er^2 /(1-nu^2) u(Er;F)


	if (VERBOSE){
		printf("u(Eit;h) %g  u(Eit;F) %g  urel(Eit;h) %g  urel(Eit;F) %g\n", unc->uEituh, unc->uEituF, unc->uEituh/Eit, unc->uEituF/Eit);
	}


	// total uncertainties
	// should take um as given by odr_wrap, otherwise doublecounting
	/* fit with both uh and uF */
	ux = uh;
	uy = uF;
	beta[0] = sldata->gamma;
	beta[1] = sldata->h0;
	beta[2] = sldata->n;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputload)
		range_to_indices(sldata->loadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->loadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->loadrange[0], sldata->loadrange[1], fd->hload, FALSE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Fload, istart, ndata);

	odr_run_fit(x, y, beta, sldata->loadifixb, unc_logfnmload, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	un = std_beta[2];
	uh0 = std_beta[1];
	covnh0 = covariance[0]; 

	unc->un = un;
	if (VERBOSE)
		printf(" fit with both uh and uF:\n  gamma %g h0 %g n %g \n  u(n) %g u(h0) %g corr(n,h0) %g \n",
	         		beta[0], beta[1], beta[2], un, uh0, covnh0/un/uh0);	       

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputunload)
		range_to_indices(sldata->unloadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->unloadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->unloadrange[0], sldata->unloadrange[1], fd->hunload, TRUE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	beta[0] = sldata->alpha;
	beta[1] = sldata->hp;
	beta[2] = sldata->m;
	odr_run_fit(x, y, beta, sldata->unloadifixb, unc_logfnmunload, ux, uy, std_beta, covariance,&SSres, &R2, &R2adj, &chi2, FALSE); /* do not rescale */
	um = std_beta[2];
	uhp = std_beta[1];
	covmhp = covariance[0]; 

	unc->um = um;

	if (VERBOSE)
		printf(" fit with both uh and uF:\n  alpha %g hp %g m %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], um, uhp, covmhp/um/uhp);	       
	/*   u(eps)    */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * ( 1 -eps/m ) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	ueps = depsdm *um;
	if (VERBOSE){
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps) %g    urel(eps) %g  \n", ueps, ueps/eps);
	}


	/*    u(Sl)   */
	uSl = sqrt(un*un/n/n  + uh *uh/(hmax-h0)/(hmax-h0) +  uF*uF/Fmax/Fmax +
			uh0*uh0/(hmax-h0)/(hmax-h0) + 2*covnh0/n/(hmax-h0)) * Sl; 

	unc->uSl = uSl;
	/* u(Sl) = Sl *sqrt [ (umh/m)^2 +  (uhmax/(hmax-hp))^2 + (uFmax/Fmax)^2 +  (uh0/(hmax-h0))^2 + 2 cov(m,h0)/(n*(hmax-h0)) ]
	 */

	if (VERBOSE){
		printf(" u(Sl) %g    urel(Sl) %g \n", uSl , uSl/Sl);
	}



	/*    u(Su)   */
	uSu = sqrt(um*um/m/m  + uh *uh/(hmax-hp)/(hmax-hp) +  uF*uF/Fmax/Fmax +
			uhp*uhp/(hmax-hp)/(hmax-hp) + 2*covmhp/m/(hmax-hp)) * Su; 
	unc->uSu = uSu;
	/* u(Su) = Su *sqrt [ (umh/m)^2 +  (uhmax/(hmax-hp))^2 + (uFmax/Fmax)^2 +  (uhp/(hmax-hp))^2 + 2 cov(m,hp)/(m*(hmax-hp)) ]
	 */

	covSum = m*Fmax/(hmax-hp)/(hmax-hp)*covmhp + Fmax/(hmax-hp)*um*um;

	/* cov(Su,m) = m Fmax/(hmax-hp)^2  cov(hp,m) + Fmax/(hmax-hp) u(m)^2
	 * */

	if (VERBOSE){
		printf(" u(Su) %g    urel(Su) %g   cov(Su,m) %g \n", uSu , uSu/Su, covSum);
	}
	
	/*    u(K)   */
	K= (2*Su - Sl*eps*slbeta)/Su/Sl;

	uK = sqrt( 4*uSl*uSl/Sl/Sl/Sl/Sl + eps *slbeta*eps*slbeta/Su/Su/Su/Su*uSu*uSu + slbeta*slbeta/Su/Su*ueps*ueps + slbeta*eps/Su/Su*depsdm*covSum);

	covKSu = eps*slbeta/Su/Su*uSu*uSu - slbeta/Su* depsdm*covSum;

	/*  K = 2 /Sl - eps*BETA/Su
	 *  u(K) = sqrt [ (-2/S1^2 u(S1))^2 + ( eps*BETA/Su^2 u(Su)^2  + ( BETA/Su u(eps))^2 + BETA eps/Su^2 depsdm cov(S,m)  ]
	 *  cov(K,Su) = eps*BETA/Su^2 u(Su)^2  -BETA/Su deps/dm cov(Su,m)  */

	if (VERBOSE) 
		printf(" u(K) %g cov(K,Su) %g    urel(K) %g  \n", uK, covKSu, uK/K);



	/*    u(A)   */
	unc->uA = 2 * Aphc * sqrt( uK*uK/K/K + uF*uF/Fmax/Fmax);
	if (VERBOSE)
		printf(" u(A) %g    urel(A) %g  \n", unc->uA, unc->uA/Aphc);


	/*     u(H_IT)   */
	unc->uHit = Hit *sqrt ( uF*uF/Fmax/Fmax + 4*uK*uK/K/K );
	if (VERBOSE)
		printf(" u(Hit)  %g   urel(Hit) %g \n", unc->uHit, unc->uHit/Hit);


	/*     u(E_r)   */
	unc ->uEr = Er * sqrt( uF*uF/Fmax/Fmax+ uK*uK/K/K + uSu*uSu/Su/Su - covKSu/K/Su);
	if (VERBOSE)
		printf(" u(Er) %g   urel(Er) %g  \n", unc->uEr, unc->uEr/Er);


	/*     u(E_IT)   */
	//contribution from h,F
	unc->uEit = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEr; // Eit^2/Er^2 /(1-nu^2) uEr

	//contribution from nu, nui, Ei
	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*inst->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *inst->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/Ei/Ei *inst->uEi)*1e9; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (VERBOSE){
		printf("u(Eit;h,F) %g    urel(Eit;hF) %g  \n", unc->uEit, unc->uEit/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}

	unc->uEit = sqrt( unc->uEit*unc->uEit  +unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
	if (VERBOSE)
		printf("u(Eit;hFnuetc) %g    urel(Eit; hF_nu_etc) %g \n", unc->uEit, unc->uEit/Eit);

	if (VERBOSE)
		printf(" \nEnd propagation of uncertainties\n\n");


	g_object_unref(x);
	g_object_unref(y);
	g_free(unc_logfnmload);
	g_free(unc_logfnmunload);
}


gboolean
slopes_uncertainty_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	Slopesdata *sldata;
	SlopesUncdata *slunc;
	Instdata *instdata;
	gint j;

	sldata = &(args->slopesdata);
	slunc = &(args->slopesunc);
	instdata = &(args->instdata);

	if (args->fddata.hload->res == 0 || args->fddata.Fload->res == 0 ||
	    args->fddata.hunload->res == 0 || args->fddata.Funload->res == 0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of two slopes analysis (ODR fit) \n");
	fprintf(file, "# u(depth): %g nm\n", slunc->uh);
	fprintf(file, "# u(load): %g nm\n", slunc->uF);
	fprintf(file, "#beta: %g \n", sldata->beta);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, instdata->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, instdata->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, instdata->uEi*1e-9); //TODO units
	fprintf(file, "#Fitted loading range: ");
	if (sldata->Finputload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->loadrange_pct_Fmax[0], sldata->loadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",sldata->loadrange[0], sldata->loadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (sldata->Finputload)
		fprintf(file, "[%g , %g ] nm \n",sldata->loadrange[0], sldata->loadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->loadrange_pct_Fmax[0], sldata->loadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", sldata->nfitloaddata);
	fprintf(file, "#Fitted unloading range: ");
	if (sldata->Finputunload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->unloadrange_pct_Fmax[0], sldata->unloadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",sldata->unloadrange[0], sldata->unloadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (sldata->Finputunload)
		fprintf(file, "[%g , %g ] nm \n",sldata->unloadrange[0], sldata->unloadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->unloadrange_pct_Fmax[0], sldata->unloadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", sldata->nfitunloaddata);

	fprintf(file, "#\n");

	fprintf(file, "#Results of two slopes analysis (ODR fit) \n");
	fprintf(file, "#Ap(hc): %g nm^2\n", sldata->Aphc);
	fprintf(file, "#Er: %g GPa\n", sldata->Er);
	fprintf(file, "#Hit: %g GPa\n", sldata->Hit);
	fprintf(file, "#Eit: %g GPa\n", sldata->Eit);
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(n) =  %g  \n", slunc->un);
	fprintf(file, "#u(Sload) =  %g mN/nm \n", slunc->uSl);
	fprintf(file, "#u(m) =  %g  \n", slunc->um);
	fprintf(file, "#u(Sunload) =  %g mN/nm \n", slunc->uSu);
	fprintf(file, "#u(Ap(hc)) =  %g nm^2 \n", slunc->uA);
	fprintf(file, "#u(Hit) =  %g MPa \n", slunc->uHit);
	fprintf(file, "#u(Er) =  %g GPa \n", slunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", slunc->uEit);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of n\n");
	fprintf(file, "#u(n;uh) = %g \n", slunc->unuh);
	fprintf(file, "#u(n;uF) = %g \n", slunc->unuF);

	fprintf(file, "#Contributions to uncertainty of Sload\n");
	fprintf(file, "#u(Sl;uh) = %g mN/nm \n", slunc->uSluh);
	fprintf(file, "#u(Sl;uF) = %g mN/nm \n", slunc->uSluF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of m\n");
	fprintf(file, "#u(m;uh) = %g \n", slunc->umuh);
	fprintf(file, "#u(m;uF) = %g \n", slunc->umuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Sunload\n");
	fprintf(file, "#u(Su;uh) = %g mN/nm \n", slunc->uSuuh);
	fprintf(file, "#u(Su;uF) = %g mN/nm \n", slunc->uSuuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Ap(hc)\n");
	fprintf(file, "#u(Ap(hc);uh) = %g nm^2 \n", slunc->uAuh);
	fprintf(file, "#u(Ap(hc);uF) = %g nm^2 \n", slunc->uAuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Hit\n");
	fprintf(file, "#u(Hit;uh) = %g MPa \n", slunc->uHituh);
	fprintf(file, "#u(Hit;uF) = %g MPa \n", slunc->uHituF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", slunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", slunc->uEruF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", slunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", slunc->uEituF);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", slunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", slunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", slunc->uEituEi);
	fprintf(file, "#\n");
	fprintf(file, "#\n");
	fprintf(file, "#\n");


	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( slunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, slunc->Ec[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");

	fprintf(file, "#Changes of the H corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#H/MPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( slunc->Hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, slunc->Hc[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");

	fprintf(file, "#Changes of the Ap corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#A/nm^2 \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( slunc->Ac[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, slunc->Ac[j+CONTACTMAX]);
	}

	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}

void
slopes_fit_shift_contact(Slopesdata *sldata, FDdata *fddata, Instdata *instdata, Area *area,gdouble *Ec,gdouble *Hc, gdouble *Ac,gint j)
{
	Slopesdata slopes;
	FDdata fdc;
	gdouble dh;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;

	Ec[j+CONTACTMAX] = -1;
	Hc[j+CONTACTMAX] = -1;
	Ac[j+CONTACTMAX] = -1;


	if (fddata->i_contact_load+j < 0)
		return;

	// create shifted FDdata
	dh = create_shifted_FDdata(fddata, &fdc, j);

	// create working copy of Slopesdata 
	init_shifted_Slopesdata(sldata,&slopes, dh);


	//check range
	if (sldata->Finputload){
		if (slopes.loadfrom_pct_Fmax ==  slopes.loadto_pct_Fmax)
			return;
	}
	else {
		if (slopes.loadfrom ==  slopes.loadto)
			return;
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputload)
		range_to_indices(slopes.loadfrom_pct_Fmax*0.01*fdc.Fmax, slopes.loadto_pct_Fmax*0.01*fdc.Fmax, fdc.Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(slopes.loadfrom, slopes.loadto, fdc.hload, FALSE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, no fitting can be performed
	ndata = MIN(ndata, fdc.hload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Fload, istart, ndata);

	// perform fit and get results
	slopes_fit_load(x,y,&slopes, &fdc, instdata, NULL);

	// clean up
	g_object_unref(x);
	g_object_unref(y);



	//check range
	if (sldata->Finputunload){
		if (slopes.unloadfrom_pct_Fmax ==  slopes.unloadto_pct_Fmax)
			return;
	}
	else {
		if (slopes.unloadfrom ==  slopes.unloadto)
			return;
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputunload)
		range_to_indices(slopes.unloadfrom_pct_Fmax*0.01*fdc.Fmax, slopes.unloadto_pct_Fmax*0.01*fdc.Fmax, fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(slopes.unloadfrom, slopes.unloadto, fdc.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, no fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);


	// perform fit and get results
	slopes_fit_unload(x,y,&slopes, &fdc, instdata, NULL);

	slopes_combine_fit_results(&slopes, &fdc, instdata);

	printf("j %d Sl %g Su %g Er %g H %g A %g \n", j, slopes.Sload, slopes.Sunload, slopes.Er, slopes.Hit, slopes.Aphc);
	printf(" logfnm load %s %s \n", slopes.logfnmload, slopes.logfnmunload);

	// clean up
	g_free(slopes.logfnmload);  //TODO
	g_free(slopes.logfnmunload);// TODO

	g_object_unref(x);
	g_object_unref(y);
	destroy_FDdata(&fdc);

	Ec[j+CONTACTMAX] = slopes.Er;
	Hc[j+CONTACTMAX] = slopes.Hit;
	Ac[j+CONTACTMAX] = slopes.Aphc;
}

void
slopes_uncertainty_montecarlo_run_calc( Args *args)
{
	gint istart, iend, ndata;
	gint i, j, Nmc;
	Slopesdata *slopes;
	SlopesMCdata *mc;
	GwyDataLine *hl, *Fl;
	GwyDataLine *hlnew, *Flnew;
	GwyDataLine *hlnoise, *Flnoise;
	GwyDataLine *hu, *Fu;
	GwyDataLine *hunew, *Funew;
	GwyDataLine *hunoise, *Funoise;
	gdouble hmax_old, Fmax_old;
	GRand *rng;
	gint nn;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;
	gdouble estimateu[3], estimatel[3];

	//not necessary to check range, couldn't get here in the first place

	//find appropriate loading data, which should be fitted, use same regime as for the main fit
	if (args->slopesdata.Finputload)
		range_to_indices(args->slopesdata.loadfrom_pct_Fmax*0.01*args->fddata.Fmax, 
				args->slopesdata.loadto_pct_Fmax*0.01*args->fddata.Fmax,
			       	args->fddata.Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(args->slopesdata.loadfrom, args->slopesdata.loadto, args->fddata.hload, FALSE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	hl = gwy_data_line_part_extract(args->fddata.hload, istart, ndata);
	Fl = gwy_data_line_part_extract(args->fddata.Fload, istart, ndata);

	//find appropriate unloadingdata, which should be fitted, use same regime as for the main fit
	if (args->slopesdata.Finputunload)
		range_to_indices(args->slopesdata.unloadfrom_pct_Fmax*0.01*args->fddata.Fmax, 
				args->slopesdata.unloadto_pct_Fmax*0.01*args->fddata.Fmax,
			       	args->fddata.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(args->slopesdata.unloadfrom, args->slopesdata.unloadto, args->fddata.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	hu = gwy_data_line_part_extract(args->fddata.hunload, istart, ndata);
	Fu = gwy_data_line_part_extract(args->fddata.Funload, istart, ndata);


	//back up hmax, Fmax
	hmax_old = args->fddata.hmax;
	Fmax_old = args->fddata.Fmax;

	//use results of previous fit as input estimates
	estimatel[0]= args->slopesdata.gamma;
	estimatel[1]= args->slopesdata.h0;
	estimatel[2]= args->slopesdata.n;
	estimateu[0]= args->slopesdata.alpha;
	estimateu[1]= args->slopesdata.hp;
	estimateu[2]= args->slopesdata.m;


	Nmc = args->slopesmc.Nmc;
	slopes = args->slopesmc.slopesmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		init_Slopesdata(slopes);
		//change log filenames
		g_free(slopes->logfnmload);
		g_free(slopes->logfnmunload);
		slopes->logfnmload = create_log_filename_path(mclogfnmload);
		slopes->logfnmunload = create_log_filename_path(mclogfnmunload);
		slopes++;
	}

	slopes = args->slopesmc.slopesmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
		/* opodr->nu = args->opodrdata.nu; */
		/* opodr->nui = args->opodrdata.nui; */
		/* opodr->Ei = args->opodrdata.Ei; */
		//		opodr->from = args->opodrdata.from;
		//		opodr->to = args->opodrdata.to;
		for (j = 0; j< 3; j++)
			slopes->loadifixb[j] = args->slopesdata.loadifixb[j];
		slopes++;
	}

	/*
	   for (i=0; i< Nmc ;i++){
	   printf ("nu %g \n", args->opodrmc.opodrmc[i].nu);
	   }
	   */

	// initialize random generator and datafield
	rng = g_rand_new();

	//prepare datafield for noise
	hlnoise=gwy_data_line_new_alike(hl,FALSE);
	Flnoise=gwy_data_line_new_alike(hl,FALSE);
	hunoise=gwy_data_line_new_alike(hu,FALSE);
	Funoise=gwy_data_line_new_alike(hu,FALSE);

	//prepare datafield for varied fields
	hlnew=gwy_data_line_new_alike(hl,FALSE);
	Flnew=gwy_data_line_new_alike(Fl,FALSE);
	hunew=gwy_data_line_new_alike(hu,FALSE);
	Funew=gwy_data_line_new_alike(Fu,FALSE);

	slopes = args->slopesmc.slopesmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hlnoise, args->slopesunc.uh, rng);
		generate_uncorrelated_noise(Flnoise, args->slopesunc.uF, rng);
		generate_uncorrelated_noise(hunoise, args->slopesunc.uh, rng);
		generate_uncorrelated_noise(Funoise, args->slopesunc.uF, rng);

		//add noise to original h-F data
		if (!sum_data_lines(hlnew, hl, hlnoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(Flnew, Fl, Flnoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(hunew, hu, hunoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(Funew, Fu, Funoise)){
			printf("Should not get here.\n");
			return;
		}

		//add random noise to hmax, Fmax
		args->fddata.hmax = hmax_old + gaussian_random_number(rng)*args->slopesunc.uh;
		args->fddata.Fmax = Fmax_old + gaussian_random_number(rng)*args->slopesunc.uF;

		//run fit

		slopes_fit_load(hlnew,Flnew,slopes, &(args->fddata), &(args->instdata),  estimatel);
		slopes_fit_unload(hunew,Funew,slopes, &(args->fddata), &(args->instdata),  estimateu);
		slopes_combine_fit_results(slopes, &(args->fddata), &(args->instdata));

		slopes++;
	}

	//reset hmax, Fmax to original value
	args->fddata.hmax = hmax_old;
	args->fddata.Fmax = Fmax_old;

//	printf("run Monte Carlo \n");	
	g_object_unref(hl);
	g_object_unref(Fl);
	g_object_unref(hlnew);
	g_object_unref(Flnew);
	g_object_unref(hlnoise);
	g_object_unref(Flnoise);
	g_object_unref(hu);
	g_object_unref(Fu);
	g_object_unref(hunew);
	g_object_unref(Funew);
	g_object_unref(hunoise);
	g_object_unref(Funoise);
	g_rand_free(rng);


		for (i=0; i< Nmc; i++){
//					printf(" %d Er %g Eit %g Hit %g \n", i, args->slopesmc.slopesmc[i].Er, args->slopesmc.slopesmc[i].Eit, args->slopesmc.slopesmc[i].Hit);
		}

		//calculate averages and stdevs
		//hp, Aphc, Hit, Er, Eit, S, m

		mc = &(args->slopesmc);
		for (i = 0; i< NPARAM; i++){
			mc->mcavg[i] =0;
			mc->mcstd[i] =0;
		}

		slopes = args->slopesmc.slopesmc;
		nn = 0;
		for (i = 0; i< Nmc; i++){
			//check sanity
			if (slopes->m >1 && slopes->n >1)
			{
				mc->mcavg[0]+=slopes->Sload;
				mc->mcavg[1]+=slopes->Sunload;
				mc->mcavg[2]+=slopes->Hit;
				mc->mcavg[3]+=slopes->Er;
				mc->mcavg[4]+=slopes->Eit;
				mc->mcavg[5]+=slopes->Aphc;
				mc->mcavg[6]+=slopes->m;
				mc->mcavg[7]+=slopes->n;
				mc->mcstd[0]+=slopes->Sload*slopes->Sload;
				mc->mcstd[1]+=slopes->Sunload*slopes->Sunload;
				mc->mcstd[2]+=slopes->Hit*slopes->Hit;
				mc->mcstd[3]+=slopes->Er*slopes->Er;
				mc->mcstd[4]+=slopes->Eit*slopes->Eit;
				mc->mcstd[5]+=slopes->Aphc*slopes->Aphc;
				mc->mcstd[6]+=slopes->m*slopes->m;
				mc->mcstd[7]+=slopes->n*slopes->n;
				nn++;
			}
			slopes++;
		}


		//use only reasonable data for average and standard deviation
		for (i = 0; i< NPARAM; i++){
			mc->mcavg[i] /= nn;
			mc->mcstd[i] /= nn;
			mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
			mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
			mc->mcstd[i] = sqrt(mc->mcstd[i]);
		}

		// how many datasets were skipped
		args->slopesmc.skipmc = Nmc-nn;

		/*
		   printf("mean  stdev \n");
		   for (i = 0; i< NPARAM; i++){
		   printf(" %g   %g \n", avg[i], std[i]);
		   }
		   */

		// Histograms
		//create data structure

		dd = args->slopesmc.mcdd;
		slopes = args->slopesmc.slopesmc;
		for (i = 0; i< Nmc; i++){
			dd[0][i] = slopes->Sload;
			dd[1][i] = slopes->Sunload;
			dd[2][i] = slopes->Hit;
			dd[3][i] = slopes->Er;
			dd[4][i] = slopes->Eit;
			dd[5][i] = slopes->Aphc;
			dd[6][i] = slopes->m;
			dd[7][i] = slopes->n;
			slopes++;
		}
		//prepare histogram structure


		for (i=0; i < NPARAM; i++){
			xhist = mc->mcxhist[i];
			yhist = mc->mcyhist[i];
			hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
			if (!hist){
				printf("Warning: No histogram could be created.\n");
				mc->mcnstat = -1;
				break;
			}
		}
}

gboolean
slopes_mc_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	Slopesdata *sldata;
	gint i,j;
	gchar str[200];
	gboolean is;

	if (args->fddata.hload->res == 0 || args->fddata.Fload->res == 0 ||
	    args->fddata.hunload->res == 0 || args->fddata.Funload->res == 0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of two slopes analysis (ODR fit)\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->slopesunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->slopesunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->slopesmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of two slopes analysis (ODR fit) \n");
	fprintf(file, "#beta: %g \n", args->slopesdata.beta);
	fprintf(file, "#nu: %g                 \n", args->instdata.nu);
	fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
	fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
	fprintf(file, "\n");
	fprintf(file, "#Fitted loadrange: ");
	if (args->slopesdata.Finputload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.loadrange_pct_Fmax[0], args->slopesdata.loadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.loadrange[0], args->slopesdata.loadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->slopesdata.Finputload)
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.loadrange[0], args->slopesdata.loadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.loadrange_pct_Fmax[0], args->slopesdata.loadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->slopesdata.nfitloaddata);
	fprintf(file, "\n");
fprintf(file, "#Fitted unloadrange: ");
	if (args->slopesdata.Finputunload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.unloadrange_pct_Fmax[0], args->slopesdata.unloadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.unloadrange[0], args->slopesdata.unloadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->slopesdata.Finputunload)
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.unloadrange[0], args->slopesdata.unloadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.unloadrange_pct_Fmax[0], args->slopesdata.unloadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->slopesdata.nfitunloaddata);
	fprintf(file, "\n");
	if (args->slopesmc.skipmc)
		fprintf(file, "#Fit failed in %d cases, the corresponding data area NOT included in the  estimates of mean and uncertainty but ARE included in the histograms.", args->slopesmc.skipmc );
	fprintf(file, "\n");
	if (args->slopesmc.skipmc)


	fprintf(file, "\n\n");


	fprintf(file, "#Results \n");
	for (i = 0; i < NPARAM; i++){
		is = nparam_to_string(i, str, sizeof(str));
		fprintf(file, "#%s = (%g ± %g ) \n", str, args->slopesmc.mcavg[i], args->slopesmc.mcstd[i]);
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->slopesmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		for (i=0; i < NPARAM; i++){
			fprintf(file,"#Histogram\n");
			is = nparam_to_string(i, str, sizeof(str));
			if (is){
				fprintf(file,"#%s    count/a.u.  \n", str);
				for (j = 0;j < args->slopesmc.mcnstat; j++)
					fprintf(file, "%g %g \n", args->slopesmc.mcxhist[i][j], args->slopesmc.mcyhist[i][j]);
				fprintf(file, "\n \n");
			}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	for ( j = 0; j< NPARAM; j++){
		is = nparam_to_string(j, str, sizeof(str));
		fprintf(file, "%s ", str);
	}
	fprintf(file,"\n");

	sldata = args->slopesmc.slopesmc;
	for (i = 0; i< args->slopesmc.Nmc; i++){
		fprintf(file, SLOPE, sldata->Sload);
		fprintf(file,"  ");
		fprintf(file, SLOPE, sldata->Sunload);
		fprintf(file,"  ");
		fprintf(file, HARD, sldata->Hit);
		fprintf(file,"  ");
		fprintf(file, MODUL, sldata->Er);
		fprintf(file,"  ");
		fprintf(file, MODUL, sldata->Eit);
		fprintf(file,"  ");
		fprintf(file, AREA, sldata->Aphc);
		fprintf(file,"  ");
		fprintf(file, NUMBER, sldata->m);
		fprintf(file,"  ");
		fprintf(file, NUMBER, sldata->n);
		fprintf(file,"  \n");
		sldata++;
	}


	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}

void
init_SlopesMCdata (SlopesMCdata *slopesmc, gint Nmc)
{
	gint i;

	//number of iterations
	slopesmc->Nmc = Nmc;
	printf("initialize for %d iterations \n", slopesmc->Nmc);

	//how much had to be skipped 
	slopesmc->skipmc = 0;

	//dynamically allocate
	slopesmc->slopesmc = (Slopesdata *)g_malloc(slopesmc->Nmc*sizeof(Slopesdata));
	slopesmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	slopesmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	slopesmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	slopesmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	slopesmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	slopesmc->mcnstat = (gint)floor(3.49*cbrt(slopesmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		slopesmc->mcdd[i] = (gdouble *) g_malloc(slopesmc->Nmc*sizeof(gdouble));
		slopesmc->mcxhist[i] = (gdouble *) g_malloc(slopesmc->mcnstat*sizeof(gdouble));
		slopesmc->mcyhist[i] = (gdouble *) g_malloc(slopesmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_SlopesMCdata (SlopesMCdata *slopesmc)
{
	gint i;

	//free everything
	g_free(slopesmc->mcavg);
	g_free(slopesmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(slopesmc->mcdd[i]);
		g_free(slopesmc->mcxhist[i]);
		g_free(slopesmc->mcyhist[i]);
	}

	g_free(slopesmc->mcdd);
	g_free(slopesmc->mcxhist);
	g_free(slopesmc->mcyhist);

	slopesmc->mcnstat = 0;
	slopesmc->skipmc = 0;

	for (i = 0; i < slopesmc->Nmc; i++){
		g_free(slopesmc->slopesmc[i].logfnmload);
		g_free(slopesmc->slopesmc[i].logfnmunload);
	}
	g_free(slopesmc->slopesmc);
}

static gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{
	switch (i){
		case 0:
			g_snprintf(str,nstr,"S_load/(mN/nm)");
			break;
		case 1:
			g_snprintf(str,nstr,"S_unload/(mN/nm)");
			break;
		case 2:
			g_snprintf(str,nstr,"H_IT/MPa");
			break;
		case 3:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 4:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		case 5:
			g_snprintf(str,nstr,"A_p(h_c)/nm^2");
			break;
		case 6:
			g_snprintf(str,nstr,"m");
			break;
		case 7:
			g_snprintf(str,nstr,"n");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}

#endif
